@@ -21,11 +21,16 @@
 #include "includes.h"
 #include "../auth/ntlmssp/ntlmssp.h"
 #include "../libcli/auth/libcli_auth.h"
-#include "../lib/crypto/md5.h"
-#include "../lib/crypto/hmacmd5.h"
-#include "../lib/crypto/crc32.h"
+#include "zlib.h"
 #include "../auth/ntlmssp/ntlmssp_private.h"
 
+#include "lib/crypto/gnutls_helpers.h"
+#include <gnutls/gnutls.h>
+#include <gnutls/crypto.h>
+
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_AUTH
+
 #define CLI_SIGN "session key to client-to-server signing key magic constant"
 #define CLI_SEAL "session key to client-to-server sealing key magic constant"
 #define SRV_SIGN "session key to server-to-client signing key magic constant"
@@ -47,15 +52,30 @@ static void dump_arc4_state(const char *description,
 	dump_data_pw(description, state->sbox, sizeof(state->sbox));
 }
 
-static void calc_ntlmv2_key(uint8_t subkey[16],
-			    DATA_BLOB session_key,
-			    const char *constant)
+static NTSTATUS calc_ntlmv2_key(uint8_t subkey[16],
+				DATA_BLOB session_key,
+				const char *constant)
 {
-	MD5_CTX ctx3;
-	MD5Init(&ctx3);
-	MD5Update(&ctx3, session_key.data, session_key.length);
-	MD5Update(&ctx3, (const uint8_t *)constant, strlen(constant)+1);
-	MD5Final(subkey, &ctx3);
+	gnutls_hash_hd_t hash_hnd = NULL;
+	int rc;
+
+	rc = gnutls_hash_init(&hash_hnd, GNUTLS_DIG_MD5);
+	if (rc < 0) {
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+	}
+	rc = gnutls_hash(hash_hnd, session_key.data, session_key.length);
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+	}
+	rc = gnutls_hash(hash_hnd, constant, strlen(constant) + 1);
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+	}
+	gnutls_hash_deinit(hash_hnd, subkey);
+
+	return NT_STATUS_OK;
 }
 
 enum ntlmssp_direction {
@@ -70,10 +90,13 @@ static NTSTATUS ntlmssp_make_packet_signature(struct ntlmssp_state *ntlmssp_stat
 					      enum ntlmssp_direction direction,
 					      DATA_BLOB *sig, bool encrypt_sig)
 {
+	NTSTATUS status;
+
 	if (ntlmssp_state->neg_flags & NTLMSSP_NEGOTIATE_NTLM2) {
-		HMACMD5Context ctx;
+		gnutls_hmac_hd_t hmac_hnd = NULL;
 		uint8_t digest[16];
 		uint8_t seq_num[4];
+		int rc;
 
 		*sig = data_blob_talloc(sig_mem_ctx, NULL, NTLMSSP_SIG_SIZE);
 		if (!sig->data) {
@@ -89,7 +112,14 @@ static NTSTATUS ntlmssp_make_packet_signature(struct ntlmssp_state *ntlmssp_stat
 
 			SIVAL(seq_num, 0, ntlmssp_state->crypt->ntlm2.sending.seq_num);
 			ntlmssp_state->crypt->ntlm2.sending.seq_num++;
-			hmac_md5_init_limK_to_64(ntlmssp_state->crypt->ntlm2.sending.sign_key, 16, &ctx);
+
+			rc = gnutls_hmac_init(&hmac_hnd,
+					      GNUTLS_MAC_MD5,
+					      ntlmssp_state->crypt->ntlm2.sending.sign_key,
+					      16);
+			if (rc < 0) {
+				return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+			}
 			break;
 		case NTLMSSP_RECEIVE:
 
@@ -100,15 +130,30 @@ static NTSTATUS ntlmssp_make_packet_signature(struct ntlmssp_state *ntlmssp_stat
 
 			SIVAL(seq_num, 0, ntlmssp_state->crypt->ntlm2.receiving.seq_num);
 			ntlmssp_state->crypt->ntlm2.receiving.seq_num++;
-			hmac_md5_init_limK_to_64(ntlmssp_state->crypt->ntlm2.receiving.sign_key, 16, &ctx);
+
+			rc = gnutls_hmac_init(&hmac_hnd,
+					      GNUTLS_MAC_MD5,
+					      ntlmssp_state->crypt->ntlm2.receiving.sign_key,
+					      16);
+			if (rc < 0) {
+				return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+			}
 			break;
 		}
 
 		dump_data_pw("pdu data ", whole_pdu, pdu_length);
 
-		hmac_md5_update(seq_num, sizeof(seq_num), &ctx);
-		hmac_md5_update(whole_pdu, pdu_length, &ctx);
-		hmac_md5_final(digest, &ctx);
+		rc = gnutls_hmac(hmac_hnd, seq_num, sizeof(seq_num));
+		if (rc < 0) {
+			gnutls_hmac_deinit(hmac_hnd, NULL);
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+		}
+		rc = gnutls_hmac(hmac_hnd, whole_pdu, pdu_length);
+		if (rc < 0) {
+			gnutls_hmac_deinit(hmac_hnd, NULL);
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_NTLM_BLOCKED);
+		}
+		gnutls_hmac_deinit(hmac_hnd, digest);
 
 		if (encrypt_sig && (ntlmssp_state->neg_flags & NTLMSSP_NEGOTIATE_KEY_EXCH)) {
 			switch (direction) {
@@ -125,15 +170,17 @@ static NTSTATUS ntlmssp_make_packet_signature(struct ntlmssp_state *ntlmssp_stat
 
 		SIVAL(sig->data, 0, NTLMSSP_SIGN_VERSION);
 		memcpy(sig->data + 4, digest, 8);
+		ZERO_ARRAY(digest);
 		memcpy(sig->data + 12, seq_num, 4);
+		ZERO_ARRAY(seq_num);
 
 		dump_data_pw("ntlmssp v2 sig ", sig->data, sig->length);
 
 	} else {
-		NTSTATUS status;
 		uint32_t crc;
 
-		crc = crc32_calc_buffer(data, length);
+		crc = crc32(0, Z_NULL, 0);
+		crc = crc32(crc, data, length);
 
 		status = msrpc_gen(sig_mem_ctx,
 			       sig, "dddd",
@@ -150,6 +197,7 @@ static NTSTATUS ntlmssp_make_packet_signature(struct ntlmssp_state *ntlmssp_stat
 		arcfour_crypt_sbox(&ntlmssp_state->crypt->ntlm.seal_state,
 				   sig->data+4, sig->length-4);
 	}
+
 	return NT_STATUS_OK;
 }
 
@@ -315,7 +363,8 @@ NTSTATUS ntlmssp_seal_packet(struct ntlmssp_state *ntlmssp_state,
 		NTSTATUS status;
 		uint32_t crc;
 
-		crc = crc32_calc_buffer(data, length);
+		crc = crc32(0, Z_NULL, 0);
+		crc = crc32(crc, data, length);
 
 		status = msrpc_gen(sig_mem_ctx,
 			       sig, "dddd",
@@ -541,6 +590,7 @@ NTSTATUS ntlmssp_sign_reset(struct ntlmssp_state *ntlmssp_state,
 		uint8_t recv_seal_key[16];
 		DATA_BLOB recv_seal_blob = data_blob_const(recv_seal_key,
 					   sizeof(recv_seal_key));
+		NTSTATUS status;
 
 		switch (ntlmssp_state->role) {
 		case NTLMSSP_CLIENT:
@@ -583,14 +633,21 @@ NTSTATUS ntlmssp_sign_reset(struct ntlmssp_state *ntlmssp_state,
 			     weak_session_key.length);
 
 		/* SEND: sign key */
-		calc_ntlmv2_key(ntlmssp_state->crypt->ntlm2.sending.sign_key,
-				ntlmssp_state->session_key, send_sign_const);
+		status = calc_ntlmv2_key(ntlmssp_state->crypt->ntlm2.sending.sign_key,
+					 ntlmssp_state->session_key, send_sign_const);
+		if (!NT_STATUS_IS_OK(status)) {
+			return status;
+		}
 		dump_data_pw("NTLMSSP send sign key:\n",
 			     ntlmssp_state->crypt->ntlm2.sending.sign_key, 16);
 
 		/* SEND: seal ARCFOUR pad */
-		calc_ntlmv2_key(send_seal_key,
-				weak_session_key, send_seal_const);
+		status = calc_ntlmv2_key(send_seal_key,
+					 weak_session_key,
+					 send_seal_const);
+		if (!NT_STATUS_IS_OK(status)) {
+			return status;
+		}
 		dump_data_pw("NTLMSSP send seal key:\n", send_seal_key, 16);
 
 		arcfour_init(&ntlmssp_state->crypt->ntlm2.sending.seal_state,
@@ -605,14 +662,21 @@ NTSTATUS ntlmssp_sign_reset(struct ntlmssp_state *ntlmssp_state,
 		}
 
 		/* RECV: sign key */
-		calc_ntlmv2_key(ntlmssp_state->crypt->ntlm2.receiving.sign_key,
-				ntlmssp_state->session_key, recv_sign_const);
+		status = calc_ntlmv2_key(ntlmssp_state->crypt->ntlm2.receiving.sign_key,
+					 ntlmssp_state->session_key, recv_sign_const);
+		if (!NT_STATUS_IS_OK(status)) {
+			return status;
+		}
 		dump_data_pw("NTLMSSP recv sign key:\n",
 			     ntlmssp_state->crypt->ntlm2.receiving.sign_key, 16);
 
 		/* RECV: seal ARCFOUR pad */
-		calc_ntlmv2_key(recv_seal_key,
-				weak_session_key, recv_seal_const);
+		status = calc_ntlmv2_key(recv_seal_key,
+					 weak_session_key,
+					 recv_seal_const);
+		if (!NT_STATUS_IS_OK(status)) {
+			return status;
+		}
 		dump_data_pw("NTLMSSP recv seal key:\n", recv_seal_key, 16);
 
 		arcfour_init(&ntlmssp_state->crypt->ntlm2.receiving.seal_state,
@@ -685,7 +749,7 @@ NTSTATUS ntlmssp_sign_reset(struct ntlmssp_state *ntlmssp_state,
 NTSTATUS ntlmssp_sign_init(struct ntlmssp_state *ntlmssp_state)
 {
 	if (ntlmssp_state->session_key.length < 8) {
-		DEBUG(3, ("NO session key, cannot intialise signing\n"));
+		DEBUG(3, ("NO session key, cannot initialise signing\n"));
 		return NT_STATUS_NO_USER_SESSION_KEY;
 	}
 
