@@ -4,7 +4,8 @@
 
 define_test "CTDB_NATGW_PUBLIC_IFACE unset, not slave-only"
 
-setup_ctdb
+setup
+
 setup_ctdb_natgw <<EOF
 192.168.1.21 master
 192.168.1.22
@@ -12,7 +13,9 @@ setup_ctdb_natgw <<EOF
 192.168.1.24
 EOF
 
+setup_script_options <<EOF
 CTDB_NATGW_PUBLIC_IFACE=""
+EOF
 
 required_result 1 "Invalid configuration: CTDB_NATGW_PUBLIC_IFACE not set"
 
