@@ -13,7 +13,7 @@ EOF
 
 . "${TEST_SCRIPTS_DIR}/integration.bash"
 
-ctdb_test_init "$@"
+ctdb_test_init
 
 set -e
 
@@ -27,10 +27,9 @@ get_debug ()
     local node="$1"
 
     local out
-    
+
     try_command_on_node -v $node "$CTDB getdebug"
-    check_debug=$(echo "$out" |
-	sed -r -e 's@Node [[:digit:]]+ is at debug level ([[:alpha:]]+) \(-?[[:digit:]]+\)$@\1@')
+    check_debug="$out"
 }
 
 set_and_check_debug ()
@@ -46,8 +45,7 @@ set_and_check_debug ()
     get_debug $node
 
     if [ "$levelstr" != "$check_debug" ] ; then
-	echo "BAD: Debug level should have changed to \"$levelstr\" but it is \"$check_debug\"."
-	testfailures=1
+	die "BAD: Debug level \"$levelstr\" != \"$check_debug\"."
     fi
 }
 
@@ -63,17 +61,21 @@ for new_debug in $levels ; do
     set_and_check_debug $test_node "$new_debug"
 done
 
-i=0
-for new_debug in $levels ; do
+while read new_debug i ; do
     [ "$initial_debug" != "$i" ] || continue
 
     echo
     set_and_check_debug $test_node "$i" "$new_debug"
-    i=$[ $i + 1 ]
-done
-
-if [ "$testfailures" != 1 ] ; then
-    echo
-    echo "Returning the debug level to its initial value..."
-    set_and_check_debug $test_node "$initial_debug"
-fi
+done <<EOF
+ERROR 0
+WARNING 1
+WARNING 2
+NOTICE 3
+NOTICE 4
+INFO 5
+INFO 6
+INFO 7
+INFO 8
+INFO 9
+DEBUG 10
+EOF
