@@ -150,10 +150,17 @@ static void cli_ll_create(fuse_req_t freq, fuse_ino_t parent, const char *name,
 	}
 
 	req = cli_smb2_create_fnum_send(
-		state, mstate->ev, mstate->cli, state->path,
-		0, FILE_GENERIC_READ|FILE_GENERIC_WRITE, FILE_ATTRIBUTE_NORMAL,
+		state,
+		mstate->ev,
+		mstate->cli, state->path,
+		0,
+		SMB2_IMPERSONATION_IMPERSONATION,
+		FILE_GENERIC_READ|FILE_GENERIC_WRITE,
+		FILE_ATTRIBUTE_NORMAL,
 		FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
-		FILE_CREATE, FILE_NON_DIRECTORY_FILE);
+		FILE_CREATE,
+		FILE_NON_DIRECTORY_FILE,
+		NULL);
 	if (req == NULL) {
 		TALLOC_FREE(state);
 		fuse_reply_err(freq, ENOMEM);
@@ -171,7 +178,7 @@ static void cli_ll_create_done(struct tevent_req *req)
 	uint16_t fnum;
 	NTSTATUS status;
 
-	status = cli_smb2_create_fnum_recv(req, &fnum, NULL);
+	status = cli_smb2_create_fnum_recv(req, &fnum, NULL, NULL, NULL);
 	TALLOC_FREE(req);
 	if (!NT_STATUS_IS_OK(status)) {
 		fuse_reply_err(state->freq, map_errno_from_nt_status(status));
@@ -484,6 +491,7 @@ static NTSTATUS parse_finfo_id_both_directory_info(uint8_t *dir_data,
 		return NT_STATUS_INFO_LENGTH_MISMATCH;
 	}
 
+	finfo->btime_ts = interpret_long_date((const char *)dir_data + 8);
 	finfo->atime_ts = interpret_long_date((const char *)dir_data + 16);
 	finfo->mtime_ts = interpret_long_date((const char *)dir_data + 24);
 	finfo->ctime_ts = interpret_long_date((const char *)dir_data + 32);
@@ -717,6 +725,72 @@ static void cli_ll_lookup_done(struct tevent_req *req)
 	TALLOC_FREE(state);
 }
 
+struct ll_getattr_state {
+	struct mount_state *mstate;
+	fuse_req_t freq;
+	struct fuse_file_info fi;
+};
+
+static void cli_ll_getattr_done(struct tevent_req *req);
+
+static void cli_ll_getattr(fuse_req_t freq, fuse_ino_t ino,
+			   struct fuse_file_info *fi)
+{
+	struct mount_state *mstate = talloc_get_type_abort(
+		fuse_req_userdata(freq), struct mount_state);
+	struct ll_getattr_state *state;
+	struct inode_state *istate;
+	struct tevent_req *req;
+
+	DBG_DEBUG("ino=%ju\n", (uintmax_t)ino);
+
+	istate = idr_find(mstate->ino_ctx, ino);
+	if (istate == NULL) {
+		fuse_reply_err(freq, ENOENT);
+		return;
+	}
+
+	state = talloc(mstate, struct ll_getattr_state);
+	if (state == NULL) {
+		fuse_reply_err(freq, ENOMEM);
+		return;
+	}
+	state->mstate = mstate;
+	state->freq = freq;
+
+	req = cli_get_unixattr_send(state, mstate->ev, mstate->cli,
+				    istate->path);
+	if (req == NULL) {
+		TALLOC_FREE(state);
+		fuse_reply_err(freq, ENOMEM);
+		return;
+	}
+	tevent_req_set_callback(req, cli_ll_getattr_done, state);
+}
+
+static void cli_ll_getattr_done(struct tevent_req *req)
+{
+	struct ll_getattr_state *state = tevent_req_callback_data(
+		req, struct ll_getattr_state);
+	struct stat st;
+	NTSTATUS status;
+	int ret;
+
+	status = cli_get_unixattr_recv(req, &st);
+	TALLOC_FREE(req);
+	if (!NT_STATUS_IS_OK(status)) {
+		fuse_reply_err(state->freq, map_errno_from_nt_status(status));
+		return;
+	}
+
+	ret = fuse_reply_attr(state->freq, &st, 1);
+	if (ret != 0) {
+		DBG_NOTICE("fuse_reply_attr failed: %s\n",
+			   strerror(-errno));
+	}
+}
+
+
 struct ll_open_state {
 	struct mount_state *mstate;
 	fuse_req_t freq;
@@ -768,10 +842,18 @@ static void cli_ll_open(fuse_req_t freq, fuse_ino_t ino,
 	}
 
 	req = cli_smb2_create_fnum_send(
-		state, mstate->ev, mstate->cli, istate->path,
-		0, acc, FILE_ATTRIBUTE_NORMAL,
+		state,
+		mstate->ev,
+		mstate->cli,
+		istate->path,
+		0,
+		SMB2_IMPERSONATION_IMPERSONATION,
+		acc,
+		FILE_ATTRIBUTE_NORMAL,
 		FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
-		FILE_OPEN, FILE_NON_DIRECTORY_FILE);
+		FILE_OPEN,
+		FILE_NON_DIRECTORY_FILE,
+		NULL);
 	if (req == NULL) {
 		TALLOC_FREE(state);
 		fuse_reply_err(freq, ENOMEM);
@@ -787,7 +869,7 @@ static void cli_ll_open_done(struct tevent_req *req)
 	uint16_t fnum;
 	NTSTATUS status;
 
-	status = cli_smb2_create_fnum_recv(req, &fnum, NULL);
+	status = cli_smb2_create_fnum_recv(req, &fnum, NULL, NULL, NULL);
 	TALLOC_FREE(req);
 	if (!NT_STATUS_IS_OK(status)) {
 		fuse_reply_err(state->freq, map_errno_from_nt_status(status));
@@ -1302,6 +1384,7 @@ static void cli_ll_releasedir_done(struct tevent_req *req)
 
 static struct fuse_lowlevel_ops cli_ll_ops = {
 	.lookup = cli_ll_lookup,
+	.getattr = cli_ll_getattr,
 	.open = cli_ll_open,
 	.create = cli_ll_create,
 	.release = cli_ll_release,
