@@ -1,4 +1,4 @@
-/* 
+/*
    ldb database library
 
    Copyright (C) Andrew Tridgell  2004
@@ -6,7 +6,7 @@
      ** NOTE! The following LGPL license applies to the ldb
      ** library. This does NOT imply that all of Samba is released
      ** under the LGPL
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
@@ -44,7 +44,7 @@ struct ldb_message *ldb_msg_new(TALLOC_CTX *mem_ctx)
 /*
   find an element in a message by attribute name
 */
-struct ldb_message_element *ldb_msg_find_element(const struct ldb_message *msg, 
+struct ldb_message_element *ldb_msg_find_element(const struct ldb_message *msg,
 						 const char *attr_name)
 {
 	unsigned int i;
@@ -77,7 +77,7 @@ int ldb_val_equal_exact(const struct ldb_val *v1, const struct ldb_val *v2)
   find a value in an element
   assumes case sensitive comparison
 */
-struct ldb_val *ldb_msg_find_val(const struct ldb_message_element *el, 
+struct ldb_val *ldb_msg_find_val(const struct ldb_message_element *el,
 				 struct ldb_val *val)
 {
 	unsigned int i;
@@ -89,6 +89,219 @@ struct ldb_val *ldb_msg_find_val(const struct ldb_message_element *el,
 	return NULL;
 }
 
+
+static int ldb_val_cmp(const struct ldb_val *v1, const struct ldb_val *v2)
+{
+	if (v1->length != v2->length) {
+		return v1->length - v2->length;
+	}
+	return memcmp(v1->data, v2->data, v1->length);
+}
+
+
+/*
+  ldb_msg_find_duplicate_val() will set the **duplicate pointer to the first
+  duplicate value it finds. It does a case sensitive comparison (memcmp).
+
+  LDB_ERR_OPERATIONS_ERROR indicates an allocation failure or an unknown
+  options flag, otherwise LDB_SUCCESS.
+*/
+#define LDB_DUP_QUADRATIC_THRESHOLD 10
+
+int ldb_msg_find_duplicate_val(struct ldb_context *ldb,
+			       TALLOC_CTX *mem_ctx,
+			       const struct ldb_message_element *el,
+			       struct ldb_val **duplicate,
+			       uint32_t options)
+{
+	unsigned int i, j;
+	struct ldb_val *val;
+
+	if (options != 0) {
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+
+	*duplicate = NULL;
+
+	/*
+	   If there are not many values, it is best to avoid the talloc
+	   overhead and just do a brute force search.
+	 */
+	if (el->num_values < LDB_DUP_QUADRATIC_THRESHOLD) {
+		for (j = 0; j < el->num_values; j++) {
+			val = &el->values[j];
+			for ( i = j + 1; i < el->num_values; i++) {
+				if (ldb_val_equal_exact(val, &el->values[i])) {
+					*duplicate = val;
+					return LDB_SUCCESS;
+				}
+			}
+		}
+	} else {
+		struct ldb_val *values;
+		values = talloc_array(mem_ctx, struct ldb_val, el->num_values);
+		if (values == NULL) {
+			return LDB_ERR_OPERATIONS_ERROR;
+		}
+
+		memcpy(values, el->values,
+		       el->num_values * sizeof(struct ldb_val));
+		TYPESAFE_QSORT(values, el->num_values, ldb_val_cmp);
+		for (i = 1; i < el->num_values; i++) {
+			if (ldb_val_equal_exact(&values[i],
+						&values[i - 1])) {
+				/* find the original location */
+				for (j = 0; j < el->num_values; j++) {
+					if (ldb_val_equal_exact(&values[i],
+								&el->values[j])
+						) {
+						*duplicate = &el->values[j];
+						break;
+					}
+				}
+				talloc_free(values);
+				if (*duplicate == NULL) {
+					/* how we got here, I don't know */
+					return LDB_ERR_OPERATIONS_ERROR;
+				}
+				return LDB_SUCCESS;
+			}
+		}
+		talloc_free(values);
+	}
+	return LDB_SUCCESS;
+}
+
+
+/*
+  Determine whether the values in an element are also in another element.
+
+  Without any flags, return LDB_ERR_ATTRIBUTE_OR_VALUE_EXISTS if the elements
+  share values, or LDB_SUCCESS if they don't. In this case, the function
+  simply determines the set intersection and it doesn't matter in which order
+  the elements are provided.
+
+  With the LDB_MSG_FIND_COMMON_REMOVE_DUPLICATES flag, any values in common are
+  removed from the first element and LDB_SUCCESS is returned.
+
+  LDB_ERR_OPERATIONS_ERROR indicates an allocation failure or an unknown option.
+  LDB_ERR_INAPPROPRIATE_MATCHING is returned if the elements differ in name.
+*/
+
+int ldb_msg_find_common_values(struct ldb_context *ldb,
+			       TALLOC_CTX *mem_ctx,
+			       struct ldb_message_element *el,
+			       struct ldb_message_element *el2,
+			       uint32_t options)
+{
+	struct ldb_val *values;
+	struct ldb_val *values2;
+	unsigned int i, j, k, n_values;
+
+	bool remove_duplicates = options & LDB_MSG_FIND_COMMON_REMOVE_DUPLICATES;
+
+	if ((options & ~LDB_MSG_FIND_COMMON_REMOVE_DUPLICATES) != 0) {
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+
+	if (strcmp(el->name, el2->name) != 0) {
+		return LDB_ERR_INAPPROPRIATE_MATCHING;
+	}
+	if (el->num_values == 0 || el2->num_values == 0) {
+		return LDB_SUCCESS;
+	}
+	/*
+	   With few values, it is better to do the brute-force search than the
+	   clever search involving tallocs, memcpys, sorts, etc.
+	*/
+	if (MIN(el->num_values, el2->num_values) == 1 ||
+	    MAX(el->num_values, el2->num_values) < LDB_DUP_QUADRATIC_THRESHOLD) {
+		for (i = 0; i < el2->num_values; i++) {
+			for (j = 0; j < el->num_values; j++) {
+				if (ldb_val_equal_exact(&el->values[j],
+							&el2->values[i])) {
+					if (! remove_duplicates) {
+					    return			\
+					      LDB_ERR_ATTRIBUTE_OR_VALUE_EXISTS;
+					}
+					/*
+					  With the remove_duplicates flag, we
+					  resolve the intersection by removing
+					  the offending one from el.
+					*/
+					el->num_values--;
+					for (k = j; k < el->num_values; k++) {
+						el->values[k] = \
+							el->values[k + 1];
+					}
+					j--; /* rewind */
+				}
+			}
+		}
+		return LDB_SUCCESS;
+	}
+
+	values = talloc_array(mem_ctx, struct ldb_val, el->num_values);
+	if (values == NULL) {
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+	values2 = talloc_array(mem_ctx, struct ldb_val,
+				    el2->num_values);
+	if (values2 == NULL) {
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+
+	memcpy(values, el->values,
+	       el->num_values * sizeof(struct ldb_val));
+	memcpy(values2, el2->values,
+	       el2->num_values * sizeof(struct ldb_val));
+	TYPESAFE_QSORT(values, el->num_values, ldb_val_cmp);
+	TYPESAFE_QSORT(values2, el2->num_values, ldb_val_cmp);
+
+	/*
+	   el->n_values may diverge from the number of values in the sorted
+	   list when the remove_duplicates flag is used.
+	*/
+	n_values = el->num_values;
+	i = 0;
+	j = 0;
+	while (i != n_values && j < el2->num_values) {
+		int ret = ldb_val_cmp(&values[i], &values2[j]);
+		if (ret < 0) {
+			i++;
+		} else if (ret > 0) {
+			j++;
+		} else {
+			/* we have a collision */
+			if (! remove_duplicates) {
+				TALLOC_FREE(values);
+				TALLOC_FREE(values2);
+				return LDB_ERR_ATTRIBUTE_OR_VALUE_EXISTS;
+			}
+			/*
+			   With the remove_duplicates flag we need to find
+			   this in the original list and remove it, which is
+			   inefficient but hopefully rare.
+			*/
+			for (k = 0; k < el->num_values; k++) {
+				if (ldb_val_equal_exact(&el->values[k],
+							&values[i])) {
+					break;
+				}
+			}
+			el->num_values--;
+			for (; k < el->num_values; k++) {
+				el->values[k] = el->values[k + 1];
+			}
+			i++;
+		}
+	}
+	TALLOC_FREE(values);
+	TALLOC_FREE(values2);
+
+	return LDB_SUCCESS;
+}
+
 /*
   duplicate a ldb_val structure
 */
@@ -181,8 +394,8 @@ int ldb_msg_add_empty(struct ldb_message *msg,
  *       is invalidated for some reason, this will
  *       corrupt *msg contents also
  */
-int ldb_msg_add(struct ldb_message *msg, 
-		const struct ldb_message_element *el, 
+int ldb_msg_add(struct ldb_message *msg,
+		const struct ldb_message_element *el,
 		int flags)
 {
 	int ret;
@@ -207,7 +420,7 @@ int ldb_msg_add(struct ldb_message *msg,
 /*
   add a value to a message
 */
-int ldb_msg_add_value(struct ldb_message *msg, 
+int ldb_msg_add_value(struct ldb_message *msg,
 		      const char *attr_name,
 		      const struct ldb_val *val,
 		      struct ldb_message_element **return_el)
@@ -244,7 +457,7 @@ int ldb_msg_add_value(struct ldb_message *msg,
 /*
   add a value to a message, stealing it into the 'right' place
 */
-int ldb_msg_add_steal_value(struct ldb_message *msg, 
+int ldb_msg_add_steal_value(struct ldb_message *msg,
 			    const char *attr_name,
 			    struct ldb_val *val)
 {
@@ -262,7 +475,7 @@ int ldb_msg_add_steal_value(struct ldb_message *msg,
 /*
   add a string element to a message
 */
-int ldb_msg_add_string(struct ldb_message *msg, 
+int ldb_msg_add_string(struct ldb_message *msg,
 		       const char *attr_name, const char *str)
 {
 	struct ldb_val val;
@@ -281,7 +494,7 @@ int ldb_msg_add_string(struct ldb_message *msg,
 /*
   add a string element to a message, stealing it into the 'right' place
 */
-int ldb_msg_add_steal_string(struct ldb_message *msg, 
+int ldb_msg_add_steal_string(struct ldb_message *msg,
 			     const char *attr_name, char *str)
 {
 	struct ldb_val val;
@@ -318,7 +531,7 @@ int ldb_msg_add_linearized_dn(struct ldb_message *msg, const char *attr_name,
 /*
   add a printf formatted element to a message
 */
-int ldb_msg_add_fmt(struct ldb_message *msg, 
+int ldb_msg_add_fmt(struct ldb_message *msg,
 		    const char *attr_name, const char *fmt, ...)
 {
 	struct ldb_val val;
@@ -341,7 +554,7 @@ int ldb_msg_add_fmt(struct ldb_message *msg,
   compare two ldb_message_element structures
   assumes case sensitive comparison
 */
-int ldb_msg_element_compare(struct ldb_message_element *el1, 
+int ldb_msg_element_compare(struct ldb_message_element *el1,
 			    struct ldb_message_element *el2)
 {
 	unsigned int i;
@@ -383,7 +596,7 @@ bool ldb_msg_element_equal_ordered(const struct ldb_message_element *el1,
   compare two ldb_message_element structures
   comparing by element name
 */
-int ldb_msg_element_compare_name(struct ldb_message_element *el1, 
+int ldb_msg_element_compare_name(struct ldb_message_element *el1,
 				 struct ldb_message_element *el2)
 {
 	return ldb_attr_cmp(el1->name, el2->name);
@@ -393,7 +606,7 @@ int ldb_msg_element_compare_name(struct ldb_message_element *el1,
   convenience functions to return common types from a message
   these return the first value if the attribute is multi-valued
 */
-const struct ldb_val *ldb_msg_find_ldb_val(const struct ldb_message *msg, 
+const struct ldb_val *ldb_msg_find_ldb_val(const struct ldb_message *msg,
 					   const char *attr_name)
 {
 	struct ldb_message_element *el = ldb_msg_find_element(msg, attr_name);
@@ -403,7 +616,7 @@ const struct ldb_val *ldb_msg_find_ldb_val(const struct ldb_message *msg,
 	return &el->values[0];
 }
 
-int ldb_msg_find_attr_as_int(const struct ldb_message *msg, 
+int ldb_msg_find_attr_as_int(const struct ldb_message *msg,
 			     const char *attr_name,
 			     int default_value)
 {
@@ -467,7 +680,7 @@ unsigned int ldb_msg_find_attr_as_uint(const struct ldb_message *msg,
 	return ret;
 }
 
-int64_t ldb_msg_find_attr_as_int64(const struct ldb_message *msg, 
+int64_t ldb_msg_find_attr_as_int64(const struct ldb_message *msg,
 				   const char *attr_name,
 				   int64_t default_value)
 {
@@ -531,7 +744,7 @@ uint64_t ldb_msg_find_attr_as_uint64(const struct ldb_message *msg,
 	return ret;
 }
 
-double ldb_msg_find_attr_as_double(const struct ldb_message *msg, 
+double ldb_msg_find_attr_as_double(const struct ldb_message *msg,
 				   const char *attr_name,
 				   double default_value)
 {
@@ -560,7 +773,7 @@ double ldb_msg_find_attr_as_double(const struct ldb_message *msg,
 	return ret;
 }
 
-int ldb_msg_find_attr_as_bool(const struct ldb_message *msg, 
+int ldb_msg_find_attr_as_bool(const struct ldb_message *msg,
 			      const char *attr_name,
 			      int default_value)
 {
@@ -577,7 +790,7 @@ int ldb_msg_find_attr_as_bool(const struct ldb_message *msg,
 	return default_value;
 }
 
-const char *ldb_msg_find_attr_as_string(const struct ldb_message *msg, 
+const char *ldb_msg_find_attr_as_string(const struct ldb_message *msg,
 					const char *attr_name,
 					const char *default_value)
 {
@@ -624,7 +837,7 @@ void ldb_msg_sort_elements(struct ldb_message *msg)
   shallow copy a message - copying only the elements array so that the caller
   can safely add new elements without changing the message
 */
-struct ldb_message *ldb_msg_copy_shallow(TALLOC_CTX *mem_ctx, 
+struct ldb_message *ldb_msg_copy_shallow(TALLOC_CTX *mem_ctx,
 					 const struct ldb_message *msg)
 {
 	struct ldb_message *msg2;
@@ -635,7 +848,7 @@ struct ldb_message *ldb_msg_copy_shallow(TALLOC_CTX *mem_ctx,
 
 	*msg2 = *msg;
 
-	msg2->elements = talloc_array(msg2, struct ldb_message_element, 
+	msg2->elements = talloc_array(msg2, struct ldb_message_element,
 				      msg2->num_elements);
 	if (msg2->elements == NULL) goto failed;
 
@@ -654,7 +867,7 @@ failed:
 /*
   copy a message, allocating new memory for all parts
 */
-struct ldb_message *ldb_msg_copy(TALLOC_CTX *mem_ctx, 
+struct ldb_message *ldb_msg_copy(TALLOC_CTX *mem_ctx,
 				 const struct ldb_message *msg)
 {
 	struct ldb_message *msg2;
@@ -692,7 +905,7 @@ failed:
 /**
  * Canonicalize a message, merging elements of the same name
  */
-struct ldb_message *ldb_msg_canonicalize(struct ldb_context *ldb, 
+struct ldb_message *ldb_msg_canonicalize(struct ldb_context *ldb,
 					 const struct ldb_message *msg)
 {
 	int ret;
@@ -764,7 +977,7 @@ int ldb_msg_normalize(struct ldb_context *ldb,
  * If you then use this in a ldb_modify() call,
  * it can be used to save edits to a message
  */
-struct ldb_message *ldb_msg_diff(struct ldb_context *ldb, 
+struct ldb_message *ldb_msg_diff(struct ldb_context *ldb,
 				 struct ldb_message *msg1,
 				 struct ldb_message *msg2)
 {
@@ -870,7 +1083,7 @@ failed:
 }
 
 
-int ldb_msg_sanity_check(struct ldb_context *ldb, 
+int ldb_msg_sanity_check(struct ldb_context *ldb,
 			 const struct ldb_message *msg)
 {
 	unsigned int i, j;
@@ -887,7 +1100,7 @@ int ldb_msg_sanity_check(struct ldb_context *ldb,
 			if (msg->elements[i].values[j].length == 0) {
 				/* an attribute cannot be empty */
 				ldb_asprintf_errstring(ldb, "Element %s has empty attribute in ldb message (%s)!",
-							    msg->elements[i].name, 
+							    msg->elements[i].name,
 							    ldb_dn_get_linearized(msg->dn));
 				return LDB_ERR_INVALID_ATTRIBUTE_SYNTAX;
 			}
@@ -1009,14 +1222,14 @@ int ldb_msg_copy_attr(struct ldb_message *msg, const char *attr, const char *rep
 void ldb_msg_remove_element(struct ldb_message *msg, struct ldb_message_element *el)
 {
 	ptrdiff_t n = (el - msg->elements);
-	if (n >= msg->num_elements) {
-		/* should we abort() here? */
+	if (n >= msg->num_elements || n < 0) {
+		/* the element is not in the list. the caller is crazy. */
 		return;
 	}
-	if (n != msg->num_elements-1) {
-		memmove(el, el+1, ((msg->num_elements-1) - n)*sizeof(*el));
-	}
 	msg->num_elements--;
+	if (n != msg->num_elements) {
+		memmove(el, el+1, (msg->num_elements - n)*sizeof(*el));
+	}
 }
 
 
@@ -1069,18 +1282,18 @@ char *ldb_timestring(TALLOC_CTX *mem_ctx, time_t t)
 time_t ldb_string_to_time(const char *s)
 {
 	struct tm tm;
-	
+
 	if (s == NULL) return 0;
-	
+
 	memset(&tm, 0, sizeof(tm));
 	if (sscanf(s, "%04u%02u%02u%02u%02u%02u.0Z",
-		   &tm.tm_year, &tm.tm_mon, &tm.tm_mday, 
+		   &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
 		   &tm.tm_hour, &tm.tm_min, &tm.tm_sec) != 6) {
 		return 0;
 	}
 	tm.tm_year -= 1900;
 	tm.tm_mon -= 1;
-	
+
 	return timegm(&tm);
 }
 
@@ -1090,8 +1303,10 @@ time_t ldb_string_to_time(const char *s)
 */
 int ldb_val_to_time(const struct ldb_val *v, time_t *t)
 {
-	char val[15] = {};
-	struct tm tm = {};
+	char val[15] = {0};
+	struct tm tm = {
+		.tm_year = 0,
+	};
 
 	if (v == NULL) {
 		return LDB_ERR_INVALID_ATTRIBUTE_SYNTAX;
@@ -1182,12 +1397,12 @@ char *ldb_timestring_utc(TALLOC_CTX *mem_ctx, time_t t)
 time_t ldb_string_utc_to_time(const char *s)
 {
 	struct tm tm;
-	
+
 	if (s == NULL) return 0;
-	
+
 	memset(&tm, 0, sizeof(tm));
 	if (sscanf(s, "%02u%02u%02u%02u%02u%02uZ",
-		   &tm.tm_year, &tm.tm_mon, &tm.tm_mday, 
+		   &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
 		   &tm.tm_hour, &tm.tm_min, &tm.tm_sec) != 6) {
 		return 0;
 	}
@@ -1195,7 +1410,7 @@ time_t ldb_string_utc_to_time(const char *s)
 		tm.tm_year += 100;
 	}
 	tm.tm_mon -= 1;
-	
+
 	return timegm(&tm);
 }
 
@@ -1224,7 +1439,7 @@ int ldb_msg_check_string_attribute(const struct ldb_message *msg,
 {
 	struct ldb_message_element *el;
 	struct ldb_val val;
-	
+
 	el = ldb_msg_find_element(msg, name);
 	if (el == NULL) {
 		return 0;
