@@ -101,6 +101,8 @@ struct torture_context
 
 	/** Loadparm context (will go away in favor of torture_setting_ at some point) */
 	struct loadparm_context *lp_ctx;
+
+	int conn_index;
 };
 
 struct torture_results
@@ -293,6 +295,15 @@ void torture_result(struct torture_context *test,
 	}\
 	} while(0)
 
+#define torture_assert_ndr_err_equal_goto(torture_ctx,got,expected,ret,label,cmt) \
+	do { enum ndr_err_code __got = got, __expected = expected; \
+	if (__got != __expected) { \
+		torture_result(torture_ctx, TORTURE_FAIL, __location__": "#got" was %d (%s), expected %d (%s): %s", __got, ndr_errstr(__got), __expected, __STRING(expected), cmt); \
+		ret = false; \
+		goto label; \
+	}\
+	} while(0)
+
 #define torture_assert_hresult_equal(torture_ctx, got, expected, cmt) \
 	do { HRESULT __got = got, __expected = expected; \
 	if (!HRES_IS_EQUAL(__got, __expected)) { \
@@ -312,7 +323,9 @@ void torture_result(struct torture_context *test,
 #define torture_assert_casestr_equal(torture_ctx,got,expected,cmt) \
 	do { const char *__got = (got), *__expected = (expected); \
 	if (!strequal(__got, __expected)) { \
-		torture_result(torture_ctx, TORTURE_FAIL, __location__": "#got" was %s, expected %s: %s", __got, __expected, cmt); \
+		torture_result(torture_ctx, TORTURE_FAIL, \
+			       __location__": "#got" was %s, expected %s: %s", \
+			       __got, __expected == NULL ? "null" : __expected, cmt); \
 		return false; \
 	} \
 	} while(0)
@@ -321,8 +334,8 @@ void torture_result(struct torture_context *test,
 	do { const char *__got = (got), *__expected = (expected); \
 	if (strcmp_safe(__got, __expected) != 0) { \
 		torture_result(torture_ctx, TORTURE_FAIL, \
-					   __location__": "#got" was %s, expected %s: %s", \
-					   __got, __expected, cmt); \
+			       __location__": "#got" was %s, expected %s: %s", \
+			       __got, __expected == NULL ? "NULL" : __expected, cmt); \
 		return false; \
 	} \
 	} while(0)
@@ -395,7 +408,7 @@ static inline void torture_dump_data_str_cb(const char *buf, void *private_data)
 		char *__dump = NULL; \
 		uint8_t __byte_a = 0x00;\
 		uint8_t __byte_b = 0x00;\
-		int __i;\
+		size_t __i;\
 		for (__i=0; __i < __expected.length; __i++) {\
 			__byte_a = __expected.data[__i];\
 			if (__i == __got.length) {\
@@ -407,14 +420,14 @@ static inline void torture_dump_data_str_cb(const char *buf, void *private_data)
 				break;\
 			}\
 		}\
-		torture_warning(torture_ctx, "blobs differ at byte 0x%02X (%u)", __i, __i);\
+		torture_warning(torture_ctx, "blobs differ at byte 0x%02X (%zu)", (unsigned int)__i, __i);\
 		torture_warning(torture_ctx, "expected byte[0x%02X] = 0x%02X got byte[0x%02X] = 0x%02X",\
-				__i, __byte_a, __i, __byte_b);\
+				(unsigned int)__i, __byte_a, (unsigned int)__i, __byte_b);\
 		__dump = talloc_strdup(torture_ctx, ""); \
 		dump_data_cb(__got.data, __got.length, true, \
 			     torture_dump_data_str_cb, &__dump); \
 		torture_warning(torture_ctx, "got[0x%02X]: \n%s", \
-				(int)__got.length, __dump); \
+				(unsigned int)__got.length, __dump); \
 		TALLOC_FREE(__dump); \
 		__dump = talloc_strdup(torture_ctx, ""); \
 		dump_data_cb(__expected.data, __expected.length, true, \
@@ -647,6 +660,9 @@ static inline void torture_dump_data_str_cb(const char *buf, void *private_data)
 #define torture_assert_ndr_success(torture_ctx,expr,cmt) \
 		torture_assert_ndr_err_equal(torture_ctx,expr,NDR_ERR_SUCCESS,cmt)
 
+#define torture_assert_ndr_success_goto(torture_ctx,expr,ret,label,cmt) \
+		torture_assert_ndr_err_equal_goto(torture_ctx,expr,NDR_ERR_SUCCESS,ret,label,cmt)
+
 #define torture_assert_hresult_ok(torture_ctx,expr,cmt) \
 		torture_assert_hresult_equal(torture_ctx,expr,HRES_ERROR(0), cmt)
 
