@@ -26,6 +26,10 @@
 #include "libcli/auth/libcli_auth.h"
 #include "../libcli/security/dom_sid.h"
 
+#include "lib/crypto/gnutls_helpers.h"
+#include <gnutls/gnutls.h>
+#include <gnutls/crypto.h>
+
 static void netlogon_creds_step_crypt(struct netlogon_creds_CredentialState *creds,
 				      const struct netr_Credential *in,
 				      struct netr_Credential *out)
@@ -71,27 +75,55 @@ static void netlogon_creds_init_64bit(struct netlogon_creds_CredentialState *cre
 
   this call is made after the netr_ServerReqChallenge call
 */
-static void netlogon_creds_init_128bit(struct netlogon_creds_CredentialState *creds,
+static NTSTATUS netlogon_creds_init_128bit(struct netlogon_creds_CredentialState *creds,
 				       const struct netr_Credential *client_challenge,
 				       const struct netr_Credential *server_challenge,
 				       const struct samr_Password *machine_password)
 {
-	unsigned char zero[4], tmp[16];
-	HMACMD5Context ctx;
-	MD5_CTX md5;
+	uint8_t zero[4] = {0};
+	uint8_t tmp[gnutls_hash_get_len(GNUTLS_MAC_MD5)];
+	gnutls_hash_hd_t hash_hnd = NULL;
+	int rc;
 
 	ZERO_ARRAY(creds->session_key);
 
-	memset(zero, 0, sizeof(zero));
+	rc = gnutls_hash_init(&hash_hnd, GNUTLS_DIG_MD5);
+	if (rc < 0) {
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
+	}
 
-	hmac_md5_init_rfc2104(machine_password->hash, sizeof(machine_password->hash), &ctx);
-	MD5Init(&md5);
-	MD5Update(&md5, zero, sizeof(zero));
-	MD5Update(&md5, client_challenge->data, 8);
-	MD5Update(&md5, server_challenge->data, 8);
-	MD5Final(tmp, &md5);
-	hmac_md5_update(tmp, sizeof(tmp), &ctx);
-	hmac_md5_final(creds->session_key, &ctx);
+	rc = gnutls_hash(hash_hnd, zero, sizeof(zero));
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
+	}
+	rc = gnutls_hash(hash_hnd, client_challenge->data, 8);
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
+	}
+	rc = gnutls_hash(hash_hnd, server_challenge->data, 8);
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
+	}
+
+	gnutls_hash_deinit(hash_hnd, tmp);
+
+	/* This doesn't require HMAC MD5 RFC2104 as the hash is only 16 bytes */
+	rc = gnutls_hmac_fast(GNUTLS_MAC_MD5,
+			      machine_password->hash,
+			      sizeof(machine_password->hash),
+			      tmp,
+			      sizeof(tmp),
+			      creds->session_key);
+	ZERO_ARRAY(tmp);
+
+	if (rc < 0) {
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
+	}
+
+	return NT_STATUS_OK;
 }
 
 /*
@@ -99,27 +131,45 @@ static void netlogon_creds_init_128bit(struct netlogon_creds_CredentialState *cr
 
   this call is made after the netr_ServerReqChallenge call
 */
-static void netlogon_creds_init_hmac_sha256(struct netlogon_creds_CredentialState *creds,
-					    const struct netr_Credential *client_challenge,
-					    const struct netr_Credential *server_challenge,
-					    const struct samr_Password *machine_password)
+static NTSTATUS netlogon_creds_init_hmac_sha256(struct netlogon_creds_CredentialState *creds,
+						const struct netr_Credential *client_challenge,
+						const struct netr_Credential *server_challenge,
+						const struct samr_Password *machine_password)
 {
-	struct HMACSHA256Context ctx;
-	uint8_t digest[SHA256_DIGEST_LENGTH];
+	gnutls_hmac_hd_t hmac_hnd = NULL;
+	uint8_t digest[gnutls_hash_get_len(GNUTLS_MAC_SHA256)];
+	int rc;
 
 	ZERO_ARRAY(creds->session_key);
 
-	hmac_sha256_init(machine_password->hash,
-			 sizeof(machine_password->hash),
-			 &ctx);
-	hmac_sha256_update(client_challenge->data, 8, &ctx);
-	hmac_sha256_update(server_challenge->data, 8, &ctx);
-	hmac_sha256_final(digest, &ctx);
+	rc = gnutls_hmac_init(&hmac_hnd,
+			      GNUTLS_MAC_SHA256,
+			      machine_password->hash,
+			      sizeof(machine_password->hash));
+	if (rc < 0) {
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+	}
+	rc = gnutls_hmac(hmac_hnd,
+			 client_challenge->data,
+			 8);
+	if (rc < 0) {
+		gnutls_hmac_deinit(hmac_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+	}
+	rc  = gnutls_hmac(hmac_hnd,
+			  server_challenge->data,
+			  8);
+	if (rc < 0) {
+		gnutls_hmac_deinit(hmac_hnd, NULL);
+		return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+	}
+	gnutls_hmac_deinit(hmac_hnd, digest);
 
 	memcpy(creds->session_key, digest, sizeof(creds->session_key));
 
-	ZERO_STRUCT(digest);
-	ZERO_STRUCT(ctx);
+	ZERO_ARRAY(digest);
+
+	return NT_STATUS_OK;
 }
 
 static void netlogon_creds_first_step(struct netlogon_creds_CredentialState *creds,
@@ -212,13 +262,35 @@ void netlogon_creds_des_decrypt(struct netlogon_creds_CredentialState *creds, st
 /*
   ARCFOUR encrypt/decrypt a password buffer using the session key
 */
-void netlogon_creds_arcfour_crypt(struct netlogon_creds_CredentialState *creds, uint8_t *data, size_t len)
+NTSTATUS netlogon_creds_arcfour_crypt(struct netlogon_creds_CredentialState *creds,
+				      uint8_t *data,
+				      size_t len)
 {
-	DATA_BLOB session_key = data_blob(creds->session_key, 16);
+	gnutls_cipher_hd_t cipher_hnd = NULL;
+	gnutls_datum_t session_key = {
+		.data = creds->session_key,
+		.size = sizeof(creds->session_key),
+	};
+	int rc;
 
-	arcfour_crypt_blob(data, len, &session_key);
+	rc = gnutls_cipher_init(&cipher_hnd,
+				GNUTLS_CIPHER_ARCFOUR_128,
+				&session_key,
+				NULL);
+	if (rc < 0) {
+		return gnutls_error_to_ntstatus(rc,
+						NT_STATUS_CRYPTO_SYSTEM_INVALID);
+	}
+	rc = gnutls_cipher_encrypt(cipher_hnd,
+				   data,
+				   len);
+	gnutls_cipher_deinit(cipher_hnd);
+	if (rc < 0) {
+		return gnutls_error_to_ntstatus(rc,
+						NT_STATUS_CRYPTO_SYSTEM_INVALID);
+	}
 
-	data_blob_free(&session_key);
+	return NT_STATUS_OK;
 }
 
 /*
@@ -268,6 +340,7 @@ struct netlogon_creds_CredentialState *netlogon_creds_client_init(TALLOC_CTX *me
 								  uint32_t negotiate_flags)
 {
 	struct netlogon_creds_CredentialState *creds = talloc_zero(mem_ctx, struct netlogon_creds_CredentialState);
+	NTSTATUS status;
 
 	if (!creds) {
 		return NULL;
@@ -293,12 +366,23 @@ struct netlogon_creds_CredentialState *netlogon_creds_client_init(TALLOC_CTX *me
 	dump_data_pw("Machine Pass", machine_password->hash, sizeof(machine_password->hash));
 
 	if (negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
-		netlogon_creds_init_hmac_sha256(creds,
-						client_challenge,
-						server_challenge,
-						machine_password);
+		status = netlogon_creds_init_hmac_sha256(creds,
+							 client_challenge,
+							 server_challenge,
+							 machine_password);
+		if (!NT_STATUS_IS_OK(status)) {
+			talloc_free(creds);
+			return NULL;
+		}
 	} else if (negotiate_flags & NETLOGON_NEG_STRONG_KEYS) {
-		netlogon_creds_init_128bit(creds, client_challenge, server_challenge, machine_password);
+		status = netlogon_creds_init_128bit(creds,
+						    client_challenge,
+						    server_challenge,
+						    machine_password);
+		if (!NT_STATUS_IS_OK(status)) {
+			talloc_free(creds);
+			return NULL;
+		}
 	} else {
 		netlogon_creds_init_64bit(creds, client_challenge, server_challenge, machine_password);
 	}
@@ -446,10 +530,16 @@ struct netlogon_creds_CredentialState *netlogon_creds_server_init(TALLOC_CTX *me
 	}
 
 	if (negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
-		netlogon_creds_init_hmac_sha256(creds,
-						client_challenge,
-						server_challenge,
-						machine_password);
+		NTSTATUS status;
+
+		status = netlogon_creds_init_hmac_sha256(creds,
+							 client_challenge,
+							 server_challenge,
+							 machine_password);
+		if (!NT_STATUS_IS_OK(status)) {
+			talloc_free(creds);
+			return NULL;
+		}
 	} else if (negotiate_flags & NETLOGON_NEG_STRONG_KEYS) {
 		netlogon_creds_init_128bit(creds, client_challenge, server_challenge,
 					   machine_password);
@@ -504,15 +594,16 @@ NTSTATUS netlogon_creds_server_step_check(struct netlogon_creds_CredentialState
 	}
 }
 
-static void netlogon_creds_crypt_samlogon_validation(struct netlogon_creds_CredentialState *creds,
-						     uint16_t validation_level,
-						     union netr_Validation *validation,
-						     bool do_encrypt)
+static NTSTATUS netlogon_creds_crypt_samlogon_validation(struct netlogon_creds_CredentialState *creds,
+							 uint16_t validation_level,
+							 union netr_Validation *validation,
+							 bool do_encrypt)
 {
 	struct netr_SamBaseInfo *base = NULL;
+	NTSTATUS status;
 
 	if (validation == NULL) {
-		return;
+		return NT_STATUS_INVALID_PARAMETER;
 	}
 
 	switch (validation_level) {
@@ -533,11 +624,11 @@ static void netlogon_creds_crypt_samlogon_validation(struct netlogon_creds_Crede
 		break;
 	default:
 		/* If we can't find it, we can't very well decrypt it */
-		return;
+		return NT_STATUS_INVALID_INFO_CLASS;
 	}
 
 	if (!base) {
-		return;
+		return NT_STATUS_INVALID_INFO_CLASS;
 	}
 
 	/* find and decyrpt the session keys, return in parameters above */
@@ -573,16 +664,22 @@ static void netlogon_creds_crypt_samlogon_validation(struct netlogon_creds_Crede
 	} else if (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR) {
 		/* Don't crypt an all-zero key, it would give away the NETLOGON pipe session key */
 		if (!all_zero(base->key.key, sizeof(base->key.key))) {
-			netlogon_creds_arcfour_crypt(creds,
-					    base->key.key,
-					    sizeof(base->key.key));
+			status = netlogon_creds_arcfour_crypt(creds,
+							      base->key.key,
+							      sizeof(base->key.key));
+			if (!NT_STATUS_IS_OK(status)) {
+				return status;
+			}
 		}
 
 		if (!all_zero(base->LMSessKey.key,
 			      sizeof(base->LMSessKey.key))) {
-			netlogon_creds_arcfour_crypt(creds,
-					    base->LMSessKey.key,
-					    sizeof(base->LMSessKey.key));
+			status = netlogon_creds_arcfour_crypt(creds,
+							      base->LMSessKey.key,
+							      sizeof(base->LMSessKey.key));
+			if (!NT_STATUS_IS_OK(status)) {
+				return status;
+			}
 		}
 	} else {
 		/* Don't crypt an all-zero key, it would give away the NETLOGON pipe session key */
@@ -597,31 +694,39 @@ static void netlogon_creds_crypt_samlogon_validation(struct netlogon_creds_Crede
 			}
 		}
 	}
+
+	return NT_STATUS_OK;
 }
 
-void netlogon_creds_decrypt_samlogon_validation(struct netlogon_creds_CredentialState *creds,
-						uint16_t validation_level,
-						union netr_Validation *validation)
+NTSTATUS netlogon_creds_decrypt_samlogon_validation(struct netlogon_creds_CredentialState *creds,
+						    uint16_t validation_level,
+						    union netr_Validation *validation)
 {
-	netlogon_creds_crypt_samlogon_validation(creds, validation_level,
-							validation, false);
+	return netlogon_creds_crypt_samlogon_validation(creds,
+							validation_level,
+							validation,
+							false);
 }
 
-void netlogon_creds_encrypt_samlogon_validation(struct netlogon_creds_CredentialState *creds,
-						uint16_t validation_level,
-						union netr_Validation *validation)
+NTSTATUS netlogon_creds_encrypt_samlogon_validation(struct netlogon_creds_CredentialState *creds,
+						    uint16_t validation_level,
+						    union netr_Validation *validation)
 {
-	netlogon_creds_crypt_samlogon_validation(creds, validation_level,
-							validation, true);
+	return netlogon_creds_crypt_samlogon_validation(creds,
+							validation_level,
+							validation,
+							true);
 }
 
-static void netlogon_creds_crypt_samlogon_logon(struct netlogon_creds_CredentialState *creds,
-						enum netr_LogonInfoClass level,
-						union netr_LogonLevel *logon,
-						bool do_encrypt)
+static NTSTATUS netlogon_creds_crypt_samlogon_logon(struct netlogon_creds_CredentialState *creds,
+						    enum netr_LogonInfoClass level,
+						    union netr_LogonLevel *logon,
+						    bool do_encrypt)
 {
+	NTSTATUS status;
+
 	if (logon == NULL) {
-		return;
+		return NT_STATUS_INVALID_PARAMETER;
 	}
 
 	switch (level) {
@@ -630,7 +735,7 @@ static void netlogon_creds_crypt_samlogon_logon(struct netlogon_creds_Credential
 	case NetlogonServiceInformation:
 	case NetlogonServiceTransitiveInformation:
 		if (logon->password == NULL) {
-			return;
+			return NT_STATUS_INVALID_PARAMETER;
 		}
 
 		if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
@@ -658,12 +763,22 @@ static void netlogon_creds_crypt_samlogon_logon(struct netlogon_creds_Credential
 
 			h = logon->password->lmpassword.hash;
 			if (!all_zero(h, 16)) {
-				netlogon_creds_arcfour_crypt(creds, h, 16);
+				status = netlogon_creds_arcfour_crypt(creds,
+								      h,
+								      16);
+				if (!NT_STATUS_IS_OK(status)) {
+					return status;
+				}
 			}
 
 			h = logon->password->ntpassword.hash;
 			if (!all_zero(h, 16)) {
-				netlogon_creds_arcfour_crypt(creds, h, 16);
+				status = netlogon_creds_arcfour_crypt(creds,
+								      h,
+								      16);
+				if (!NT_STATUS_IS_OK(status)) {
+					return status;
+				}
 			}
 		} else {
 			struct samr_Password *p;
@@ -693,7 +808,7 @@ static void netlogon_creds_crypt_samlogon_logon(struct netlogon_creds_Credential
 
 	case NetlogonGenericInformation:
 		if (logon->generic == NULL) {
-			return;
+			return NT_STATUS_INVALID_PARAMETER;
 		}
 
 		if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
@@ -707,28 +822,33 @@ static void netlogon_creds_crypt_samlogon_logon(struct netlogon_creds_Credential
 						logon->generic->length);
 			}
 		} else if (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR) {
-			netlogon_creds_arcfour_crypt(creds,
-						     logon->generic->data,
-						     logon->generic->length);
+			status = netlogon_creds_arcfour_crypt(creds,
+							      logon->generic->data,
+							      logon->generic->length);
+			if (!NT_STATUS_IS_OK(status)) {
+				return status;
+			}
 		} else {
 			/* Using DES to verify kerberos tickets makes no sense */
 		}
 		break;
 	}
+
+	return NT_STATUS_OK;
 }
 
-void netlogon_creds_decrypt_samlogon_logon(struct netlogon_creds_CredentialState *creds,
-					   enum netr_LogonInfoClass level,
-					   union netr_LogonLevel *logon)
+NTSTATUS netlogon_creds_decrypt_samlogon_logon(struct netlogon_creds_CredentialState *creds,
+					       enum netr_LogonInfoClass level,
+					       union netr_LogonLevel *logon)
 {
-	netlogon_creds_crypt_samlogon_logon(creds, level, logon, false);
+	return netlogon_creds_crypt_samlogon_logon(creds, level, logon, false);
 }
 
-void netlogon_creds_encrypt_samlogon_logon(struct netlogon_creds_CredentialState *creds,
-					   enum netr_LogonInfoClass level,
-					   union netr_LogonLevel *logon)
+NTSTATUS netlogon_creds_encrypt_samlogon_logon(struct netlogon_creds_CredentialState *creds,
+					       enum netr_LogonInfoClass level,
+					       union netr_LogonLevel *logon)
 {
-	netlogon_creds_crypt_samlogon_logon(creds, level, logon, true);
+	return netlogon_creds_crypt_samlogon_logon(creds, level, logon, true);
 }
 
 union netr_LogonLevel *netlogon_creds_shallow_copy_logon(TALLOC_CTX *mem_ctx,
