@@ -31,6 +31,9 @@
 #include "librpc/ndr/libndr.h"
 #include "../lib/util/dlinklist.h"
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_RPC_PARSE
+
 #define NDR_BASE_MARSHALL_SIZE 1024
 
 /* this guid indicates NDR encoding in a protocol tower */
@@ -138,11 +141,11 @@ _PUBLIC_ enum ndr_err_code ndr_pull_pop(struct ndr_pull *ndr)
 		return ndr_pull_error(ndr, NDR_ERR_RELATIVE,
 				      "%s", __location__);
 	}
-	if (ndr->relative_list != NULL) {
+	if (ndr->relative_list.count != 0) {
 		return ndr_pull_error(ndr, NDR_ERR_RELATIVE,
 				      "%s", __location__);
 	}
-	if (ndr->relative_base_list != NULL) {
+	if (ndr->relative_base_list.count != 0) {
 		return ndr_pull_error(ndr, NDR_ERR_RELATIVE,
 				      "%s", __location__);
 	}
@@ -187,7 +190,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_advance(struct ndr_pull *ndr, uint32_t size)
 {
 	ndr->offset += size;
 	if (ndr->offset > ndr->data_size) {
-		return ndr_pull_error(ndr, NDR_ERR_BUFSIZE, 
+		return ndr_pull_error(ndr, NDR_ERR_BUFSIZE,
 				      "ndr_pull_advance by %u failed",
 				      size);
 	}
@@ -201,7 +204,7 @@ static enum ndr_err_code ndr_pull_set_offset(struct ndr_pull *ndr, uint32_t ofs)
 {
 	ndr->offset = ofs;
 	if (ndr->offset > ndr->data_size) {
-		return ndr_pull_error(ndr, NDR_ERR_BUFSIZE, 
+		return ndr_pull_error(ndr, NDR_ERR_BUFSIZE,
 				      "ndr_pull_set_offset %u failed",
 				      ofs);
 	}
@@ -254,6 +257,17 @@ _PUBLIC_ enum ndr_err_code ndr_push_expand(struct ndr_push *ndr, uint32_t extra_
 				      size);
 	}
 
+	if (ndr->fixed_buf_size) {
+		if (ndr->alloc_size >= size) {
+			return NDR_ERR_SUCCESS;
+		}
+		return ndr_push_error(ndr,
+				      NDR_ERR_BUFSIZE,
+				      "Overflow of fixed buffer in "
+				      "push_expand to %u",
+				      size);
+	}
+
 	if (ndr->alloc_size > size) {
 		return NDR_ERR_SUCCESS;
 	}
@@ -303,7 +317,7 @@ _PUBLIC_ void ndr_print_debugc_helper(struct ndr_print *ndr, const char *format,
 	free(s);
 }
 
-_PUBLIC_ void ndr_print_debug_helper(struct ndr_print *ndr, const char *format, ...) 
+_PUBLIC_ void ndr_print_debug_helper(struct ndr_print *ndr, const char *format, ...)
 {
 	va_list ap;
 	char *s = NULL;
@@ -332,7 +346,7 @@ _PUBLIC_ void ndr_print_debug_helper(struct ndr_print *ndr, const char *format,
 	free(s);
 }
 
-_PUBLIC_ void ndr_print_printf_helper(struct ndr_print *ndr, const char *format, ...) 
+_PUBLIC_ void ndr_print_printf_helper(struct ndr_print *ndr, const char *format, ...)
 {
 	va_list ap;
 	uint32_t i;
@@ -364,7 +378,7 @@ _PUBLIC_ void ndr_print_string_helper(struct ndr_print *ndr, const char *format,
 	}
 
 	va_start(ap, format);
-	ndr->private_data = talloc_vasprintf_append_buffer((char *)ndr->private_data, 
+	ndr->private_data = talloc_vasprintf_append_buffer((char *)ndr->private_data,
 						    format, ap);
 	va_end(ap);
 	if (!ndr->no_newline) {
@@ -388,6 +402,12 @@ _PUBLIC_ void ndr_print_debugc(int dbgc_class, ndr_print_fn_t fn, const char *na
 	ndr->print = ndr_print_debugc_helper;
 	ndr->depth = 1;
 	ndr->flags = 0;
+#ifdef DEBUG_PASSWORD
+	if (CHECK_DEBUGLVL(100)) {
+		ndr->print_secrets = true;
+	}
+#endif
+
 	fn(ndr, name, ptr);
 	talloc_free(ndr);
 }
@@ -406,6 +426,12 @@ _PUBLIC_ void ndr_print_debug(ndr_print_fn_t fn, const char *name, void *ptr)
 	ndr->print = ndr_print_debug_helper;
 	ndr->depth = 1;
 	ndr->flags = 0;
+#ifdef DEBUG_PASSWORD
+	if (CHECK_DEBUGLVL(100)) {
+		ndr->print_secrets = true;
+	}
+#endif
+
 	fn(ndr, name, ptr);
 	talloc_free(ndr);
 }
@@ -424,6 +450,12 @@ _PUBLIC_ void ndr_print_union_debug(ndr_print_fn_t fn, const char *name, uint32_
 	ndr->print = ndr_print_debug_helper;
 	ndr->depth = 1;
 	ndr->flags = 0;
+#ifdef DEBUG_PASSWORD
+	if (CHECK_DEBUGLVL(100)) {
+		ndr->print_secrets = true;
+	}
+#endif
+
 	ndr_print_set_switch_value(ndr, ptr, level);
 	fn(ndr, name, ptr);
 	talloc_free(ndr);
@@ -443,6 +475,11 @@ _PUBLIC_ void ndr_print_function_debug(ndr_print_function_t fn, const char *name
 	ndr->print = ndr_print_debug_helper;
 	ndr->depth = 1;
 	ndr->flags = 0;
+#ifdef DEBUG_PASSWORD
+	if (CHECK_DEBUGLVL(100)) {
+		ndr->print_secrets = true;
+	}
+#endif
 
 	fn(ndr, name, flags, ptr);
 	talloc_free(ndr);
@@ -502,7 +539,7 @@ failed:
   a useful helper function for printing idl function calls to a string
 */
 _PUBLIC_ char *ndr_print_function_string(TALLOC_CTX *mem_ctx,
-				ndr_print_function_t fn, const char *name, 
+				ndr_print_function_t fn, const char *name,
 				int flags, void *ptr)
 {
 	struct ndr_print *ndr;
@@ -587,7 +624,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_error(struct ndr_pull *ndr,
 */
 _PUBLIC_ enum ndr_err_code ndr_push_error(struct ndr_push *ndr,
 				 enum ndr_err_code ndr_err,
-				 const char *format, ...) 
+				 const char *format, ...)
 {
 	char *s=NULL;
 	va_list ap;
@@ -743,7 +780,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_subcontext_start(struct ndr_pull *ndr,
 		return NDR_ERR_SUCCESS;
 
 	default:
-		return ndr_pull_error(ndr, NDR_ERR_SUBCONTEXT, "Bad subcontext (PULL) header_size %d", 
+		return ndr_pull_error(ndr, NDR_ERR_SUBCONTEXT, "Bad subcontext (PULL) header_size %d",
 				      (int)header_size);
 	}
 
@@ -829,7 +866,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_subcontext_start(struct ndr_push *ndr,
 }
 
 /*
-  push a subcontext header 
+  push a subcontext header
 */
 _PUBLIC_ enum ndr_err_code ndr_push_subcontext_end(struct ndr_push *ndr,
 				 struct ndr_push *subndr,
@@ -848,14 +885,14 @@ _PUBLIC_ enum ndr_err_code ndr_push_subcontext_end(struct ndr_push *ndr,
 	}
 
 	switch (header_size) {
-	case 0: 
+	case 0:
 		break;
 
-	case 2: 
+	case 2:
 		NDR_CHECK(ndr_push_uint16(ndr, NDR_SCALARS, subndr->offset));
 		break;
 
-	case 4: 
+	case 4:
 		NDR_CHECK(ndr_push_uint3264(ndr, NDR_SCALARS, subndr->offset));
 		break;
 
@@ -895,7 +932,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_subcontext_end(struct ndr_push *ndr,
 		break;
 
 	default:
-		return ndr_push_error(ndr, NDR_ERR_SUBCONTEXT, "Bad subcontext header size %d", 
+		return ndr_push_error(ndr, NDR_ERR_SUBCONTEXT, "Bad subcontext header size %d",
 				      (int)header_size);
 	}
 
@@ -903,40 +940,78 @@ _PUBLIC_ enum ndr_err_code ndr_push_subcontext_end(struct ndr_push *ndr,
 	return NDR_ERR_SUCCESS;
 }
 
+
+struct ndr_token {
+	const void *key;
+	uint32_t value;
+};
+
 /*
   store a token in the ndr context, for later retrieval
 */
 _PUBLIC_ enum ndr_err_code ndr_token_store(TALLOC_CTX *mem_ctx,
-			 struct ndr_token_list **list, 
-			 const void *key, 
+			 struct ndr_token_list *list,
+			 const void *key,
 			 uint32_t value)
 {
-	struct ndr_token_list *tok;
-	tok = talloc(mem_ctx, struct ndr_token_list);
-	NDR_ERR_HAVE_NO_MEMORY(tok);
-	tok->key = key;
-	tok->value = value;
-	DLIST_ADD((*list), tok);
+	if (list->tokens == NULL) {
+		list->tokens = talloc_array(mem_ctx, struct ndr_token, 10);
+		if (list->tokens == NULL) {
+			NDR_ERR_HAVE_NO_MEMORY(list->tokens);
+		}
+	} else {
+		uint32_t alloc_count = talloc_array_length(list->tokens);
+		if (list->count == alloc_count) {
+			unsigned new_alloc;
+			unsigned increment = MIN(list->count, 1000);
+			new_alloc = alloc_count + increment;
+			if (new_alloc < alloc_count) {
+				return NDR_ERR_RANGE;
+			}
+			list->tokens = talloc_realloc(mem_ctx, list->tokens,
+						      struct ndr_token, new_alloc);
+			if (list->tokens == NULL) {
+				NDR_ERR_HAVE_NO_MEMORY(list->tokens);
+			}
+		}
+	}
+	list->tokens[list->count].key = key;
+	list->tokens[list->count].value = value;
+	list->count++;
 	return NDR_ERR_SUCCESS;
 }
 
 /*
   retrieve a token from a ndr context, using cmp_fn to match the tokens
 */
-_PUBLIC_ enum ndr_err_code ndr_token_retrieve_cmp_fn(struct ndr_token_list **list, const void *key, uint32_t *v,
-				   comparison_fn_t _cmp_fn, bool _remove_tok)
+_PUBLIC_ enum ndr_err_code ndr_token_retrieve_cmp_fn(struct ndr_token_list *list,
+						     const void *key, uint32_t *v,
+						     comparison_fn_t _cmp_fn,
+						     bool erase)
 {
-	struct ndr_token_list *tok;
-	for (tok=*list;tok;tok=tok->next) {
-		if (_cmp_fn && _cmp_fn(tok->key,key)==0) goto found;
-		else if (!_cmp_fn && tok->key == key) goto found;
+	struct ndr_token *tokens = list->tokens;
+	unsigned i;
+	if (_cmp_fn) {
+		for (i = list->count - 1; i < list->count; i--) {
+			if (_cmp_fn(tokens[i].key, key) == 0) {
+				goto found;
+			}
+		}
+	} else {
+		for (i = list->count - 1; i < list->count; i--) {
+			if (tokens[i].key == key) {
+				goto found;
+			}
+		}
 	}
 	return NDR_ERR_TOKEN;
 found:
-	*v = tok->value;
-	if (_remove_tok) {
-		DLIST_REMOVE((*list), tok);
-		talloc_free(tok);
+	*v = tokens[i].value;
+	if (erase) {
+		if (i != list->count - 1) {
+			tokens[i] = tokens[list->count - 1];
+		}
+		list->count--;
 	}
 	return NDR_ERR_SUCCESS;
 }
@@ -944,7 +1019,8 @@ found:
 /*
   retrieve a token from a ndr context
 */
-_PUBLIC_ enum ndr_err_code ndr_token_retrieve(struct ndr_token_list **list, const void *key, uint32_t *v)
+_PUBLIC_ enum ndr_err_code ndr_token_retrieve(struct ndr_token_list *list,
+					      const void *key, uint32_t *v)
 {
 	return ndr_token_retrieve_cmp_fn(list, key, v, NULL, true);
 }
@@ -952,17 +1028,18 @@ _PUBLIC_ enum ndr_err_code ndr_token_retrieve(struct ndr_token_list **list, cons
 /*
   peek at but don't removed a token from a ndr context
 */
-_PUBLIC_ uint32_t ndr_token_peek(struct ndr_token_list **list, const void *key)
+_PUBLIC_ uint32_t ndr_token_peek(struct ndr_token_list *list, const void *key)
 {
-	enum ndr_err_code status;
-	uint32_t v;
+	unsigned i;
+	struct ndr_token *tokens = list->tokens;
 
-	status = ndr_token_retrieve_cmp_fn(list, key, &v, NULL, false);
-	if (!NDR_ERR_CODE_IS_SUCCESS(status)) {
-		return 0;
+	for (i = list->count - 1; i < list->count; i--) {
+		if (tokens[i].key == key) {
+			return tokens[i].value;
+		}
 	}
 
-	return v;
+	return 0;
 }
 
 /*
@@ -991,7 +1068,7 @@ _PUBLIC_ enum ndr_err_code ndr_check_array_size(struct ndr_pull *ndr, void *p, u
 	uint32_t stored;
 	stored = ndr_token_peek(&ndr->array_size_list, p);
 	if (stored != size) {
-		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE, 
+		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE,
 				      "Bad array size - got %u expected %u\n",
 				      stored, size);
 	}
@@ -1006,7 +1083,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_array_length(struct ndr_pull *ndr, const voi
 	uint32_t length, offset;
 	NDR_CHECK(ndr_pull_uint3264(ndr, NDR_SCALARS, &offset));
 	if (offset != 0) {
-		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE, 
+		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE,
 				      "non-zero array offset %u\n", offset);
 	}
 	NDR_CHECK(ndr_pull_uint3264(ndr, NDR_SCALARS, &length));
@@ -1029,7 +1106,7 @@ _PUBLIC_ enum ndr_err_code ndr_check_array_length(struct ndr_pull *ndr, void *p,
 	uint32_t stored;
 	stored = ndr_token_peek(&ndr->array_length_list, p);
 	if (stored != length) {
-		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE, 
+		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE,
 				      "Bad array length - got %u expected %u\n",
 				      stored, length);
 	}
@@ -1106,6 +1183,20 @@ _PUBLIC_ uint32_t ndr_print_get_switch_value(struct ndr_print *ndr, const void *
 	return ndr_token_peek(&ndr->switch_list, p);
 }
 
+/* retrieve a switch value and remove it from the list */
+_PUBLIC_ uint32_t ndr_pull_steal_switch_value(struct ndr_pull *ndr, const void *p)
+{
+	enum ndr_err_code status;
+	uint32_t v;
+
+	status = ndr_token_retrieve(&ndr->switch_list, p, &v);
+	if (!NDR_ERR_CODE_IS_SUCCESS(status)) {
+		return 0;
+	}
+
+	return v;
+}
+
 /*
   pull a struct from a blob using NDR
 */
@@ -1123,7 +1214,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_struct_blob(const DATA_BLOB *blob, TALLOC_CT
 /*
   pull a struct from a blob using NDR - failing if all bytes are not consumed
 */
-_PUBLIC_ enum ndr_err_code ndr_pull_struct_blob_all(const DATA_BLOB *blob, TALLOC_CTX *mem_ctx, 
+_PUBLIC_ enum ndr_err_code ndr_pull_struct_blob_all(const DATA_BLOB *blob, TALLOC_CTX *mem_ctx,
 						    void *p, ndr_pull_flags_fn_t fn)
 {
 	struct ndr_pull *ndr;
@@ -1148,10 +1239,49 @@ _PUBLIC_ enum ndr_err_code ndr_pull_struct_blob_all(const DATA_BLOB *blob, TALLO
 	return NDR_ERR_SUCCESS;
 }
 
+/*
+  pull a struct from a blob using NDR - failing if all bytes are not consumed
+
+  This only works for structures with NO allocated memory, like
+  objectSID and GUID.  This helps because we parse these a lot.
+*/
+_PUBLIC_ enum ndr_err_code ndr_pull_struct_blob_all_noalloc(const DATA_BLOB *blob,
+							    void *p, ndr_pull_flags_fn_t fn)
+{
+	/*
+	 * We init this structure on the stack here, to avoid a
+	 * talloc() as otherwise this call to the fn() is assured not
+	 * to be doing any allocation, eg SIDs and GUIDs.
+	 *
+	 * This allows us to keep the safety of the PIDL-generated
+	 * code without the talloc() overhead.
+	 */
+	struct ndr_pull ndr = {
+		.data = blob->data,
+		.data_size = blob->length,
+		.current_mem_ctx = (void *)-1
+	};
+	uint32_t highest_ofs;
+	NDR_CHECK(fn(&ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	if (ndr.offset > ndr.relative_highest_offset) {
+		highest_ofs = ndr.offset;
+	} else {
+		highest_ofs = ndr.relative_highest_offset;
+	}
+	if (highest_ofs < ndr.data_size) {
+		enum ndr_err_code ret;
+		ret = ndr_pull_error(&ndr, NDR_ERR_UNREAD_BYTES,
+				     "not all bytes consumed ofs[%u] size[%u]",
+				     highest_ofs, ndr.data_size);
+		return ret;
+	}
+	return NDR_ERR_SUCCESS;
+}
+
 /*
   pull a union from a blob using NDR, given the union discriminator
 */
-_PUBLIC_ enum ndr_err_code ndr_pull_union_blob(const DATA_BLOB *blob, TALLOC_CTX *mem_ctx, 
+_PUBLIC_ enum ndr_err_code ndr_pull_union_blob(const DATA_BLOB *blob, TALLOC_CTX *mem_ctx,
 					       void *p,
 			     uint32_t level, ndr_pull_flags_fn_t fn)
 {
@@ -1168,7 +1298,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_union_blob(const DATA_BLOB *blob, TALLOC_CTX
   pull a union from a blob using NDR, given the union discriminator,
   failing if all bytes are not consumed
 */
-_PUBLIC_ enum ndr_err_code ndr_pull_union_blob_all(const DATA_BLOB *blob, TALLOC_CTX *mem_ctx, 
+_PUBLIC_ enum ndr_err_code ndr_pull_union_blob_all(const DATA_BLOB *blob, TALLOC_CTX *mem_ctx,
 						   void *p,
 			     uint32_t level, ndr_pull_flags_fn_t fn)
 {
@@ -1213,6 +1343,35 @@ _PUBLIC_ enum ndr_err_code ndr_push_struct_blob(DATA_BLOB *blob, TALLOC_CTX *mem
 	return NDR_ERR_SUCCESS;
 }
 
+/*
+  push a struct into a provided blob using NDR.
+
+  We error because we want to have the performance issue (extra
+  talloc() calls) show up as an error, not just slower code.  This is
+  used for things like GUIDs, which we expect to be a fixed size, and
+  SIDs that we can pre-calculate the size for.
+*/
+_PUBLIC_ enum ndr_err_code ndr_push_struct_into_fixed_blob(
+	DATA_BLOB *blob, const void *p, ndr_push_flags_fn_t fn)
+{
+	struct ndr_push ndr = {
+		.data = blob->data,
+		.alloc_size = blob->length,
+		.fixed_buf_size = true
+	};
+
+	NDR_CHECK(fn(&ndr, NDR_SCALARS|NDR_BUFFERS, p));
+
+	if (ndr.offset != blob->length) {
+		return ndr_push_error(&ndr, NDR_ERR_BUFSIZE,
+				      "buffer was either to large or small "
+				      "ofs[%u] size[%zu]",
+				      ndr.offset, blob->length);
+	}
+
+	return NDR_ERR_SUCCESS;
+}
+
 /*
   push a union to a blob using NDR
 */
@@ -1367,16 +1526,16 @@ static enum ndr_err_code ndr_push_relative_ptr2(struct ndr_push *ndr, const void
 	save_offset = ndr->offset;
 	NDR_CHECK(ndr_token_retrieve(&ndr->relative_list, p, &ptr_offset));
 	if (ptr_offset > ndr->offset) {
-		return ndr_push_error(ndr, NDR_ERR_BUFSIZE, 
+		return ndr_push_error(ndr, NDR_ERR_BUFSIZE,
 				      "ndr_push_relative_ptr2 ptr_offset(%u) > ndr->offset(%u)",
 				      ptr_offset, ndr->offset);
 	}
 	ndr->offset = ptr_offset;
 	if (save_offset < ndr->relative_base_offset) {
-		return ndr_push_error(ndr, NDR_ERR_BUFSIZE, 
+		return ndr_push_error(ndr, NDR_ERR_BUFSIZE,
 				      "ndr_push_relative_ptr2 save_offset(%u) < ndr->relative_base_offset(%u)",
 				      save_offset, ndr->relative_base_offset);
-	}	
+	}
 	NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, save_offset - ndr->relative_base_offset));
 	ndr->offset = save_offset;
 	return NDR_ERR_SUCCESS;
@@ -1389,9 +1548,44 @@ _PUBLIC_ enum ndr_err_code ndr_push_short_relative_ptr2(struct ndr_push *ndr, co
 {
 	uint32_t save_offset;
 	uint32_t ptr_offset = 0xFFFF;
+	uint32_t relative_offset;
+	size_t pad;
+	size_t align = 1;
+
 	if (p == NULL) {
 		return NDR_ERR_SUCCESS;
 	}
+
+	if (ndr->offset < ndr->relative_base_offset) {
+		return ndr_push_error(ndr, NDR_ERR_BUFSIZE,
+				      "ndr_push_relative_ptr2 ndr->offset(%u) < ndr->relative_base_offset(%u)",
+				      ndr->offset, ndr->relative_base_offset);
+	}
+
+	relative_offset = ndr->offset - ndr->relative_base_offset;
+
+	if (ndr->flags & LIBNDR_FLAG_NOALIGN) {
+		align = 1;
+	} else if (ndr->flags & LIBNDR_FLAG_ALIGN2) {
+		align = 2;
+	} else if (ndr->flags & LIBNDR_FLAG_ALIGN4) {
+		align = 4;
+	} else if (ndr->flags & LIBNDR_FLAG_ALIGN8) {
+		align = 8;
+	}
+
+	pad = ndr_align_size(relative_offset, align);
+	if (pad != 0) {
+		NDR_CHECK(ndr_push_zero(ndr, pad));
+	}
+
+	relative_offset = ndr->offset - ndr->relative_base_offset;
+	if (relative_offset > UINT16_MAX) {
+		return ndr_push_error(ndr, NDR_ERR_BUFSIZE,
+				      "ndr_push_relative_ptr2 relative_offset(%u) > UINT16_MAX",
+				      relative_offset);
+	}
+
 	save_offset = ndr->offset;
 	NDR_CHECK(ndr_token_retrieve(&ndr->relative_list, p, &ptr_offset));
 	if (ptr_offset > ndr->offset) {
@@ -1400,12 +1594,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_short_relative_ptr2(struct ndr_push *ndr, co
 				      ptr_offset, ndr->offset);
 	}
 	ndr->offset = ptr_offset;
-	if (save_offset < ndr->relative_base_offset) {
-		return ndr_push_error(ndr, NDR_ERR_BUFSIZE,
-				      "ndr_push_relative_ptr2 save_offset(%u) < ndr->relative_base_offset(%u)",
-				      save_offset, ndr->relative_base_offset);
-	}
-	NDR_CHECK(ndr_push_uint16(ndr, NDR_SCALARS, save_offset - ndr->relative_base_offset));
+	NDR_CHECK(ndr_push_uint16(ndr, NDR_SCALARS, relative_offset));
 	ndr->offset = save_offset;
 	return NDR_ERR_SUCCESS;
 }
@@ -1606,7 +1795,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_relative_ptr1(struct ndr_pull *ndr, const vo
 {
 	rel_offset += ndr->relative_base_offset;
 	if (rel_offset > ndr->data_size) {
-		return ndr_pull_error(ndr, NDR_ERR_BUFSIZE, 
+		return ndr_pull_error(ndr, NDR_ERR_BUFSIZE,
 				      "ndr_pull_relative_ptr1 rel_offset(%u) > ndr->data_size(%u)",
 				      rel_offset, ndr->data_size);
 	}
