@@ -39,7 +39,7 @@ from samba.dsdb import (
     DS_DOMAIN_FUNCTION_2008_R2,
     DS_DOMAIN_FUNCTION_2012_R2,
     DS_DOMAIN_FUNCTION_2016
-    )
+)
 from samba.descriptor import (
     get_domain_descriptor,
     get_domain_delete_protected1_descriptor,
@@ -47,7 +47,7 @@ from samba.descriptor import (
     get_dns_partition_descriptor,
     get_dns_forest_microsoft_dns_descriptor,
     get_dns_domain_microsoft_dns_descriptor
-    )
+)
 from samba.provision.common import (
     setup_path,
     setup_add_ldif,
@@ -57,13 +57,14 @@ from samba.provision.common import (
     FILL_SUBDOMAIN,
     FILL_NT4SYNC,
     FILL_DRS,
-    )
+)
 
 from samba.samdb import get_default_backend_store
+from samba.compat import get_string
 
 def get_domainguid(samdb, domaindn):
     res = samdb.search(base=domaindn, scope=ldb.SCOPE_BASE, attrs=["objectGUID"])
-    domainguid =  str(ndr_unpack(misc.GUID, res[0]["objectGUID"][0]))
+    domainguid = str(ndr_unpack(misc.GUID, res[0]["objectGUID"][0]))
     return domainguid
 
 
@@ -141,7 +142,7 @@ class SOARecord(dnsp.DnssrvRpcRecord):
 class SRVRecord(dnsp.DnssrvRpcRecord):
 
     def __init__(self, target, port, priority=0, weight=100, serial=1, ttl=900,
-                rank=dnsp.DNS_RANK_ZONE):
+                 rank=dnsp.DNS_RANK_ZONE):
         super(SRVRecord, self).__init__()
         self.wType = dnsp.DNS_TYPE_SRV
         self.rank = rank
@@ -234,7 +235,7 @@ class AgingEnabledTimeProperty(dnsp.DnsProperty):
     def __init__(self, next_cycle_hours=0):
         super(AgingEnabledTimeProperty, self).__init__()
         self.wDataLength = 1
-        self.version = 1;
+        self.version = 1
         self.id = dnsp.DSPROPERTY_ZONE_AGING_ENABLED_TIME
         self.data = next_cycle_hours
 
@@ -247,12 +248,12 @@ def setup_dns_partitions(samdb, domainsid, domaindn, forestdn, configdn,
 
     setup_add_ldif(samdb, setup_path("provision_dnszones_partitions.ldif"), {
         "ZONE_DN": domainzone_dn,
-        "SECDESC"      : b64encode(descriptor).decode('utf8')
-        })
+        "SECDESC": b64encode(descriptor).decode('utf8')
+    })
     if fill_level != FILL_SUBDOMAIN:
         setup_add_ldif(samdb, setup_path("provision_dnszones_partitions.ldif"), {
             "ZONE_DN": forestzone_dn,
-            "SECDESC"      : b64encode(descriptor).decode('utf8')
+            "SECDESC": b64encode(descriptor).decode('utf8')
         })
 
     domainzone_guid = get_domainguid(samdb, domainzone_dn)
@@ -269,7 +270,7 @@ def setup_dns_partitions(samdb, domainsid, domaindn, forestdn, configdn,
         "SERVERDN": serverdn,
         "LOSTANDFOUND_DESCRIPTOR": b64encode(protected2_desc).decode('utf8'),
         "INFRASTRUCTURE_DESCRIPTOR": b64encode(protected1_desc).decode('utf8'),
-        })
+    })
     setup_modify_ldif(samdb, setup_path("provision_dnszones_modify.ldif"), {
         "CONFIGDN": configdn,
         "SERVERDN": serverdn,
@@ -300,7 +301,7 @@ def setup_dns_partitions(samdb, domainsid, domaindn, forestdn, configdn,
 def add_dns_accounts(samdb, domaindn):
     setup_add_ldif(samdb, setup_path("provision_dns_accounts_add.ldif"), {
         "DOMAINDN": domaindn,
-        })
+    })
 
 
 def add_dns_container(samdb, domaindn, prefix, domain_sid, dnsadmins_sid, forest=False):
@@ -314,8 +315,9 @@ def add_dns_container(samdb, domaindn, prefix, domain_sid, dnsadmins_sid, forest
     # CN=MicrosoftDNS,<PREFIX>,<DOMAINDN>
     msg = ldb.Message(ldb.Dn(samdb, "CN=MicrosoftDNS,%s,%s" % (prefix, domaindn)))
     msg["objectClass"] = ["top", "container"]
-    msg["nTSecurityDescriptor"] = ldb.MessageElement(sd_val, ldb.FLAG_MOD_ADD,
-        "nTSecurityDescriptor")
+    msg["nTSecurityDescriptor"] = \
+        ldb.MessageElement(sd_val, ldb.FLAG_MOD_ADD,
+                           "nTSecurityDescriptor")
     samdb.add(msg)
 
 
@@ -382,13 +384,14 @@ def add_rootservers(samdb, domaindn, prefix):
     for rserver in rootservers:
         record = [ndr_pack(ARecord(rootservers[rserver], serial=0, ttl=0, rank=dnsp.DNS_RANK_ROOT_HINT))]
         # Add AAAA record as well (How does W2K* add IPv6 records?)
-        #if rserver in rootservers_v6:
+        # if rserver in rootservers_v6:
         #    record.append(ndr_pack(AAAARecord(rootservers_v6[rserver], serial=0, ttl=0)))
         msg = ldb.Message(ldb.Dn(samdb, "DC=%s,%s" % (rserver, container_dn)))
         msg["objectClass"] = ["top", "dnsNode"]
         msg["dnsRecord"] = ldb.MessageElement(record, ldb.FLAG_MOD_ADD, "dnsRecord")
         samdb.add(msg)
 
+
 def add_at_record(samdb, container_dn, prefix, hostname, dnsdomain, hostip, hostip6):
 
     fqdn_hostname = "%s.%s" % (hostname, dnsdomain)
@@ -469,18 +472,18 @@ def add_host_record(samdb, container_dn, prefix, hostip, hostip6):
 def add_domain_record(samdb, domaindn, prefix, dnsdomain, domainsid, dnsadmins_sid):
     # DC=<DNSDOMAIN>,CN=MicrosoftDNS,<PREFIX>,<DOMAINDN>
     sddl = "O:SYG:BAD:AI" \
-    "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)" \
-    "(A;;CC;;;AU)" \
-    "(A;;RPLCLORC;;;WD)" \
-    "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)" \
-    "(A;CI;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)" \
-    "(A;CIID;RPWPCRCCDCLCRCWOWDSDDTSW;;;%s)" \
-    "(A;CIID;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)" \
-    "(OA;CIID;RPWPCR;91e647de-d96f-4b70-9557-d63ff4f3ccd8;;PS)" \
-    "(A;CIID;RPWPCRCCDCLCLORCWOWDSDDTSW;;;EA)" \
-    "(A;CIID;LC;;;RU)" \
-    "(A;CIID;RPWPCRCCLCLORCWOWDSDSW;;;BA)" \
-    "S:AI" % dnsadmins_sid
+        "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)" \
+        "(A;;CC;;;AU)" \
+        "(A;;RPLCLORC;;;WD)" \
+        "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)" \
+        "(A;CI;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)" \
+        "(A;CIID;RPWPCRCCDCLCRCWOWDSDDTSW;;;%s)" \
+        "(A;CIID;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)" \
+        "(OA;CIID;RPWPCR;91e647de-d96f-4b70-9557-d63ff4f3ccd8;;PS)" \
+        "(A;CIID;RPWPCRCCDCLCLORCWOWDSDDTSW;;;EA)" \
+        "(A;CIID;LC;;;RU)" \
+        "(A;CIID;RPWPCRCCLCLORCWOWDSDSW;;;BA)" \
+        "S:AI" % dnsadmins_sid
     sec = security.descriptor.from_sddl(sddl, domainsid)
     props = []
     props.append(ndr_pack(TypeProperty()))
@@ -492,8 +495,10 @@ def add_domain_record(samdb, domaindn, prefix, dnsdomain, domainsid, dnsadmins_s
     props.append(ndr_pack(AgingEnabledTimeProperty()))
     msg = ldb.Message(ldb.Dn(samdb, "DC=%s,CN=MicrosoftDNS,%s,%s" % (dnsdomain, prefix, domaindn)))
     msg["objectClass"] = ["top", "dnsZone"]
-    msg["ntSecurityDescriptor"] = ldb.MessageElement(ndr_pack(sec), ldb.FLAG_MOD_ADD,
-        "nTSecurityDescriptor")
+    msg["ntSecurityDescriptor"] = \
+        ldb.MessageElement(ndr_pack(sec),
+                           ldb.FLAG_MOD_ADD,
+                           "nTSecurityDescriptor")
     msg["dNSProperty"] = ldb.MessageElement(props, ldb.FLAG_MOD_ADD, "dNSProperty")
     samdb.add(msg)
 
@@ -507,49 +512,49 @@ def add_msdcs_record(samdb, forestdn, prefix, dnsforest):
 
 
 def add_dc_domain_records(samdb, domaindn, prefix, site, dnsdomain, hostname,
-        hostip, hostip6):
+                          hostip, hostip6):
 
     fqdn_hostname = "%s.%s" % (hostname, dnsdomain)
 
     # Set up domain container - DC=<DNSDOMAIN>,CN=MicrosoftDNS,<PREFIX>,<DOMAINDN>
     domain_container_dn = ldb.Dn(samdb, "DC=%s,CN=MicrosoftDNS,%s,%s" %
-                                    (dnsdomain, prefix, domaindn))
+                                 (dnsdomain, prefix, domaindn))
 
     # DC=@ record
     add_at_record(samdb, domain_container_dn, "DC=@", hostname, dnsdomain,
-            hostip, hostip6)
+                  hostip, hostip6)
 
     # DC=<HOSTNAME> record
     add_host_record(samdb, domain_container_dn, "DC=%s" % hostname, hostip,
-            hostip6)
+                    hostip6)
 
     # DC=_kerberos._tcp record
     add_srv_record(samdb, domain_container_dn, "DC=_kerberos._tcp",
-            fqdn_hostname, 88)
+                   fqdn_hostname, 88)
 
     # DC=_kerberos._tcp.<SITENAME>._sites record
     add_srv_record(samdb, domain_container_dn, "DC=_kerberos._tcp.%s._sites" %
-            site, fqdn_hostname, 88)
+                   site, fqdn_hostname, 88)
 
     # DC=_kerberos._udp record
     add_srv_record(samdb, domain_container_dn, "DC=_kerberos._udp",
-            fqdn_hostname, 88)
+                   fqdn_hostname, 88)
 
     # DC=_kpasswd._tcp record
     add_srv_record(samdb, domain_container_dn, "DC=_kpasswd._tcp",
-            fqdn_hostname, 464)
+                   fqdn_hostname, 464)
 
     # DC=_kpasswd._udp record
     add_srv_record(samdb, domain_container_dn, "DC=_kpasswd._udp",
-            fqdn_hostname, 464)
+                   fqdn_hostname, 464)
 
     # DC=_ldap._tcp record
     add_srv_record(samdb, domain_container_dn, "DC=_ldap._tcp", fqdn_hostname,
-            389)
+                   389)
 
     # DC=_ldap._tcp.<SITENAME>._sites record
     add_srv_record(samdb, domain_container_dn, "DC=_ldap._tcp.%s._sites" %
-            site, fqdn_hostname, 389)
+                   site, fqdn_hostname, 389)
 
     # FIXME: The number of SRV records depend on the various roles this DC has.
     #        _gc and _msdcs records are added if the we are the forest dc and not subdomain dc
@@ -558,11 +563,11 @@ def add_dc_domain_records(samdb, domaindn, prefix, site, dnsdomain, hostname,
 
     # DC=_gc._tcp record
     add_srv_record(samdb, domain_container_dn, "DC=_gc._tcp", fqdn_hostname,
-            3268)
+                   3268)
 
     # DC=_gc._tcp.<SITENAME>,_sites record
     add_srv_record(samdb, domain_container_dn, "DC=_gc._tcp.%s._sites" % site,
-            fqdn_hostname, 3268)
+                   fqdn_hostname, 3268)
 
     # DC=_msdcs record
     add_ns_glue_record(samdb, domain_container_dn, "DC=_msdcs", fqdn_hostname)
@@ -574,82 +579,82 @@ def add_dc_domain_records(samdb, domaindn, prefix, site, dnsdomain, hostname,
 
     # DC=_ldap._tcp.<SITENAME>._sites.DomainDnsZones
     add_srv_record(samdb, domain_container_dn,
-            "DC=_ldap._tcp.%s._sites.DomainDnsZones" % site, fqdn_hostname,
-            389)
+                   "DC=_ldap._tcp.%s._sites.DomainDnsZones" % site, fqdn_hostname,
+                   389)
 
     # DC=_ldap._tcp.<SITENAME>._sites.ForestDnsZones
     add_srv_record(samdb, domain_container_dn,
-            "DC=_ldap._tcp.%s._sites.ForestDnsZones" % site, fqdn_hostname,
-            389)
+                   "DC=_ldap._tcp.%s._sites.ForestDnsZones" % site, fqdn_hostname,
+                   389)
 
     # DC=_ldap._tcp.DomainDnsZones
     add_srv_record(samdb, domain_container_dn, "DC=_ldap._tcp.DomainDnsZones",
-                    fqdn_hostname, 389)
+                   fqdn_hostname, 389)
 
     # DC=_ldap._tcp.ForestDnsZones
     add_srv_record(samdb, domain_container_dn, "DC=_ldap._tcp.ForestDnsZones",
-                    fqdn_hostname, 389)
+                   fqdn_hostname, 389)
 
     # DC=DomainDnsZones
     add_host_record(samdb, domain_container_dn, "DC=DomainDnsZones", hostip,
-            hostip6)
+                    hostip6)
 
     # DC=ForestDnsZones
     add_host_record(samdb, domain_container_dn, "DC=ForestDnsZones", hostip,
-            hostip6)
+                    hostip6)
 
 
 def add_dc_msdcs_records(samdb, forestdn, prefix, site, dnsforest, hostname,
-                            hostip, hostip6, domainguid, ntdsguid):
+                         hostip, hostip6, domainguid, ntdsguid):
 
     fqdn_hostname = "%s.%s" % (hostname, dnsforest)
 
     # Set up forest container - DC=<DNSDOMAIN>,CN=MicrosoftDNS,<PREFIX>,<DOMAINDN>
     forest_container_dn = ldb.Dn(samdb, "DC=_msdcs.%s,CN=MicrosoftDNS,%s,%s" %
-                                    (dnsforest, prefix, forestdn))
+                                 (dnsforest, prefix, forestdn))
 
     # DC=@ record
     add_at_record(samdb, forest_container_dn, "DC=@", hostname, dnsforest,
-            None, None)
+                  None, None)
 
     # DC=_kerberos._tcp.dc record
     add_srv_record(samdb, forest_container_dn, "DC=_kerberos._tcp.dc",
-            fqdn_hostname, 88)
+                   fqdn_hostname, 88)
 
     # DC=_kerberos._tcp.<SITENAME>._sites.dc record
     add_srv_record(samdb, forest_container_dn,
-            "DC=_kerberos._tcp.%s._sites.dc" % site, fqdn_hostname, 88)
+                   "DC=_kerberos._tcp.%s._sites.dc" % site, fqdn_hostname, 88)
 
     # DC=_ldap._tcp.dc record
     add_srv_record(samdb, forest_container_dn, "DC=_ldap._tcp.dc",
-            fqdn_hostname, 389)
+                   fqdn_hostname, 389)
 
     # DC=_ldap._tcp.<SITENAME>._sites.dc record
     add_srv_record(samdb, forest_container_dn, "DC=_ldap._tcp.%s._sites.dc" %
-            site, fqdn_hostname, 389)
+                   site, fqdn_hostname, 389)
 
     # DC=_ldap._tcp.<SITENAME>._sites.gc record
     add_srv_record(samdb, forest_container_dn, "DC=_ldap._tcp.%s._sites.gc" %
-            site, fqdn_hostname, 3268)
+                   site, fqdn_hostname, 3268)
 
     # DC=_ldap._tcp.gc record
     add_srv_record(samdb, forest_container_dn, "DC=_ldap._tcp.gc",
-            fqdn_hostname, 3268)
+                   fqdn_hostname, 3268)
 
     # DC=_ldap._tcp.pdc record
     add_srv_record(samdb, forest_container_dn, "DC=_ldap._tcp.pdc",
-            fqdn_hostname, 389)
+                   fqdn_hostname, 389)
 
     # DC=gc record
     add_host_record(samdb, forest_container_dn, "DC=gc", hostip, hostip6)
 
     # DC=_ldap._tcp.<DOMAINGUID>.domains record
     add_srv_record(samdb, forest_container_dn,
-            "DC=_ldap._tcp.%s.domains" % domainguid, fqdn_hostname, 389)
+                   "DC=_ldap._tcp.%s.domains" % domainguid, fqdn_hostname, 389)
 
     # DC=<NTDSGUID>
     add_cname_record(samdb, forest_container_dn, "DC=%s" % ntdsguid,
-            fqdn_hostname)
+                     fqdn_hostname)
 
 
 def secretsdb_setup_dns(secretsdb, names, private_dir, binddns_dir, realm,
@@ -678,7 +683,7 @@ def secretsdb_setup_dns(secretsdb, names, private_dir, binddns_dir, realm,
             "DNSPASS_B64": b64encode(dnspass.encode('utf-8')).decode('utf8'),
             "KEY_VERSION_NUMBER": str(key_version_number),
             "HOSTNAME": names.hostname,
-            "DNSNAME" : '%s.%s' % (
+            "DNSNAME": '%s.%s' % (
                 names.netbiosname.lower(), names.dnsdomain.lower())
             })
 
@@ -704,7 +709,7 @@ def create_dns_dir(logger, paths):
             # chmod needed to cope with umask
             os.chmod(dns_dir, 0o770)
         except OSError:
-            if not os.environ.has_key('SAMBA_SELFTEST'):
+            if 'SAMBA_SELFTEST' not in os.environ:
                 logger.error("Failed to chown %s to bind gid %u" % (
                     dns_dir, paths.bind_gid))
 
@@ -763,7 +768,7 @@ def create_zone_file(lp, logger, paths, targetdir, dnsdomain,
             "HOSTIP6_HOST_LINE": hostip6_host_line,
             "GC_MSDCS_IP_LINE": gc_msdcs_ip_line,
             "GC_MSDCS_IP6_LINE": gc_msdcs_ip6_line,
-        })
+    })
 
     if paths.bind_gid is not None:
         try:
@@ -771,7 +776,7 @@ def create_zone_file(lp, logger, paths, targetdir, dnsdomain,
             # chmod needed to cope with umask
             os.chmod(paths.dns, 0o664)
         except OSError:
-            if not os.environ.has_key('SAMBA_SELFTEST'):
+            if 'SAMBA_SELFTEST' not in os.environ:
                 logger.error("Failed to chown %s to bind gid %u" % (
                     paths.dns, paths.bind_gid))
 
@@ -793,12 +798,12 @@ def create_samdb_copy(samdb, logger, paths, names, domainsid, domainguid):
                        scope=ldb.SCOPE_BASE,
                        attrs=["partition", "backendStore"])
     for tmp in res[0]["partition"]:
-        (nc, fname) = tmp.split(':')
+        (nc, fname) = str(tmp).split(':')
         partfile[nc.upper()] = fname
 
     backend_store = get_default_backend_store()
     if "backendStore" in res[0]:
-        backend_store = res[0]["backendStore"][0]
+        backend_store = str(res[0]["backendStore"][0])
 
     # Create empty domain partition
 
@@ -819,12 +824,12 @@ def create_samdb_copy(samdb, logger, paths, names, domainsid, domainguid):
         domainguid_line = "objectGUID: %s\n-" % domainguid
         descr = b64encode(get_domain_descriptor(domainsid)).decode('utf8')
         setup_add_ldif(dom_ldb, setup_path("provision_basedn.ldif"), {
-            "DOMAINDN" : names.domaindn,
-            "DOMAINGUID" : domainguid_line,
-            "DOMAINSID" : str(domainsid),
-            "DESCRIPTOR" : descr})
+            "DOMAINDN": names.domaindn,
+            "DOMAINGUID": domainguid_line,
+            "DOMAINSID": str(domainsid),
+            "DESCRIPTOR": descr})
         setup_add_ldif(dom_ldb,
-            setup_path("provision_basedn_options.ldif"), None)
+                       setup_path("provision_basedn_options.ldif"), None)
 
     except:
         logger.error(
@@ -850,12 +855,22 @@ def create_samdb_copy(samdb, logger, paths, names, domainsid, domainguid):
     metadata_file = "metadata.tdb"
     try:
         os.link(os.path.join(samldb_dir, metadata_file),
-            os.path.join(dns_samldb_dir, metadata_file))
+                os.path.join(dns_samldb_dir, metadata_file))
         os.link(os.path.join(private_dir, domainzone_file),
-            os.path.join(dns_dir, domainzone_file))
+                os.path.join(dns_dir, domainzone_file))
+        if backend_store == "mdb":
+            # If the file is an lmdb data file need to link the
+            # lock file as well
+            os.link(os.path.join(private_dir, domainzone_file + "-lock"),
+                    os.path.join(dns_dir, domainzone_file + "-lock"))
         if forestzone_file:
             os.link(os.path.join(private_dir, forestzone_file),
                     os.path.join(dns_dir, forestzone_file))
+            if backend_store == "mdb":
+                # If the database file is an lmdb data file need to link the
+                # lock file as well
+                os.link(os.path.join(private_dir, forestzone_file + "-lock"),
+                        os.path.join(dns_dir, forestzone_file + "-lock"))
     except OSError:
         logger.error(
             "Failed to setup database for BIND, AD based DNS cannot be used")
@@ -873,10 +888,10 @@ def create_samdb_copy(samdb, logger, paths, names, domainsid, domainguid):
             pfile = partfile[nc]
             if backend_store == "mdb":
                 mdb_copy(os.path.join(private_dir, pfile),
-                        os.path.join(dns_dir, pfile))
+                         os.path.join(dns_dir, pfile))
             else:
                 tdb_copy(os.path.join(private_dir, pfile),
-                        os.path.join(dns_dir, pfile))
+                         os.path.join(dns_dir, pfile))
     except:
         logger.error(
             "Failed to setup database for BIND, AD based DNS cannot be used")
@@ -896,11 +911,11 @@ def create_samdb_copy(samdb, logger, paths, names, domainsid, domainguid):
                         os.chown(fpath, -1, paths.bind_gid)
                         os.chmod(fpath, 0o660)
         except OSError:
-            if not os.environ.has_key('SAMBA_SELFTEST'):
+            if 'SAMBA_SELFTEST' not in os.environ:
                 logger.error(
                     "Failed to set permissions to sam.ldb* files, fix manually")
     else:
-        if not os.environ.has_key('SAMBA_SELFTEST'):
+        if 'SAMBA_SELFTEST' not in os.environ:
             logger.warning("""Unable to find group id for BIND,
                 set permissions to sam.ldb* files manually""")
 
@@ -949,10 +964,12 @@ def create_named_conf(paths, realm, dnsdomain, dns_backend, logger):
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.STDOUT,
                                      cwd='.').communicate()[0]
+        bind_info = get_string(bind_info)
         bind9_8 = '#'
         bind9_9 = '#'
         bind9_10 = '#'
         bind9_11 = '#'
+        bind9_12 = '#'
         if bind_info.upper().find('BIND 9.8') != -1:
             bind9_8 = ''
         elif bind_info.upper().find('BIND 9.9') != -1:
@@ -961,22 +978,26 @@ def create_named_conf(paths, realm, dnsdomain, dns_backend, logger):
             bind9_10 = ''
         elif bind_info.upper().find('BIND 9.11') != -1:
             bind9_11 = ''
+        elif bind_info.upper().find('BIND 9.12') != -1:
+            bind9_12 = ''
         elif bind_info.upper().find('BIND 9.7') != -1:
             raise ProvisioningError("DLZ option incompatible with BIND 9.7.")
         else:
             logger.warning("BIND version unknown, please modify %s manually." % paths.namedconf)
         setup_file(setup_path("named.conf.dlz"), paths.namedconf, {
                     "NAMED_CONF": paths.namedconf,
-                    "MODULESDIR" : samba.param.modules_dir(),
-                    "BIND9_8" : bind9_8,
-                    "BIND9_9" : bind9_9,
-                    "BIND9_10" : bind9_10,
-                    "BIND9_11" : bind9_11
+                    "MODULESDIR": samba.param.modules_dir(),
+                    "BIND9_8": bind9_8,
+                    "BIND9_9": bind9_9,
+                    "BIND9_10": bind9_10,
+                    "BIND9_11": bind9_11,
+                    "BIND9_12": bind9_12
+
                     })
 
 
 def create_named_txt(path, realm, dnsdomain, dnsname, binddns_dir,
-    keytab_name):
+                     keytab_name):
     """Write out a file containing zone statements suitable for inclusion in a
     named.conf file (including GSS-TSIG configuration).
 
@@ -988,12 +1009,12 @@ def create_named_txt(path, realm, dnsdomain, dnsname, binddns_dir,
     """
     setup_file(setup_path("named.txt"), path, {
             "DNSDOMAIN": dnsdomain,
-            "DNSNAME" : dnsname,
+            "DNSNAME": dnsname,
             "REALM": realm,
             "DNS_KEYTAB": keytab_name,
             "DNS_KEYTAB_ABS": os.path.join(binddns_dir, keytab_name),
             "PRIVATE_DIR": binddns_dir
-        })
+    })
 
 
 def is_valid_dns_backend(dns_backend):
@@ -1026,7 +1047,7 @@ def create_dns_partitions(samdb, domainsid, names, domaindn, forestdn,
                           dnsadmins_sid, fill_level):
     # Set up additional partitions (DomainDnsZones, ForstDnsZones)
     setup_dns_partitions(samdb, domainsid, domaindn, forestdn,
-                        names.configdn, names.serverdn, fill_level)
+                         names.configdn, names.serverdn, fill_level)
 
     # Set up MicrosoftDNS containers
     add_dns_container(samdb, domaindn, "DC=DomainDnsZones", domainsid,
@@ -1058,7 +1079,7 @@ def fill_dns_data_partitions(samdb, domainsid, site, domaindn, forestdn,
     :param autofill: Create DNS records (using fixed template)
     """
 
-    ##### Set up DC=DomainDnsZones,<DOMAINDN>
+    # Set up DC=DomainDnsZones,<DOMAINDN>
     # Add rootserver records
     if add_root:
         add_rootservers(samdb, domaindn, "DC=DomainDnsZones")
@@ -1073,7 +1094,7 @@ def fill_dns_data_partitions(samdb, domainsid, site, domaindn, forestdn,
                               dnsdomain, hostname, hostip, hostip6)
 
     if fill_level != FILL_SUBDOMAIN:
-        ##### Set up DC=ForestDnsZones,<FORESTDN>
+        # Set up DC=ForestDnsZones,<FORESTDN>
         # Add _msdcs record
         add_msdcs_record(samdb, forestdn, "DC=ForestDnsZones", dnsforest)
 
@@ -1085,8 +1106,8 @@ def fill_dns_data_partitions(samdb, domainsid, site, domaindn, forestdn,
 
 
 def setup_ad_dns(samdb, secretsdb, names, paths, lp, logger,
-        dns_backend, os_level, dnspass=None, hostip=None, hostip6=None,
-        targetdir=None, fill_level=FILL_FULL, backend_store=None):
+                 dns_backend, os_level, dnspass=None, hostip=None, hostip6=None,
+                 targetdir=None, fill_level=FILL_FULL, backend_store=None):
     """Provision DNS information (assuming GC role)
 
     :param samdb: LDB object connected to sam.ldb file
@@ -1186,9 +1207,9 @@ def setup_ad_dns(samdb, secretsdb, names, paths, lp, logger,
 
 
 def setup_bind9_dns(samdb, secretsdb, names, paths, lp, logger,
-        dns_backend, os_level, site=None, dnspass=None, hostip=None,
-        hostip6=None, targetdir=None, key_version_number=None,
-        backend_store=None):
+                    dns_backend, os_level, site=None, dnspass=None, hostip=None,
+                    hostip6=None, targetdir=None, key_version_number=None,
+                    backend_store=None):
     """Provision DNS information (assuming BIND9 backend in DC role)
 
     :param samdb: LDB object connected to sam.ldb file
@@ -1244,7 +1265,7 @@ def setup_bind9_dns(samdb, secretsdb, names, paths, lp, logger,
 
     create_named_txt(paths.namedtxt,
                      realm=names.realm, dnsdomain=names.dnsdomain,
-                     dnsname = "%s.%s" % (names.hostname, names.dnsdomain),
+                     dnsname="%s.%s" % (names.hostname, names.dnsdomain),
                      binddns_dir=paths.binddns_dir,
                      keytab_name=paths.dns_keytab)
     logger.info("See %s for an example configuration include file for BIND",
