@@ -22,9 +22,10 @@ from __future__ import print_function
 """Blackbox tests for ndrdump."""
 
 import os
-from samba.tests import BlackboxTestCase
+from samba.tests import BlackboxTestCase, BlackboxProcessError
 
-for p in [ "../../../../../source4/librpc/tests", "../../../../../librpc/tests"]:
+for p in ["../../../../../source4/librpc/tests",
+          "../../../../../librpc/tests"]:
     data_path_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), p))
     print(data_path_dir)
     if os.path.exists(data_path_dir):
@@ -38,17 +39,32 @@ class NdrDumpTests(BlackboxTestCase):
         return os.path.join(data_path_dir, name)
 
     def test_ndrdump_with_in(self):
-        self.check_run("ndrdump samr samr_CreateUser in %s" % (self.data_path("samr-CreateUser-in.dat")))
+        self.check_run(("ndrdump samr samr_CreateUser in %s" %
+                       (self.data_path("samr-CreateUser-in.dat"))))
 
     def test_ndrdump_with_out(self):
-        self.check_run("ndrdump samr samr_CreateUser out %s" % (self.data_path("samr-CreateUser-out.dat")))
+        self.check_run(("ndrdump samr samr_CreateUser out %s" %
+                       (self.data_path("samr-CreateUser-out.dat"))))
 
     def test_ndrdump_context_file(self):
-        self.check_run("ndrdump --context-file %s samr samr_CreateUser out %s" % (self.data_path("samr-CreateUser-in.dat"), self.data_path("samr-CreateUser-out.dat")))
+        self.check_run(
+            ("ndrdump --context-file %s samr samr_CreateUser out %s" %
+                (self.data_path("samr-CreateUser-in.dat"),
+                self.data_path("samr-CreateUser-out.dat"))))
 
     def test_ndrdump_with_validate(self):
-        self.check_run("ndrdump --validate samr samr_CreateUser in %s" % (self.data_path("samr-CreateUser-in.dat")))
+        self.check_run(("ndrdump --validate samr samr_CreateUser in %s" %
+                       (self.data_path("samr-CreateUser-in.dat"))))
 
-    def test_ndrdump_with_hex(self):
-        self.check_run("ndrdump dns decode_dns_name_packet in --hex-input %s" %
-                       self.data_path("dns-decode_dns_name_packet-hex.dat"))
+    def test_ndrdump_with_hex_decode_function(self):
+        self.check_run(
+            ("ndrdump dns decode_dns_name_packet in --hex-input %s" %
+                self.data_path("dns-decode_dns_name_packet-hex.dat")))
+
+    def test_ndrdump_with_hex_struct_name(self):
+        try:
+            self.check_run(
+                "ndrdump dns dns_name_packet struct --hex-input %s" %
+                self.data_path("dns-decode_dns_name_packet-hex.dat"))
+        except BlackboxProcessError as e:
+            self.fail(e)
