@@ -53,7 +53,6 @@ else:
     has_perl_test_more = False
 
 python = os.getenv("PYTHON", "python")
-extra_python = os.getenv("EXTRA_PYTHON", None)
 
 tap2subunit = python + " " + os.path.join(srcdir(), "selftest", "tap2subunit")
 
@@ -136,8 +135,7 @@ def planperltestsuite(name, path):
         skiptestsuite(name, "Test::More not available")
 
 
-def planpythontestsuite(env, module, name=None, extra_path=None,
-                        py3_compatible=False):
+def planpythontestsuite(env, module, name=None, extra_path=None):
     if name is None:
         name = module
     args = [python, "-m", "samba.subunit.run", "$LISTOPT", "$LOADLIST", module]
@@ -147,11 +145,6 @@ def planpythontestsuite(env, module, name=None, extra_path=None,
         pypath = []
 
     plantestsuite_loadlist(name, env, pypath + args)
-    if py3_compatible and extra_python is not None:
-        # Plan one more test for Python 3 compatible module
-        args[0] = extra_python
-        python_name = os.path.basename(extra_python)
-        plantestsuite_loadlist(name + "." + python_name, env, pypath + args)
 
 
 def get_env_torture_options():
@@ -208,3 +201,4 @@ smbget = binpath('smbget')
 rpcclient = binpath('rpcclient')
 smbcacls = binpath('smbcacls')
 smbcontrol = binpath('smbcontrol')
+smbstatus = binpath('smbstatus')
