@@ -329,7 +329,17 @@ Byte offset   Type     name                description
 #define SMB_FILE_NETWORK_OPEN_INFORMATION		1034
 #define SMB_FILE_ATTRIBUTE_TAG_INFORMATION		1035
 #define SMB_FILE_TRACKING_INFORMATION			1036
-#define SMB_FILE_MAXIMUM_INFORMATION			1037
+#define SMB_FILE_ID_BOTH_DIRECTORY_INFORMATION		1037
+#define SMB_FILE_ID_FULL_DIRECTORY_INFORMATION		1038
+#define SMB_FILE_VALID_DATA_LENGTH_INFORMATION		1039
+#define SMB_FILE_SHORT_NAME_INFORMATION			1040
+#define SMB_FILE_SFIO_RESERVE_INFORMATION		1044
+#define SMB_FILE_SFIO_VOLUME_INFORMATION		1045
+#define SMB_FILE_HARD_LINK_INFORMATION			1046
+#define SMB_FILE_NORMALIZED_NAME_INFORMATION		1048
+#define SMB_FILE_ID_GLOBAL_TX_DIRECTORY_INFORMATION	1050
+#define SMB_FILE_STANDARD_LINK_INFORMATION		1054
+#define SMB_FILE_MAXIMUM_INFORMATION			1055
 
 /* NT passthough levels for qfsinfo. */
 
