@@ -64,23 +64,23 @@ struct db_context *db_open(TALLOC_CTX *mem_ctx,
 			   uint64_t dbwrap_flags)
 {
 	struct db_context *result = NULL;
-	const char *sockname;
+	const char *base;
+	struct loadparm_context *lp_ctx = NULL;
 
 	if (!DBWRAP_LOCK_ORDER_VALID(lock_order)) {
 		errno = EINVAL;
 		return NULL;
 	}
 
-	if (tdb_flags & TDB_CLEAR_IF_FIRST) {
-		const char *base;
-		bool try_readonly = false;
+	base = strrchr_m(name, '/');
+	if (base != NULL) {
+		base++;
+	} else {
+		base = name;
+	}
 
-		base = strrchr_m(name, '/');
-		if (base != NULL) {
-			base += 1;
-		} else {
-			base = name;
-		}
+	if (tdb_flags & TDB_CLEAR_IF_FIRST) {
+		bool try_readonly = false;
 
 		if (dbwrap_flags & DBWRAP_FLAG_OPTIMIZE_READONLY_ACCESS) {
 			try_readonly = true;
@@ -97,17 +97,9 @@ struct db_context *db_open(TALLOC_CTX *mem_ctx,
 	}
 
 	if (tdb_flags & TDB_CLEAR_IF_FIRST) {
-		const char *base;
 		bool try_mutex = true;
 		bool require_mutex = false;
 
-		base = strrchr_m(name, '/');
-		if (base != NULL) {
-			base += 1;
-		} else {
-			base = name;
-		}
-
 		try_mutex = lp_parm_bool(-1, "dbwrap_tdb_mutexes", "*", try_mutex);
 		try_mutex = lp_parm_bool(-1, "dbwrap_tdb_mutexes", base, try_mutex);
 
@@ -134,70 +126,66 @@ struct db_context *db_open(TALLOC_CTX *mem_ctx,
 		}
 	}
 
-	sockname = lp_ctdbd_socket();
-
 	if (lp_clustering()) {
-		const char *partname;
+		const char *sockname;
 
+		sockname = lp_ctdbd_socket();
 		if (!socket_exist(sockname)) {
 			DEBUG(1, ("ctdb socket does not exist - is ctdb not "
 				  "running?\n"));
 			return NULL;
 		}
 
-		/* ctdb only wants the file part of the name */
-		partname = strrchr(name, '/');
-		if (partname) {
-			partname++;
-		} else {
-			partname = name;
-		}
 		/* allow ctdb for individual databases to be disabled */
-		if (lp_parm_bool(-1, "ctdb", partname, True)) {
+		if (lp_parm_bool(-1, "ctdb", base, true)) {
 			struct messaging_context *msg_ctx;
 			struct ctdbd_connection *conn;
 
+			/*
+			 * Initialize messaging before getting the ctdb
+			 * connection, as the ctdb connection requires messaging
+			 * to be initialized.
+			 */
+			msg_ctx = global_messaging_context();
+
 			conn = messaging_ctdb_connection();
 			if (conn == NULL) {
 				DBG_WARNING("No ctdb connection\n");
 				errno = EIO;
 				return NULL;
 			}
-			msg_ctx = server_messaging_context();
 
-			result = db_open_ctdb(mem_ctx, msg_ctx, partname,
+			result = db_open_ctdb(mem_ctx, msg_ctx, base,
 					      hash_size,
 					      tdb_flags, open_flags, mode,
 					      lock_order, dbwrap_flags);
 			if (result == NULL) {
-				DEBUG(0,("failed to attach to ctdb %s\n",
-					 partname));
+				DBG_ERR("failed to attach to ctdb %s\n", base);
 				if (errno == 0) {
 					errno = EIO;
 				}
 				return NULL;
 			}
+
+			return result;
 		}
 	}
 
-	if (result == NULL) {
-		struct loadparm_context *lp_ctx = loadparm_init_s3(mem_ctx, loadparm_s3_helpers());
+	lp_ctx = loadparm_init_s3(mem_ctx, loadparm_s3_helpers());
 
-		if (hash_size == 0) {
-			hash_size = lpcfg_tdb_hash_size(lp_ctx, name);
-		}
-		tdb_flags = lpcfg_tdb_flags(lp_ctx, tdb_flags);
-
-		result = dbwrap_local_open(
-			mem_ctx,
-			name,
-			hash_size,
-			tdb_flags,
-			open_flags,
-			mode,
-			lock_order,
-			dbwrap_flags);
-		talloc_unlink(mem_ctx, lp_ctx);
+	if (hash_size == 0) {
+		hash_size = lpcfg_tdb_hash_size(lp_ctx, name);
 	}
+	tdb_flags = lpcfg_tdb_flags(lp_ctx, tdb_flags);
+
+	result = dbwrap_local_open(mem_ctx,
+				   name,
+				   hash_size,
+				   tdb_flags,
+				   open_flags,
+				   mode,
+				   lock_order,
+				   dbwrap_flags);
+	talloc_unlink(mem_ctx, lp_ctx);
 	return result;
 }
