@@ -71,6 +71,22 @@ struct smb_filename *synthetic_smb_fname(TALLOC_CTX *mem_ctx,
 	return cp_smb_filename(mem_ctx, &smb_fname_loc);
 }
 
+/**
+ * Utility function used by VFS calls that must *NOT* operate
+ * on a stream filename, only the base_name.
+ */
+struct smb_filename *cp_smb_filename_nostream(TALLOC_CTX *mem_ctx,
+					const struct smb_filename *smb_fname_in)
+{
+	struct smb_filename *smb_fname = cp_smb_filename(mem_ctx,
+							smb_fname_in);
+	if (smb_fname == NULL) {
+		return NULL;
+	}
+	TALLOC_FREE(smb_fname->stream_name);
+	return smb_fname;
+}
+
 /**
  * There are a few legitimate users of this.
  */
@@ -223,10 +239,7 @@ struct smb_filename *cp_smb_filename(TALLOC_CTX *mem_ctx,
 	return out;
 }
 
-/****************************************************************************
- Simple check to determine if the filename is a stream.
- ***************************************************************************/
-bool is_ntfs_stream_smb_fname(const struct smb_filename *smb_fname)
+static void assert_valid_stream_smb_fname(const struct smb_filename *smb_fname)
 {
 	/* stream_name must always be NULL if there is no stream. */
 	if (smb_fname->stream_name) {
@@ -234,10 +247,50 @@ bool is_ntfs_stream_smb_fname(const struct smb_filename *smb_fname)
 	}
 
 	if (smb_fname->flags & SMB_FILENAME_POSIX_PATH) {
+		SMB_ASSERT(smb_fname->stream_name == NULL);
+	}
+}
+
+/****************************************************************************
+ Simple check to determine if a smb_fname is a real named stream or the
+ default stream.
+ ***************************************************************************/
+
+bool is_ntfs_stream_smb_fname(const struct smb_filename *smb_fname)
+{
+	assert_valid_stream_smb_fname(smb_fname);
+
+	if (smb_fname->stream_name == NULL) {
 		return false;
 	}
 
-	return smb_fname->stream_name != NULL;
+	return true;
+}
+
+/****************************************************************************
+ Simple check to determine if a smb_fname is pointing to a normal file or
+ a named stream that is not the default stream "::$DATA".
+
+  foo           -> false
+  foo::$DATA    -> false
+  foo:bar       -> true
+  foo:bar:$DATA -> true
+
+ ***************************************************************************/
+
+bool is_named_stream(const struct smb_filename *smb_fname)
+{
+	assert_valid_stream_smb_fname(smb_fname);
+
+	if (smb_fname->stream_name == NULL) {
+		return false;
+	}
+
+	if (strequal_m(smb_fname->stream_name, "::$DATA")) {
+		return false;
+	}
+
+	return true;
 }
 
 /****************************************************************************
@@ -245,11 +298,13 @@ bool is_ntfs_stream_smb_fname(const struct smb_filename *smb_fname)
  ***************************************************************************/
 bool is_ntfs_default_stream_smb_fname(const struct smb_filename *smb_fname)
 {
-	if (!is_ntfs_stream_smb_fname(smb_fname)) {
+	assert_valid_stream_smb_fname(smb_fname);
+
+	if (smb_fname->stream_name == NULL) {
 		return false;
 	}
 
-	return strcasecmp_m(smb_fname->stream_name, "::$DATA") == 0;
+	return strequal_m(smb_fname->stream_name, "::$DATA");
 }
 
 /****************************************************************************
