@@ -18,6 +18,7 @@
  */
 
 #include "replace.h"
+#include "util/util.h"
 #include "system/network.h"
 #include "system/filesys.h"
 #include "system/dir.h"
@@ -212,6 +213,7 @@ static int messaging_dgm_out_create(TALLOC_CTX *mem_ctx,
 	struct sockaddr_un addr = { .sun_family = AF_UNIX };
 	int ret = ENOMEM;
 	int out_pathlen;
+	char addr_buf[sizeof(addr.sun_path) + (3 * sizeof(unsigned) + 2)];
 
 	out = talloc(mem_ctx, struct messaging_dgm_out);
 	if (out == NULL) {
@@ -224,7 +226,7 @@ static int messaging_dgm_out_create(TALLOC_CTX *mem_ctx,
 		.cookie = 1
 	};
 
-	out_pathlen = snprintf(addr.sun_path, sizeof(addr.sun_path),
+	out_pathlen = snprintf(addr_buf, sizeof(addr_buf),
 			       "%s/%u", ctx->socket_dir.buf, (unsigned)pid);
 	if (out_pathlen < 0) {
 		goto errno_fail;
@@ -234,6 +236,8 @@ static int messaging_dgm_out_create(TALLOC_CTX *mem_ctx,
 		goto fail;
 	}
 
+	memcpy(addr.sun_path, addr_buf, out_pathlen + 1);
+
 	out->queue = tevent_queue_create(out, addr.sun_path);
 	if (out->queue == NULL) {
 		ret = ENOMEM;
@@ -319,6 +323,13 @@ static int messaging_dgm_out_get(struct messaging_dgm_context *ctx, pid_t pid,
 	}
 
 	messaging_dgm_out_rearm_idle_timer(out);
+	/*
+	 * shouldn't be possible, should be set if messaging_dgm_out_create
+	 * succeeded. This check is to satisfy static checker
+	 */
+	if (out == NULL) {
+		return EINVAL;
+	}
 
 	*pout = out;
 	return 0;
@@ -361,7 +372,7 @@ static ssize_t messaging_dgm_sendmsg(int sock,
 		msghdr_prep_fds(&msg, buf, fdlen, fds, num_fds);
 
 		do {
-			ret = sendmsg(sock, &msg, MSG_NOSIGNAL);
+			ret = sendmsg(sock, &msg, 0);
 		} while ((ret == -1) && (errno == EINTR));
 	}
 
@@ -539,9 +550,35 @@ static void messaging_dgm_out_threaded_job(void *private_data)
 	struct iovec iov = { .iov_base = state->buf,
 			     .iov_len = talloc_get_size(state->buf) };
 	size_t num_fds = talloc_array_length(state->fds);
+	int msec = 1;
 
-	state->sent = messaging_dgm_sendmsg(state->sock, &iov, 1,
+	while (true) {
+		int ret;
+
+		state->sent = messaging_dgm_sendmsg(state->sock, &iov, 1,
 					    state->fds, num_fds, &state->err);
+
+		if (state->sent != -1) {
+			return;
+		}
+		if (state->err != ENOBUFS) {
+			return;
+		}
+
+		/*
+		 * ENOBUFS is the FreeBSD way of saying "Try
+		 * again". We have to do polling.
+		 */
+		do {
+			ret = poll(NULL, 0, msec);
+		} while ((ret == -1) && (errno == EINTR));
+
+		/*
+		 * Exponential backoff up to once a second
+		 */
+		msec *= 2;
+		msec = MIN(msec, 1000);
+	}
 }
 
 /*
@@ -617,6 +654,15 @@ static int messaging_dgm_out_send_fragment(
 			return 0;
 		}
 
+		if (err == ENOBUFS) {
+			/*
+			 * FreeBSD's way of telling us the dst socket
+			 * is full. EWOULDBLOCK makes us spawn a
+			 * polling helper thread.
+			 */
+			err = EWOULDBLOCK;
+		}
+
 		if (err != EWOULDBLOCK) {
 			return err;
 		}
@@ -1028,7 +1074,7 @@ int messaging_dgm_init(struct tevent_context *ev,
 
 	ctx->have_dgm_context = &have_dgm_context;
 
-	ret = pthreadpool_tevent_init(ctx, 0, &ctx->pool);
+	ret = pthreadpool_tevent_init(ctx, UINT_MAX, &ctx->pool);
 	if (ret != 0) {
 		DBG_WARNING("pthreadpool_tevent_init failed: %s\n",
 			    strerror(ret));
@@ -1100,6 +1146,88 @@ static int messaging_dgm_context_destructor(struct messaging_dgm_context *c)
 	return 0;
 }
 
+static void messaging_dgm_validate(struct messaging_dgm_context *ctx)
+{
+#ifdef DEVELOPER
+	pid_t pid = getpid();
+	struct sockaddr_storage addr;
+	socklen_t addrlen = sizeof(addr);
+	struct sockaddr_un *un_addr;
+	struct sun_path_buf pathbuf;
+	struct stat st1, st2;
+	int ret;
+
+	/*
+	 * Protect against using the wrong messaging context after a
+	 * fork without reinit_after_fork.
+	 */
+
+	ret = getsockname(ctx->sock, (struct sockaddr *)&addr, &addrlen);
+	if (ret == -1) {
+		DBG_ERR("getsockname failed: %s\n", strerror(errno));
+		goto fail;
+	}
+	if (addr.ss_family != AF_UNIX) {
+		DBG_ERR("getsockname returned family %d\n",
+			(int)addr.ss_family);
+		goto fail;
+	}
+	un_addr = (struct sockaddr_un *)&addr;
+
+	ret = snprintf(pathbuf.buf, sizeof(pathbuf.buf),
+		       "%s/%u", ctx->socket_dir.buf, (unsigned)pid);
+	if (ret < 0) {
+		DBG_ERR("snprintf failed: %s\n", strerror(errno));
+		goto fail;
+	}
+	if ((size_t)ret >= sizeof(pathbuf.buf)) {
+		DBG_ERR("snprintf returned %d chars\n", (int)ret);
+		goto fail;
+	}
+
+	if (strcmp(pathbuf.buf, un_addr->sun_path) != 0) {
+		DBG_ERR("sockname wrong: Expected %s, got %s\n",
+			pathbuf.buf, un_addr->sun_path);
+		goto fail;
+	}
+
+	ret = snprintf(pathbuf.buf, sizeof(pathbuf.buf),
+		       "%s/%u", ctx->lockfile_dir.buf, (unsigned)pid);
+	if (ret < 0) {
+		DBG_ERR("snprintf failed: %s\n", strerror(errno));
+		goto fail;
+	}
+	if ((size_t)ret >= sizeof(pathbuf.buf)) {
+		DBG_ERR("snprintf returned %d chars\n", (int)ret);
+		goto fail;
+	}
+
+	ret = stat(pathbuf.buf, &st1);
+	if (ret == -1) {
+		DBG_ERR("stat failed: %s\n", strerror(errno));
+		goto fail;
+	}
+	ret = fstat(ctx->lockfile_fd, &st2);
+	if (ret == -1) {
+		DBG_ERR("fstat failed: %s\n", strerror(errno));
+		goto fail;
+	}
+
+	if ((st1.st_dev != st2.st_dev) || (st1.st_ino != st2.st_ino)) {
+		DBG_ERR("lockfile differs, expected (%d/%d), got (%d/%d)\n",
+			(int)st2.st_dev, (int)st2.st_ino,
+			(int)st1.st_dev, (int)st1.st_ino);
+		goto fail;
+	}
+
+	return;
+fail:
+	abort();
+#else
+	return;
+#endif
+}
+
 static void messaging_dgm_recv(struct messaging_dgm_context *ctx,
 			       struct tevent_context *ev,
 			       uint8_t *msg, size_t msg_len,
@@ -1123,6 +1251,9 @@ static void messaging_dgm_read_handler(struct tevent_context *ev,
 	size_t msgbufsize = msghdr_prep_recv_fds(NULL, NULL, 0, INT8_MAX);
 	uint8_t msgbuf[msgbufsize];
 	uint8_t buf[MESSAGING_DGM_FRAGMENT_LENGTH];
+	size_t num_fds;
+
+	messaging_dgm_validate(ctx);
 
 	if ((flags & TEVENT_FD_READ) == 0) {
 		return;
@@ -1156,8 +1287,12 @@ static void messaging_dgm_read_handler(struct tevent_context *ev,
 		return;
 	}
 
-	{
-		size_t num_fds = msghdr_extract_fds(&msg, NULL, 0);
+	num_fds = msghdr_extract_fds(&msg, NULL, 0);
+	if (num_fds == 0) {
+		int fds[1];
+
+		messaging_dgm_recv(ctx, ev, buf, received, fds, 0);
+	} else {
 		size_t i;
 		int fds[num_fds];
 
@@ -1175,7 +1310,6 @@ static void messaging_dgm_read_handler(struct tevent_context *ev,
 
 		messaging_dgm_recv(ctx, ev, buf, received, fds, num_fds);
 	}
-
 }
 
 static int messaging_dgm_in_msg_destructor(struct messaging_dgm_in_msg *m)
@@ -1293,11 +1427,15 @@ int messaging_dgm_send(pid_t pid,
 	struct messaging_dgm_context *ctx = global_dgm_context;
 	struct messaging_dgm_out *out;
 	int ret;
+	unsigned retries = 0;
 
 	if (ctx == NULL) {
 		return ENOTCONN;
 	}
 
+	messaging_dgm_validate(ctx);
+
+again:
 	ret = messaging_dgm_out_get(ctx, pid, &out);
 	if (ret != 0) {
 		return ret;
@@ -1307,6 +1445,20 @@ int messaging_dgm_send(pid_t pid,
 
 	ret = messaging_dgm_out_send_fragmented(ctx->ev, out, iov, iovlen,
 						fds, num_fds);
+	if (ret == ECONNREFUSED) {
+		/*
+		 * We cache outgoing sockets. If the receiver has
+		 * closed and re-opened the socket since our last
+		 * message, we get connection refused. Retry.
+		 */
+
+		TALLOC_FREE(out);
+
+		if (retries < 5) {
+			retries += 1;
+			goto again;
+		}
+	}
 	return ret;
 }
 
@@ -1314,6 +1466,7 @@ static int messaging_dgm_read_unique(int fd, uint64_t *punique)
 {
 	char buf[25];
 	ssize_t rw_ret;
+	int error = 0;
 	unsigned long long unique;
 	char *endptr;
 
@@ -1323,13 +1476,11 @@ static int messaging_dgm_read_unique(int fd, uint64_t *punique)
 	}
 	buf[rw_ret] = '\0';
 
-	unique = strtoull(buf, &endptr, 10);
-	if ((unique == 0) && (errno == EINVAL)) {
-		return EINVAL;
-	}
-	if ((unique == ULLONG_MAX) && (errno == ERANGE)) {
-		return ERANGE;
+	unique = smb_strtoull(buf, &endptr, 10, &error, SMB_STR_STANDARD);
+	if (error != 0) {
+		return error;
 	}
+
 	if (endptr[0] != '\n') {
 		return EINVAL;
 	}
@@ -1347,6 +1498,8 @@ int messaging_dgm_get_unique(pid_t pid, uint64_t *unique)
 		return EBADF;
 	}
 
+	messaging_dgm_validate(ctx);
+
 	if (pid == getpid()) {
 		/*
 		 * Protect against losing our own lock
@@ -1378,7 +1531,9 @@ int messaging_dgm_cleanup(pid_t pid)
 	struct messaging_dgm_context *ctx = global_dgm_context;
 	struct sun_path_buf lockfile_name, socket_name;
 	int fd, len, ret;
-	struct flock lck = {};
+	struct flock lck = {
+		.l_pid = 0,
+	};
 
 	if (ctx == NULL) {
 		return ENOTCONN;
@@ -1436,18 +1591,47 @@ int messaging_dgm_cleanup(pid_t pid)
 	return 0;
 }
 
+static int messaging_dgm_wipe_fn(pid_t pid, void *private_data)
+{
+	pid_t *our_pid = (pid_t *)private_data;
+	int ret;
+
+	if (pid == *our_pid) {
+		/*
+		 * fcntl(F_GETLK) will succeed for ourselves, we hold
+		 * that lock ourselves.
+		 */
+		return 0;
+	}
+
+	ret = messaging_dgm_cleanup(pid);
+	DEBUG(10, ("messaging_dgm_cleanup(%lu) returned %s\n",
+		   (unsigned long)pid, ret ? strerror(ret) : "ok"));
+
+	return 0;
+}
+
 int messaging_dgm_wipe(void)
+{
+	pid_t pid = getpid();
+	messaging_dgm_forall(messaging_dgm_wipe_fn, &pid);
+	return 0;
+}
+
+int messaging_dgm_forall(int (*fn)(pid_t pid, void *private_data),
+			 void *private_data)
 {
 	struct messaging_dgm_context *ctx = global_dgm_context;
 	DIR *msgdir;
 	struct dirent *dp;
-	pid_t our_pid = getpid();
-	int ret;
+	int error = 0;
 
 	if (ctx == NULL) {
 		return ENOTCONN;
 	}
 
+	messaging_dgm_validate(ctx);
+
 	/*
 	 * We scan the socket directory and not the lock directory. Otherwise
 	 * we would race against messaging_dgm_lockfile_create's open(O_CREAT)
@@ -1461,25 +1645,20 @@ int messaging_dgm_wipe(void)
 
 	while ((dp = readdir(msgdir)) != NULL) {
 		unsigned long pid;
+		int ret;
 
-		pid = strtoul(dp->d_name, NULL, 10);
-		if (pid == 0) {
+		pid = smb_strtoul(dp->d_name, NULL, 10, &error, SMB_STR_STANDARD);
+		if ((pid == 0) || (error != 0)) {
 			/*
 			 * . and .. and other malformed entries
 			 */
 			continue;
 		}
-		if ((pid_t)pid == our_pid) {
-			/*
-			 * fcntl(F_GETLK) will succeed for ourselves, we hold
-			 * that lock ourselves.
-			 */
-			continue;
-		}
 
-		ret = messaging_dgm_cleanup(pid);
-		DEBUG(10, ("messaging_dgm_cleanup(%lu) returned %s\n",
-			   pid, ret ? strerror(ret) : "ok"));
+		ret = fn(pid, private_data);
+		if (ret != 0) {
+			break;
+		}
 	}
 	closedir(msgdir);
 
@@ -1530,8 +1709,18 @@ struct messaging_dgm_fde *messaging_dgm_register_tevent_context(
 	}
 
 	for (fde_ev = ctx->fde_evs; fde_ev != NULL; fde_ev = fde_ev->next) {
-		if ((fde_ev->ev == ev) &&
-		    (tevent_fd_get_flags(fde_ev->fde) != 0)) {
+		if (tevent_fd_get_flags(fde_ev->fde) == 0) {
+			/*
+			 * If the event context got deleted,
+			 * tevent_fd_get_flags() will return 0
+			 * for the stale fde.
+			 *
+			 * In that case we should not
+			 * use fde_ev->ev anymore.
+			 */
+			continue;
+		}
+		if (fde_ev->ev == ev) {
 			break;
 		}
 	}
