@@ -68,7 +68,7 @@ static void convert_USER_INFO_X_to_samr_user_info21(struct USER_INFO_X *infoX,
 		fields_present |= SAMR_FIELD_DESCRIPTION;
 	}
 	if (infoX->usriX_password_age) {
-		fields_present |= SAMR_FIELD_FORCE_PWD_CHANGE;
+		fields_present |= SAMR_FIELD_EXPIRED_FLAG;
 	}
 	if (infoX->usriX_full_name) {
 		fields_present |= SAMR_FIELD_FULL_NAME;
@@ -385,7 +385,7 @@ WERROR NetUserAdd_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(user_handle);
 
 	if (!r->in.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	switch (r->in.level) {
@@ -400,7 +400,7 @@ WERROR NetUserAdd_r(struct libnetapi_ctx *ctx,
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -466,7 +466,7 @@ WERROR NetUserAdd_r(struct libnetapi_ctx *ctx,
 	}
 
 	if (!(user_info->info16.acct_flags & ACB_NORMAL)) {
-		werr = WERR_INVALID_PARAM;
+		werr = WERR_INVALID_PARAMETER;
 		goto done;
 	}
 
@@ -552,7 +552,7 @@ WERROR NetUserDel_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(user_handle);
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 
 	if (!W_ERROR_IS_OK(werr)) {
@@ -1201,9 +1201,6 @@ static NTSTATUS libnetapi_samr_lookup_user_map_USER_INFO(TALLOC_CTX *mem_ctx,
 	}
 
 	switch (level) {
-		case 0:
-			/* already returned above */
-			break;
 		case 1:
 			status = info21_to_USER_INFO_1(mem_ctx, info21, &info1);
 			NT_STATUS_NOT_OK_RETURN(status);
@@ -1300,7 +1297,7 @@ WERROR NetUserEnum_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(builtin_handle);
 
 	if (!r->out.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	*r->out.buffer = NULL;
@@ -1318,11 +1315,11 @@ WERROR NetUserEnum_r(struct libnetapi_ctx *ctx,
 		case 23:
 			break;
 		default:
-			return WERR_UNKNOWN_LEVEL;
+			return WERR_INVALID_LEVEL;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -1464,7 +1461,7 @@ static WERROR convert_samr_dispinfo_to_NET_DISPLAY_USER(TALLOC_CTX *mem_ctx,
 			info->entries[i].idx;
 
 		if (!user[i].usri1_name) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 	}
 
@@ -1506,7 +1503,7 @@ static WERROR convert_samr_dispinfo_to_NET_DISPLAY_MACHINE(TALLOC_CTX *mem_ctx,
 			info->entries[i].idx;
 
 		if (!machine[i].usri2_name) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 	}
 
@@ -1548,7 +1545,7 @@ static WERROR convert_samr_dispinfo_to_NET_DISPLAY_GROUP(TALLOC_CTX *mem_ctx,
 			info->entries[i].idx;
 
 		if (!group[i].grpi3_name) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 	}
 
@@ -1591,7 +1588,7 @@ static WERROR convert_samr_dispinfo_to_NET_DISPLAY(TALLOC_CTX *mem_ctx,
 			break;
 	}
 
-	return WERR_UNKNOWN_LEVEL;
+	return WERR_INVALID_LEVEL;
 }
 
 /****************************************************************
@@ -1626,11 +1623,11 @@ WERROR NetQueryDisplayInformation_r(struct libnetapi_ctx *ctx,
 		case 3:
 			break;
 		default:
-			return WERR_UNKNOWN_LEVEL;
+			return WERR_INVALID_LEVEL;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -1744,7 +1741,7 @@ WERROR NetUserGetInfo_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(user_handle);
 
 	if (!r->out.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	switch (r->in.level) {
@@ -1759,12 +1756,12 @@ WERROR NetUserGetInfo_r(struct libnetapi_ctx *ctx,
 		case 23:
 			break;
 		default:
-			werr = WERR_UNKNOWN_LEVEL;
+			werr = WERR_INVALID_LEVEL;
 			goto done;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -1882,7 +1879,7 @@ WERROR NetUserSetInfo_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(user_handle);
 
 	if (!r->in.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	switch (r->in.level) {
@@ -1931,12 +1928,12 @@ WERROR NetUserSetInfo_r(struct libnetapi_ctx *ctx,
 			werr = WERR_NOT_SUPPORTED;
 			goto done;
 		default:
-			werr = WERR_UNKNOWN_LEVEL;
+			werr = WERR_INVALID_LEVEL;
 			goto done;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -2374,7 +2371,7 @@ WERROR NetUserModalsGet_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(domain_handle);
 
 	if (!r->out.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	switch (r->in.level) {
@@ -2390,12 +2387,12 @@ WERROR NetUserModalsGet_r(struct libnetapi_ctx *ctx,
 			access_mask |= SAMR_DOMAIN_ACCESS_LOOKUP_INFO_1;
 			break;
 		default:
-			werr = WERR_UNKNOWN_LEVEL;
+			werr = WERR_INVALID_LEVEL;
 			goto done;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -2846,7 +2843,7 @@ WERROR NetUserModalsSet_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(domain_handle);
 
 	if (!r->in.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	switch (r->in.level) {
@@ -2875,12 +2872,12 @@ WERROR NetUserModalsSet_r(struct libnetapi_ctx *ctx,
 			werr = WERR_NOT_SUPPORTED;
 			break;
 		default:
-			werr = WERR_UNKNOWN_LEVEL;
+			werr = WERR_INVALID_LEVEL;
 			goto done;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -2999,7 +2996,7 @@ WERROR NetUserGetGroups_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(domain_handle);
 
 	if (!r->out.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	*r->out.buffer = NULL;
@@ -3011,11 +3008,11 @@ WERROR NetUserGetGroups_r(struct libnetapi_ctx *ctx,
 		case 1:
 			break;
 		default:
-			return WERR_UNKNOWN_LEVEL;
+			return WERR_INVALID_LEVEL;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -3090,7 +3087,7 @@ WERROR NetUserGetGroups_r(struct libnetapi_ctx *ctx,
 
 	rids = talloc_array(ctx, uint32_t, rid_array->count);
 	if (!rids) {
-		werr = WERR_NOMEM;
+		werr = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
@@ -3194,7 +3191,7 @@ WERROR NetUserSetGroups_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(group_handle);
 
 	if (!r->in.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	switch (r->in.level) {
@@ -3202,11 +3199,11 @@ WERROR NetUserSetGroups_r(struct libnetapi_ctx *ctx,
 		case 1:
 			break;
 		default:
-			return WERR_UNKNOWN_LEVEL;
+			return WERR_INVALID_LEVEL;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -3277,7 +3274,7 @@ WERROR NetUserSetGroups_r(struct libnetapi_ctx *ctx,
 
 	lsa_names = talloc_array(ctx, struct lsa_String, r->in.num_entries);
 	if (!lsa_names) {
-		werr = WERR_NOMEM;
+		werr = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
@@ -3348,7 +3345,7 @@ WERROR NetUserSetGroups_r(struct libnetapi_ctx *ctx,
 			if (!add_rid_to_array_unique(ctx,
 						     member_rids[i],
 						     &add_rids, &num_add_rids)) {
-				werr = WERR_GENERAL_FAILURE;
+				werr = WERR_GEN_FAILURE;
 				goto done;
 			}
 		}
@@ -3368,7 +3365,7 @@ WERROR NetUserSetGroups_r(struct libnetapi_ctx *ctx,
 			if (!add_rid_to_array_unique(ctx,
 						     rid_array->rids[k].rid,
 						     &del_rids, &num_del_rids)) {
-				werr = WERR_GENERAL_FAILURE;
+				werr = WERR_GEN_FAILURE;
 				goto done;
 			}
 		}
@@ -3531,7 +3528,7 @@ WERROR NetUserGetLocalGroups_r(struct libnetapi_ctx *ctx,
 	ZERO_STRUCT(domain_handle);
 
 	if (!r->out.buffer) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	*r->out.buffer = NULL;
@@ -3543,11 +3540,11 @@ WERROR NetUserGetLocalGroups_r(struct libnetapi_ctx *ctx,
 		case 1:
 			break;
 		default:
-			return WERR_UNKNOWN_LEVEL;
+			return WERR_INVALID_LEVEL;
 	}
 
 	werr = libnetapi_open_pipe(ctx, r->in.server_name,
-				   &ndr_table_samr.syntax_id,
+				   &ndr_table_samr,
 				   &pipe_cli);
 	if (!W_ERROR_IS_OK(werr)) {
 		goto done;
@@ -3633,20 +3630,20 @@ WERROR NetUserGetLocalGroups_r(struct libnetapi_ctx *ctx,
 	}
 
 	if (!sid_compose(&user_sid, domain_sid, user_rids.ids[0])) {
-		werr = WERR_NOMEM;
+		werr = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
 	sid_array.num_sids = rid_array->count + 1;
 	sid_array.sids = talloc_array(ctx, struct lsa_SidPtr, sid_array.num_sids);
 	if (!sid_array.sids) {
-		werr = WERR_NOMEM;
+		werr = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
 	sid_array.sids[0].sid = dom_sid_dup(ctx, &user_sid);
 	if (!sid_array.sids[0].sid) {
-		werr = WERR_NOMEM;
+		werr = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
@@ -3654,13 +3651,13 @@ WERROR NetUserGetLocalGroups_r(struct libnetapi_ctx *ctx,
 		struct dom_sid sid;
 
 		if (!sid_compose(&sid, domain_sid, rid_array->rids[i].rid)) {
-			werr = WERR_NOMEM;
+			werr = WERR_NOT_ENOUGH_MEMORY;
 			goto done;
 		}
 
 		sid_array.sids[i+1].sid = dom_sid_dup(ctx, &sid);
 		if (!sid_array.sids[i+1].sid) {
-			werr = WERR_NOMEM;
+			werr = WERR_NOT_ENOUGH_MEMORY;
 			goto done;
 		}
 	}
@@ -3682,7 +3679,7 @@ WERROR NetUserGetLocalGroups_r(struct libnetapi_ctx *ctx,
 	for (i=0; i < domain_rids.count; i++) {
 		if (!add_rid_to_array_unique(ctx, domain_rids.ids[i],
 					     &rids, &num_rids)) {
-			werr = WERR_NOMEM;
+			werr = WERR_NOT_ENOUGH_MEMORY;
 			goto done;
 		}
 	}
@@ -3704,7 +3701,7 @@ WERROR NetUserGetLocalGroups_r(struct libnetapi_ctx *ctx,
 	for (i=0; i < builtin_rids.count; i++) {
 		if (!add_rid_to_array_unique(ctx, builtin_rids.ids[i],
 					     &rids, &num_rids)) {
-			werr = WERR_NOMEM;
+			werr = WERR_NOT_ENOUGH_MEMORY;
 			goto done;
 		}
 	}
