@@ -32,16 +32,11 @@
  * Generate an inode number from file name for those things that need it
  */
 
-static ino_t
-generate_inode(SMBCCTX *context,
-               const char *name)
+static ino_t generate_inode(const char *name)
 {
-	if (!context || !context->internal->initialized) {
-		errno = EINVAL;
-		return -1;
+	if (name == NULL) {
+		return (ino_t)-1;
 	}
-
-	if (!*name) return 2; /* FIXME, why 2 ??? */
 	return (ino_t)str_checksum(name);
 }
 
@@ -50,15 +45,16 @@ generate_inode(SMBCCTX *context,
  * fstat below.
  */
 
-static int
-setup_stat(SMBCCTX *context,
-           struct stat *st,
-           const char *fname,
-           off_t size,
-           int mode)
+void setup_stat(struct stat *st,
+		const char *fname,
+		off_t size,
+		int mode,
+		ino_t ino,
+		dev_t dev,
+		struct timespec access_time_ts,
+		struct timespec change_time_ts,
+		struct timespec write_time_ts)
 {
-	TALLOC_CTX *frame = talloc_stackframe();
-
 	st->st_mode = 0;
 
 	if (IS_DOS_DIR(mode)) {
@@ -67,10 +63,18 @@ setup_stat(SMBCCTX *context,
 		st->st_mode = SMBC_FILE_MODE;
 	}
 
-	if (IS_DOS_ARCHIVE(mode)) st->st_mode |= S_IXUSR;
-	if (IS_DOS_SYSTEM(mode)) st->st_mode |= S_IXGRP;
-	if (IS_DOS_HIDDEN(mode)) st->st_mode |= S_IXOTH;
-	if (!IS_DOS_READONLY(mode)) st->st_mode |= S_IWUSR;
+	if (IS_DOS_ARCHIVE(mode)) {
+		st->st_mode |= S_IXUSR;
+	}
+	if (IS_DOS_SYSTEM(mode)) {
+		st->st_mode |= S_IXGRP;
+	}
+	if (IS_DOS_HIDDEN(mode)) {
+		st->st_mode |= S_IXOTH;
+	}
+	if (!IS_DOS_READONLY(mode)) {
+		st->st_mode |= S_IWUSR;
+	}
 
 	st->st_size = size;
 #ifdef HAVE_STAT_ST_BLKSIZE
@@ -91,12 +95,53 @@ setup_stat(SMBCCTX *context,
 		st->st_nlink = 1;
 	}
 
-	if (st->st_ino == 0) {
-		st->st_ino = generate_inode(context, fname);
+	if (ino != 0) {
+		st->st_ino = ino;
+	} else {
+		st->st_ino = generate_inode(fname);
 	}
 
-	TALLOC_FREE(frame);
-	return True;  /* FIXME: Is this needed ? */
+	st->st_dev = dev;
+	st->st_atime = convert_timespec_to_time_t(access_time_ts);
+	st->st_ctime = convert_timespec_to_time_t(change_time_ts);
+	st->st_mtime = convert_timespec_to_time_t(write_time_ts);
+}
+
+void setup_stat_from_stat_ex(const struct stat_ex *stex,
+			     const char *fname,
+			     struct stat *st)
+{
+	st->st_atime = convert_timespec_to_time_t(stex->st_ex_atime);
+	st->st_ctime = convert_timespec_to_time_t(stex->st_ex_ctime);
+	st->st_mtime = convert_timespec_to_time_t(stex->st_ex_mtime);
+
+	st->st_mode = stex->st_ex_mode;
+	st->st_size = stex->st_ex_size;
+#ifdef HAVE_STAT_ST_BLKSIZE
+	st->st_blksize = 512;
+#endif
+#ifdef HAVE_STAT_ST_BLOCKS
+	st->st_blocks = (st->st_size + 511) / 512;
+#endif
+#ifdef HAVE_STRUCT_STAT_ST_RDEV
+	st->st_rdev = 0;
+#endif
+	st->st_uid = stex->st_ex_uid;
+	st->st_gid = stex->st_ex_gid;
+
+	st->st_nlink = stex->st_ex_nlink;
+
+	if (stex->st_ex_ino == 0) {
+		st->st_ino = 0;
+		if (fname != NULL) {
+			st->st_ino = generate_inode(fname);
+		}
+	} else {
+		st->st_ino = stex->st_ex_ino;
+	}
+
+	st->st_dev = stex->st_ex_dev;
+
 }
 
 /*
@@ -115,13 +160,7 @@ SMBC_stat_ctx(SMBCCTX *context,
 	char *password = NULL;
 	char *workgroup = NULL;
 	char *path = NULL;
-	struct timespec write_time_ts;
-        struct timespec access_time_ts;
-        struct timespec change_time_ts;
-	off_t size = 0;
-	uint16_t mode = 0;
 	uint16_t port = 0;
-	SMB_INO_T ino = 0;
 	TALLOC_CTX *frame = talloc_stackframe();
 
 	if (!context || !context->internal->initialized) {
@@ -170,26 +209,12 @@ SMBC_stat_ctx(SMBCCTX *context,
 		return -1;  /* errno set by SMBC_server */
 	}
 
-	if (!SMBC_getatr(context, srv, path, &mode, &size,
-			 NULL,
-                         &access_time_ts,
-                         &write_time_ts,
-                         &change_time_ts,
-                         &ino)) {
+	if (!SMBC_getatr(context, srv, path, st)) {
 		errno = SMBC_errno(context, srv->cli);
 		TALLOC_FREE(frame);
 		return -1;
 	}
 
-	st->st_ino = ino;
-
-	setup_stat(context, st, fname, size, mode);
-
-	st->st_atime = convert_timespec_to_time_t(access_time_ts);
-	st->st_ctime = convert_timespec_to_time_t(change_time_ts);
-	st->st_mtime = convert_timespec_to_time_t(write_time_ts);
-	st->st_dev   = srv->dev;
-
 	TALLOC_FREE(frame);
 	return 0;
 }
@@ -286,14 +311,15 @@ SMBC_fstat_ctx(SMBCCTX *context,
 		write_time_ts = convert_time_t_to_timespec(write_time);
 	}
 
-	st->st_ino = ino;
-
-	setup_stat(context, st, file->fname, size, mode);
-
-	st->st_atime = convert_timespec_to_time_t(access_time_ts);
-	st->st_ctime = convert_timespec_to_time_t(change_time_ts);
-	st->st_mtime = convert_timespec_to_time_t(write_time_ts);
-	st->st_dev = file->srv->dev;
+	setup_stat(st,
+		path,
+		size,
+		mode,
+		ino,
+		file->srv->dev,
+		access_time_ts,
+		change_time_ts,
+		write_time_ts);
 
 	TALLOC_FREE(frame);
 	return 0;
