@@ -31,7 +31,7 @@
 #include <linux/falloc.h>
 #endif
 
-#if defined(HAVE_OPENAT) && defined(USE_LINUX_THREAD_CREDENTIALS)
+#if defined(HAVE_OPENAT) && defined(HAVE_LINUX_THREAD_CREDENTIALS)
 
 /*
  * We must have openat() to do any thread-based
@@ -50,7 +50,7 @@ struct aio_open_private_data {
 	bool in_progress;
 	const char *fname;
 	char *dname;
-	struct smbd_server_connection *sconn;
+	connection_struct *conn;
 	const struct security_unix_token *ux_tok;
 	uint64_t initial_allocation_size;
 	/* Returns. */
@@ -61,6 +61,8 @@ struct aio_open_private_data {
 /* List of outstanding requests we have. */
 static struct aio_open_private_data *open_pd_list;
 
+static void aio_open_do(struct aio_open_private_data *opd);
+
 /************************************************************************
  Find the open private data by mid.
 ***********************************************************************/
@@ -93,9 +95,28 @@ static void aio_open_handle_completion(struct tevent_req *subreq)
 	ret = pthreadpool_tevent_job_recv(subreq);
 	TALLOC_FREE(subreq);
 	if (ret != 0) {
-		smb_panic("aio_open_handle_completion");
-		/* notreached. */
-		return;
+		bool ok;
+
+		if (ret != EAGAIN) {
+			smb_panic("aio_open_handle_completion");
+			/* notreached. */
+			return;
+		}
+		/*
+		 * Make sure we run as the user again
+		 */
+		ok = change_to_user(opd->conn, opd->conn->vuid);
+		if (!ok) {
+			smb_panic("Can't change to user");
+			return;
+		}
+		/*
+		 * If we get EAGAIN from pthreadpool_tevent_job_recv() this
+		 * means the lower level pthreadpool failed to create a new
+		 * thread. Fallback to sync processing in that case to allow
+		 * some progress for the client.
+		 */
+		aio_open_do(opd);
 	}
 
 	DEBUG(10,("aio_open_handle_completion: mid %llu "
@@ -111,7 +132,7 @@ static void aio_open_handle_completion(struct tevent_req *subreq)
 	 * to find the correct connection for a fsp.
 	 * For now we only have one connection, so this is correct...
 	 */
-	xconn = opd->sconn->client->connections;
+	xconn = opd->conn->sconn->client->connections;
 
 	/* Find outstanding event and reschedule. */
 	if (!schedule_deferred_open_message_smb(xconn, opd->mid)) {
@@ -150,6 +171,11 @@ static void aio_open_worker(void *private_data)
 		return;
 	}
 
+	aio_open_do(opd);
+}
+
+static void aio_open_do(struct aio_open_private_data *opd)
+{
 	opd->ret_fd = openat(opd->dir_fd,
 			opd->fname,
 			opd->flags,
@@ -209,15 +235,17 @@ static struct aio_open_private_data *create_private_open_data(const files_struct
 		return NULL;
 	}
 
-	opd->dir_fd = -1;
-	opd->ret_fd = -1;
-	opd->ret_errno = EINPROGRESS;
-	opd->flags = flags;
-	opd->mode = mode;
-	opd->mid = fsp->mid;
-	opd->in_progress = true;
-	opd->sconn = fsp->conn->sconn;
-	opd->initial_allocation_size = fsp->initial_allocation_size;
+	*opd = (struct aio_open_private_data) {
+		.dir_fd = -1,
+		.ret_fd = -1,
+		.ret_errno = EINPROGRESS,
+		.flags = flags,
+		.mode = mode,
+		.mid = fsp->mid,
+		.in_progress = true,
+		.conn = fsp->conn,
+		.initial_allocation_size = fsp->initial_allocation_size,
+	};
 
 	/* Copy our current credentials. */
 	opd->ux_tok = copy_unix_token(opd, get_current_utok(fsp->conn));
@@ -276,7 +304,7 @@ static int open_async(const files_struct *fsp,
 	}
 
 	subreq = pthreadpool_tevent_job_send(opd,
-					     fsp->conn->user_ev_ctx,
+					     fsp->conn->sconn->ev_ctx,
 					     fsp->conn->sconn->pool,
 					     aio_open_worker, opd);
 	if (subreq == NULL) {
@@ -395,7 +423,7 @@ static int aio_pthread_open_fn(vfs_handle_struct *handle,
 #endif
 
 static struct vfs_fn_pointers vfs_aio_pthread_fns = {
-#if defined(HAVE_OPENAT) && defined(USE_LINUX_THREAD_CREDENTIALS)
+#if defined(HAVE_OPENAT) && defined(HAVE_LINUX_THREAD_CREDENTIALS)
 	.open_fn = aio_pthread_open_fn,
 #endif
 };
