@@ -186,10 +186,12 @@ static mode_t recycle_subdir_mode(vfs_handle_struct *handle)
 
 static bool recycle_directory_exist(vfs_handle_struct *handle, const char *dname)
 {
-	SMB_STRUCT_STAT st;
+	struct smb_filename smb_fname = {
+			.base_name = discard_const_p(char, dname)
+	};
 
-	if (vfs_stat_smb_fname(handle->conn, dname, &st) == 0) {
-		if (S_ISDIR(st.st_ex_mode)) {
+	if (SMB_VFS_STAT(handle->conn, &smb_fname) == 0) {
+		if (S_ISDIR(smb_fname.st.st_ex_mode)) {
 			return True;
 		}
 	}
@@ -292,12 +294,26 @@ static bool recycle_create_dir(vfs_handle_struct *handle, const char *dname)
 		if (recycle_directory_exist(handle, new_dir))
 			DEBUG(10, ("recycle: dir %s already exists\n", new_dir));
 		else {
+			struct smb_filename *smb_fname = NULL;
+
 			DEBUG(5, ("recycle: creating new dir %s\n", new_dir));
-			if (SMB_VFS_NEXT_MKDIR(handle, new_dir, mode) != 0) {
+
+			smb_fname = synthetic_smb_fname(talloc_tos(),
+						new_dir,
+						NULL,
+						NULL,
+						0);
+			if (smb_fname == NULL) {
+				goto done;
+			}
+
+			if (SMB_VFS_NEXT_MKDIR(handle, smb_fname, mode) != 0) {
 				DEBUG(1,("recycle: mkdir failed for %s with error: %s\n", new_dir, strerror(errno)));
+				TALLOC_FREE(smb_fname);
 				ret = False;
 				goto done;
 			}
+			TALLOC_FREE(smb_fname);
 		}
 		if (strlcat(new_dir, "/", len+1) >= len+1) {
 			goto done;
@@ -444,7 +460,7 @@ static int recycle_unlink(vfs_handle_struct *handle,
 	bool exist;
 	int rc = -1;
 
-	repository = talloc_sub_advanced(NULL, lp_servicename(talloc_tos(), SNUM(conn)),
+	repository = talloc_sub_full(NULL, lp_servicename(talloc_tos(), SNUM(conn)),
 					conn->session_info->unix_info->unix_name,
 					conn->connectpath,
 					conn->session_info->unix_token->gid,
@@ -574,8 +590,11 @@ static int recycle_unlink(vfs_handle_struct *handle,
 	}
 
 	/* Create smb_fname with final base name and orig stream name. */
-	smb_fname_final = synthetic_smb_fname(talloc_tos(), final_name,
-					      smb_fname->stream_name, NULL);
+	smb_fname_final = synthetic_smb_fname(talloc_tos(),
+					final_name,
+					smb_fname->stream_name,
+					NULL,
+					smb_fname->flags);
 	if (smb_fname_final == NULL) {
 		rc = SMB_VFS_NEXT_UNLINK(handle, smb_fname);
 		goto done;
@@ -643,8 +662,8 @@ static struct vfs_fn_pointers vfs_recycle_fns = {
 	.unlink_fn = recycle_unlink
 };
 
-NTSTATUS vfs_recycle_init(void);
-NTSTATUS vfs_recycle_init(void)
+static_decl_vfs;
+NTSTATUS vfs_recycle_init(TALLOC_CTX *ctx)
 {
 	NTSTATUS ret = smb_register_vfs(SMB_VFS_INTERFACE_VERSION, "recycle",
 					&vfs_recycle_fns);
