@@ -100,9 +100,9 @@ typedef struct um_dirinfo_struct {
 static bool get_digit_group(const char *path, uintmax_t *digit)
 {
 	const char *p = path;
-	char *endp = NULL;
 	codepoint_t cp;
 	size_t size;
+	int error = 0;
 
 	DEBUG(10, ("get_digit_group entering with path '%s'\n",
 		   path));
@@ -120,7 +120,14 @@ static bool get_digit_group(const char *path, uintmax_t *digit)
 			return false;
 		}
 		if ((size == 1) && (isdigit(cp))) {
-			*digit = (uintmax_t)strtoul(p, &endp, 10);
+			*digit = (uintmax_t)smb_strtoul(p,
+							NULL,
+							10,
+							&error,
+							SMB_STR_STANDARD);
+			if (error != 0) {
+				return false;
+			}
 			DEBUG(10, ("num_suffix = '%ju'\n",
 				   *digit));
 			return true;
