@@ -26,6 +26,7 @@
 #include "dbwrap/dbwrap_open.h"
 #include "../libcli/security/security.h"
 #include "lib/privileges.h"
+#include "lib/gencache.h"
 
 static struct db_context *db;
 
@@ -220,7 +221,7 @@ bool init_account_policy(void)
 		return True;
 	}
 
-	db_path = state_path("account_policy.tdb");
+	db_path = state_path(talloc_tos(), "account_policy.tdb");
 	if (db_path == NULL) {
 		return false;
 	}
@@ -455,7 +456,17 @@ bool cache_account_policy_get(enum pdb_policy_type type, uint32_t *value)
 	}
 
 	if (gencache_get(cache_key, talloc_tos(), &cache_value, NULL)) {
-		uint32_t tmp = strtoul(cache_value, NULL, 10);
+		int error = 0;
+		uint32_t tmp;
+
+		tmp = smb_strtoul(cache_value,
+				  NULL,
+				  10,
+				  &error,
+				  SMB_STR_STANDARD);
+		if (error != 0) {
+			goto done;
+		}
 		*value = tmp;
 		ret = True;
 	}
