@@ -70,6 +70,7 @@
 #include "reg_dispatcher.h"
 #include "reg_objects.h"
 #include "../librpc/gen_ndr/ndr_security.h"
+#include "reg_parse_internal.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_REGISTRY
@@ -95,7 +96,7 @@ static WERROR fill_value_cache(struct registry_key *key)
 
 	if (fetch_reg_values(key->key, key->values) == -1) {
 		TALLOC_FREE(key->values);
-		return WERR_BADFILE;
+		return WERR_FILE_NOT_FOUND;
 	}
 
 	return WERR_OK;
@@ -111,12 +112,13 @@ static WERROR fill_subkey_cache(struct registry_key *key)
 		}
 	}
 
+	TALLOC_FREE(key->subkeys);
 	werr = regsubkey_ctr_init(key, &(key->subkeys));
 	W_ERROR_NOT_OK_RETURN(werr);
 
 	if (fetch_reg_keys(key->key, key->subkeys) == -1) {
 		TALLOC_FREE(key->subkeys);
-		return WERR_BADFILE;
+		return WERR_FILE_NOT_FOUND;
 	}
 
 	return WERR_OK;
@@ -131,7 +133,7 @@ static WERROR regkey_open_onelevel(TALLOC_CTX *mem_ctx,
 				   struct registry_key *parent,
 				   const char *name,
 				   const struct security_token *token,
-				   uint32 access_desired,
+				   uint32_t access_desired,
 				   struct registry_key **pregkey)
 {
 	WERROR     	result = WERR_OK;
@@ -142,11 +144,11 @@ static WERROR regkey_open_onelevel(TALLOC_CTX *mem_ctx,
 
 	SMB_ASSERT(strchr(name, '\\') == NULL);
 
-	if (!(regkey = TALLOC_ZERO_P(mem_ctx, struct registry_key)) ||
+	if (!(regkey = talloc_zero(mem_ctx, struct registry_key)) ||
 	    !(regkey->token = dup_nt_token(regkey, token)) ||
-	    !(regkey->key = TALLOC_ZERO_P(regkey, struct registry_key_handle)))
+	    !(regkey->key = talloc_zero(regkey, struct registry_key_handle)))
 	{
-		result = WERR_NOMEM;
+		result = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
@@ -167,7 +169,7 @@ static WERROR regkey_open_onelevel(TALLOC_CTX *mem_ctx,
 		 * Open a copy of the parent key
 		 */
 		if (!parent) {
-			result = WERR_BADFILE;
+			result = WERR_FILE_NOT_FOUND;
 			goto done;
 		}
 		key->name = talloc_strdup(key, parent->key->name);
@@ -183,13 +185,13 @@ static WERROR regkey_open_onelevel(TALLOC_CTX *mem_ctx,
 	}
 
 	if (key->name == NULL) {
-		result = WERR_NOMEM;
+		result = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
 	/* Tag this as a Performance Counter Key */
 
-	if( StrnCaseCmp(key->name, KEY_HKPD, strlen(KEY_HKPD)) == 0 )
+	if( strncasecmp_m(key->name, KEY_HKPD, strlen(KEY_HKPD)) == 0 )
 		key->type = REG_KEY_HKPD;
 
 	/* Look up the table of registry I/O operations */
@@ -198,7 +200,7 @@ static WERROR regkey_open_onelevel(TALLOC_CTX *mem_ctx,
 	if (key->ops == NULL) {
 		DEBUG(0,("reg_open_onelevel: Failed to assign "
 			 "registry_ops to [%s]\n", key->name ));
-		result = WERR_BADFILE;
+		result = WERR_FILE_NOT_FOUND;
 		goto done;
 	}
 
@@ -227,16 +229,21 @@ done:
 }
 
 WERROR reg_openhive(TALLOC_CTX *mem_ctx, const char *hive,
-		    uint32 desired_access,
+		    uint32_t desired_access,
 		    const struct security_token *token,
 		    struct registry_key **pkey)
 {
+	const struct hive_info *hi;
 	SMB_ASSERT(hive != NULL);
-	SMB_ASSERT(hive[0] != '\0');
 	SMB_ASSERT(strchr(hive, '\\') == NULL);
 
-	return regkey_open_onelevel(mem_ctx, NULL, hive, token, desired_access,
-				    pkey);
+	hi = hive_info(hive);
+	if (hi == NULL) {
+		return WERR_FILE_NOT_FOUND;
+	}
+
+	return regkey_open_onelevel(mem_ctx, NULL, hi->short_name, token,
+				    desired_access, pkey);
 }
 
 
@@ -245,7 +252,7 @@ WERROR reg_openhive(TALLOC_CTX *mem_ctx, const char *hive,
  **********************************************************************/
 
 WERROR reg_openkey(TALLOC_CTX *mem_ctx, struct registry_key *parent,
-		   const char *name, uint32 desired_access,
+		   const char *name, uint32_t desired_access,
 		   struct registry_key **pkey)
 {
 	struct registry_key *direct_parent = parent;
@@ -256,7 +263,7 @@ WERROR reg_openkey(TALLOC_CTX *mem_ctx, struct registry_key *parent,
 
 	path = talloc_strdup(frame, name);
 	if (path == NULL) {
-		err = WERR_NOMEM;
+		err = WERR_NOT_ENOUGH_MEMORY;
 		goto error;
 	}
 
@@ -272,7 +279,7 @@ WERROR reg_openkey(TALLOC_CTX *mem_ctx, struct registry_key *parent,
 
 		name_component = talloc_strndup(frame, path, (p - path));
 		if (name_component == NULL) {
-			err = WERR_NOMEM;
+			err = WERR_NOT_ENOUGH_MEMORY;
 			goto error;
 		}
 
@@ -297,7 +304,7 @@ error:
 }
 
 WERROR reg_enumkey(TALLOC_CTX *mem_ctx, struct registry_key *key,
-		   uint32 idx, char **name, NTTIME *last_write_time)
+		   uint32_t idx, char **name, NTTIME *last_write_time)
 {
 	WERROR err;
 
@@ -305,7 +312,8 @@ WERROR reg_enumkey(TALLOC_CTX *mem_ctx, struct registry_key *key,
 		return WERR_ACCESS_DENIED;
 	}
 
-	if (!W_ERROR_IS_OK(err = fill_subkey_cache(key))) {
+	err = fill_subkey_cache(key);
+	if (!W_ERROR_IS_OK(err)) {
 		return err;
 	}
 
@@ -316,7 +324,7 @@ WERROR reg_enumkey(TALLOC_CTX *mem_ctx, struct registry_key *key,
 	if (!(*name = talloc_strdup(mem_ctx,
 			regsubkey_ctr_specific_key(key->subkeys, idx))))
 	{
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	if (last_write_time) {
@@ -327,7 +335,7 @@ WERROR reg_enumkey(TALLOC_CTX *mem_ctx, struct registry_key *key,
 }
 
 WERROR reg_enumvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
-		     uint32 idx, char **pname, struct registry_value **pval)
+		     uint32_t idx, char **pname, struct registry_value **pval)
 {
 	struct registry_value *val;
 	struct regval_blob *blob;
@@ -337,7 +345,8 @@ WERROR reg_enumvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
 		return WERR_ACCESS_DENIED;
 	}
 
-	if (!(W_ERROR_IS_OK(err = fill_value_cache(key)))) {
+	err = fill_value_cache(key);
+	if (!(W_ERROR_IS_OK(err))) {
 		return err;
 	}
 
@@ -349,7 +358,7 @@ WERROR reg_enumvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
 
 	val = talloc_zero(mem_ctx, struct registry_value);
 	if (val == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	val->type = regval_type(blob);
@@ -359,7 +368,7 @@ WERROR reg_enumvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
 	    && !(*pname = talloc_strdup(
 			 mem_ctx, regval_name(blob)))) {
 		TALLOC_FREE(val);
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	*pval = val;
@@ -368,7 +377,7 @@ WERROR reg_enumvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
 
 static WERROR reg_enumvalue_nocachefill(TALLOC_CTX *mem_ctx,
 					struct registry_key *key,
-					uint32 idx, char **pname,
+					uint32_t idx, char **pname,
 					struct registry_value **pval)
 {
 	struct registry_value *val;
@@ -386,7 +395,7 @@ static WERROR reg_enumvalue_nocachefill(TALLOC_CTX *mem_ctx,
 
 	val = talloc_zero(mem_ctx, struct registry_value);
 	if (val == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	val->type = regval_type(blob);
@@ -396,7 +405,7 @@ static WERROR reg_enumvalue_nocachefill(TALLOC_CTX *mem_ctx,
 	    && !(*pname = talloc_strdup(
 			 mem_ctx, regval_name(blob)))) {
 		TALLOC_FREE(val);
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	*pval = val;
@@ -407,7 +416,7 @@ WERROR reg_queryvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
 		      const char *name, struct registry_value **pval)
 {
 	WERROR err;
-	uint32 i;
+	uint32_t i;
 
 	if (!(key->key->access_granted & KEY_QUERY_VALUE)) {
 		return WERR_ACCESS_DENIED;
@@ -432,7 +441,7 @@ WERROR reg_queryvalue(TALLOC_CTX *mem_ctx, struct registry_key *key,
 		}
 	}
 
-	return WERR_BADFILE;
+	return WERR_FILE_NOT_FOUND;
 }
 
 WERROR reg_querymultiplevalues(TALLOC_CTX *mem_ctx,
@@ -460,7 +469,7 @@ WERROR reg_querymultiplevalues(TALLOC_CTX *mem_ctx,
 
 	vals = talloc_zero_array(mem_ctx, struct registry_value, num_names);
 	if (vals == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	for (n=0; n < num_names; n++) {
@@ -491,7 +500,7 @@ WERROR reg_queryinfokey(struct registry_key *key, uint32_t *num_subkeys,
 			uint32_t *max_valbufsize, uint32_t *secdescsize,
 			NTTIME *last_changed_time)
 {
-	uint32 i, max_size;
+	uint32_t i, max_size;
 	size_t max_len;
 	TALLOC_CTX *mem_ctx;
 	WERROR err;
@@ -503,7 +512,7 @@ WERROR reg_queryinfokey(struct registry_key *key, uint32_t *num_subkeys,
 
 	if (!W_ERROR_IS_OK(fill_subkey_cache(key)) ||
 	    !W_ERROR_IS_OK(fill_value_cache(key))) {
-		return WERR_BADFILE;
+		return WERR_FILE_NOT_FOUND;
 	}
 
 	max_len = 0;
@@ -530,7 +539,7 @@ WERROR reg_queryinfokey(struct registry_key *key, uint32_t *num_subkeys,
 	*max_valbufsize = max_size;
 
 	if (!(mem_ctx = talloc_new(key))) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	err = regkey_get_secdesc(mem_ctx, key->key, &secdesc);
@@ -548,12 +557,11 @@ WERROR reg_queryinfokey(struct registry_key *key, uint32_t *num_subkeys,
 }
 
 WERROR reg_createkey(TALLOC_CTX *ctx, struct registry_key *parent,
-		     const char *subkeypath, uint32 desired_access,
+		     const char *subkeypath, uint32_t desired_access,
 		     struct registry_key **pkey,
 		     enum winreg_CreateAction *paction)
 {
 	struct registry_key *key = parent;
-	struct registry_key *create_parent;
 	TALLOC_CTX *mem_ctx;
 	char *path, *end;
 	WERROR err;
@@ -561,12 +569,12 @@ WERROR reg_createkey(TALLOC_CTX *ctx, struct registry_key *parent,
 
 	mem_ctx = talloc_new(ctx);
 	if (mem_ctx == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	path = talloc_strdup(mem_ctx, subkeypath);
 	if (path == NULL) {
-		err = WERR_NOMEM;
+		err = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
@@ -610,7 +618,7 @@ WERROR reg_createkey(TALLOC_CTX *ctx, struct registry_key *parent,
 		goto trans_done;
 	}
 
-	if (!W_ERROR_EQUAL(err, WERR_BADFILE)) {
+	if (!W_ERROR_EQUAL(err, WERR_FILE_NOT_FOUND)) {
 		/*
 		 * Something but "notfound" has happened, so bail out
 		 */
@@ -626,7 +634,7 @@ WERROR reg_createkey(TALLOC_CTX *ctx, struct registry_key *parent,
 				 key->token))
 	{
 		err = WERR_ACCESS_DENIED;
-		goto done;
+		goto trans_done;
 	}
 
 	/*
@@ -665,21 +673,51 @@ trans_done:
 	return err;
 }
 
-WERROR reg_deletekey(struct registry_key *parent, const char *path)
+static WERROR reg_deletekey_internal(TALLOC_CTX *mem_ctx,
+				     struct registry_key *parent,
+				     const char *path, bool lazy)
 {
 	WERROR err;
 	char *name, *end;
-	struct registry_key *tmp_key, *key;
-	TALLOC_CTX *mem_ctx = talloc_stackframe();
-
+	struct registry_key *key;
 	name = talloc_strdup(mem_ctx, path);
 	if (name == NULL) {
-		err = WERR_NOMEM;
+		err = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
+	/* no subkeys - proceed with delete */
+	end = strrchr(name, '\\');
+	if (end != NULL) {
+		*end = '\0';
+
+		err = reg_openkey(mem_ctx, parent, name,
+				  KEY_CREATE_SUB_KEY, &key);
+		W_ERROR_NOT_OK_GOTO_DONE(err);
+
+		parent = key;
+		name = end+1;
+	}
+
+	if (name[0] == '\0') {
+		err = WERR_INVALID_PARAMETER;
+		goto done;
+	}
+
+	err = delete_reg_subkey(parent->key, name, lazy);
+
+done:
+	return err;
+}
+
+WERROR reg_deletekey(struct registry_key *parent, const char *path)
+{
+	WERROR err;
+	struct registry_key *key;
+	TALLOC_CTX *mem_ctx = talloc_stackframe();
+
 	/* check if the key has subkeys */
-	err = reg_openkey(mem_ctx, parent, name, REG_KEY_READ, &key);
+	err = reg_openkey(mem_ctx, parent, path, REG_KEY_READ, &key);
 	W_ERROR_NOT_OK_GOTO_DONE(err);
 
 	err = regdb_transaction_start();
@@ -690,32 +728,15 @@ WERROR reg_deletekey(struct registry_key *parent, const char *path)
 	}
 
 	err = fill_subkey_cache(key);
-	W_ERROR_NOT_OK_GOTO(err, trans_done);
+	if (!W_ERROR_IS_OK(err)) {
+		goto trans_done;
+	}
 
 	if (regsubkey_ctr_numkeys(key->subkeys) > 0) {
 		err = WERR_ACCESS_DENIED;
 		goto trans_done;
 	}
-
-	/* no subkeys - proceed with delete */
-	end = strrchr(name, '\\');
-	if (end != NULL) {
-		*end = '\0';
-
-		err = reg_openkey(mem_ctx, parent, name,
-				  KEY_CREATE_SUB_KEY, &tmp_key);
-		W_ERROR_NOT_OK_GOTO(err, trans_done);
-
-		parent = tmp_key;
-		name = end+1;
-	}
-
-	if (name[0] == '\0') {
-		err = WERR_INVALID_PARAM;
-		goto trans_done;
-	}
-
-	err = delete_reg_subkey(parent->key, name);
+	err = reg_deletekey_internal(mem_ctx, parent, path, false);
 
 trans_done:
 	if (W_ERROR_IS_OK(err)) {
@@ -735,6 +756,7 @@ done:
 	return err;
 }
 
+
 WERROR reg_setvalue(struct registry_key *key, const char *name,
 		    const struct registry_value *val)
 {
@@ -775,14 +797,14 @@ WERROR reg_setvalue(struct registry_key *key, const char *name,
 
 	if (res == 0) {
 		TALLOC_FREE(key->values);
-		err = WERR_NOMEM;
+		err = WERR_NOT_ENOUGH_MEMORY;
 		goto done;
 	}
 
 	if (!store_reg_values(key->key, key->values)) {
 		TALLOC_FREE(key->values);
 		DEBUG(0, ("reg_setvalue: store_reg_values failed\n"));
-		err = WERR_REG_IO_FAILURE;
+		err = WERR_REGISTRY_IO_FAILED;
 		goto done;
 	}
 
@@ -811,7 +833,7 @@ static WERROR reg_value_exists(struct registry_key *key, const char *name)
 	blob = regval_ctr_getvalue(key->values, name);
 
 	if (blob == NULL) {
-		return WERR_BADFILE;
+		return WERR_FILE_NOT_FOUND;
 	} else {
 		return WERR_OK;
 	}
@@ -848,7 +870,7 @@ WERROR reg_deletevalue(struct registry_key *key, const char *name)
 
 	if (!store_reg_values(key->key, key->values)) {
 		TALLOC_FREE(key->values);
-		err = WERR_REG_IO_FAILURE;
+		err = WERR_REGISTRY_IO_FAILED;
 		DEBUG(0, ("reg_deletevalue: store_reg_values failed\n"));
 		goto done;
 	}
@@ -886,7 +908,7 @@ WERROR reg_setkeysecurity(struct registry_key *key,
 WERROR reg_getversion(uint32_t *version)
 {
 	if (version == NULL) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	*version = 0x00000005; /* Windows 2000 registry API version */
@@ -918,7 +940,7 @@ WERROR reg_deleteallvalues(struct registry_key *key)
 
 	if (!store_reg_values(key->key, key->values)) {
 		TALLOC_FREE(key->values);
-		return WERR_REG_IO_FAILURE;
+		return WERR_REGISTRY_IO_FAILED;
 	}
 
 	return WERR_OK;
@@ -931,17 +953,23 @@ WERROR reg_deleteallvalues(struct registry_key *key)
  */
 static WERROR reg_deletekey_recursive_internal(struct registry_key *parent,
 					       const char *path,
-					       bool del_key)
+					       bool del_key, bool lazy)
 {
 	WERROR werr = WERR_OK;
 	struct registry_key *key;
 	char *subkey_name = NULL;
-	uint32 i;
+	uint32_t i;
 	TALLOC_CTX *mem_ctx = talloc_stackframe();
 
+	DEBUG(5, ("reg_deletekey_recursive_internal: deleting '%s' from '%s'\n",
+		  path, parent->key->name));
+
 	/* recurse through subkeys first */
 	werr = reg_openkey(mem_ctx, parent, path, REG_KEY_ALL, &key);
 	if (!W_ERROR_IS_OK(werr)) {
+		DEBUG(3, ("reg_deletekey_recursive_internal: error opening "
+			  "subkey '%s' of '%s': '%s'\n",
+			  path, parent->key->name, win_errstr(werr)));
 		goto done;
 	}
 
@@ -949,21 +977,25 @@ static WERROR reg_deletekey_recursive_internal(struct registry_key *parent,
 	W_ERROR_NOT_OK_GOTO_DONE(werr);
 
 	/*
-	 * loop from top to bottom for perfomance:
+	 * loop from top to bottom for performance:
 	 * this way, we need to rehash the regsubkey containers less
 	 */
 	for (i = regsubkey_ctr_numkeys(key->subkeys) ; i > 0; i--) {
 		subkey_name = regsubkey_ctr_specific_key(key->subkeys, i-1);
-		werr = reg_deletekey_recursive_internal(key, subkey_name, true);
+		werr = reg_deletekey_recursive_internal(key, subkey_name, true, del_key);
 		W_ERROR_NOT_OK_GOTO_DONE(werr);
 	}
 
 	if (del_key) {
 		/* now delete the actual key */
-		werr = reg_deletekey(parent, path);
+		werr = reg_deletekey_internal(mem_ctx, parent, path, lazy);
 	}
 
 done:
+
+	DEBUG(5, ("reg_deletekey_recursive_internal: done deleting '%s' from "
+		  "'%s': %s\n",
+		  path, parent->key->name, win_errstr(werr)));
 	TALLOC_FREE(mem_ctx);
 	return werr;
 }
@@ -982,14 +1014,14 @@ static WERROR reg_deletekey_recursive_trans(struct registry_key *parent,
 		return werr;
 	}
 
-	werr = reg_deletekey_recursive_internal(parent, path, del_key);
+	werr = reg_deletekey_recursive_internal(parent, path, del_key, false);
 
 	if (!W_ERROR_IS_OK(werr)) {
 		WERROR werr2;
-
-		DEBUG(1, (__location__ " failed to delete key '%s' from key "
-			  "'%s': %s\n", path, parent->key->name,
-			  win_errstr(werr)));
+		DEBUG(W_ERROR_EQUAL(werr, WERR_FILE_NOT_FOUND) ? 5 : 1,
+		      (__location__ ": failed to delete key '%s' from key "
+		       "'%s': %s\n", path, parent->key->name,
+		       win_errstr(werr)));
 
 		werr2 = regdb_transaction_cancel();
 		if (!W_ERROR_IS_OK(werr2)) {
@@ -1007,6 +1039,10 @@ static WERROR reg_deletekey_recursive_trans(struct registry_key *parent,
 			DEBUG(0, ("reg_deletekey_recursive_trans: "
 				  "error committing transaction: %s\n",
 				  win_errstr(werr)));
+		} else {
+			DEBUG(5, ("reg_deletekey_recursive_trans: deleted key '%s' from '%s'\n",
+				  path, parent->key->name));
+
 		}
 	}
 
