@@ -22,7 +22,6 @@
 
 #include "includes.h"
 
-#include "serverid.h"
 #include "ntdomain.h"
 #include "messages.h"
 
@@ -31,6 +30,7 @@
 #include "rpc_server/rpc_server.h"
 #include "rpc_server/rpc_sock_helper.h"
 #include "rpc_server/fss/srv_fss_agent.h"
+#include "rpc_server/fssd.h"
 
 #define DAEMON_NAME "fssd"
 
@@ -182,14 +182,6 @@ void start_fssd(struct tevent_context *ev_ctx,
 	fssd_setup_sig_term_handler(ev_ctx);
 	fssd_setup_sig_hup_handler(ev_ctx, msg_ctx);
 
-	ok = serverid_register(messaging_server_id(msg_ctx),
-			       FLAG_MSG_GENERAL |
-			       FLAG_MSG_PRINT_GENERAL);
-	if (!ok) {
-		DEBUG(0, ("Failed to register serverid in fssd!\n"));
-		exit(1);
-	}
-
 	messaging_register(msg_ctx,
 			   ev_ctx,
 			   MSG_SMB_CONF_UPDATED,
