@@ -38,6 +38,7 @@
 #include "librpc/gen_ndr/srv_lsa.h"
 #include "librpc/gen_ndr/srv_samr.h"
 #include "librpc/gen_ndr/srv_netlogon.h"
+#include "rpc_server/lsasd.h"
 
 #define DAEMON_NAME "lsasd"
 #define LSASD_MAX_SOCKETS 64
@@ -56,9 +57,6 @@ static struct pf_daemon_config default_pf_lsasd_cfg = {
 };
 static struct pf_daemon_config pf_lsasd_cfg = { 0 };
 
-void start_lsasd(struct tevent_context *ev_ctx,
-		 struct messaging_context *msg_ctx);
-
 static void lsasd_reopen_logs(int child_id)
 {
 	char *lfile = lp_logfile(talloc_tos());
@@ -243,7 +241,7 @@ static bool lsasd_child_init(struct tevent_context *ev_ctx,
 			     struct pf_worker_data *pf)
 {
 	NTSTATUS status;
-	struct messaging_context *msg_ctx = server_messaging_context();
+	struct messaging_context *msg_ctx = global_messaging_context();
 	bool ok;
 
 	status = reinit_after_fork(msg_ctx, ev_ctx,
