@@ -20,6 +20,8 @@
 #ifndef _RPC_SERVER_H_
 #define _RPC_SERVER_H_
 
+#include "librpc/rpc/rpc_common.h" /* For enum dcerpc_transport_t */
+
 struct pipes_struct;
 
 typedef bool (*dcerpc_ncacn_disconnect_fn)(struct pipes_struct *p);
@@ -37,10 +39,10 @@ struct named_pipe_client {
 
 	struct tstream_context *tstream;
 
-	struct tsocket_address *client;
-	char *client_name;
-	struct tsocket_address *server;
-	char *server_name;
+	struct tsocket_address *remote_client_addr;
+	char *remote_client_name;
+	struct tsocket_address *local_server_addr;
+	char *local_server_name;
 
 	struct auth_session_info *session_info;
 
@@ -69,9 +71,9 @@ int make_server_pipes_struct(TALLOC_CTX *mem_ctx,
 			     struct messaging_context *msg_ctx,
 			     const char *pipe_name,
 			     enum dcerpc_transport_t transport,
-			     const struct tsocket_address *local_address,
 			     const struct tsocket_address *remote_address,
-			     struct auth_session_info *session_info,
+			     const struct tsocket_address *local_address,
+			     struct auth_session_info **session_info,
 			     struct pipes_struct **_p,
 			     int *perrno);
 
