@@ -39,7 +39,6 @@
 #include "auth.h"
 #include "messages.h"
 #include "serverid.h"
-#include "lib/conn_tdb.h"
 
 extern const struct generic_mapping file_generic_mapping;
 
@@ -81,11 +80,9 @@ struct share_conn_stat {
 /*******************************************************************
 ********************************************************************/
 
-static int enum_file_fn(const struct share_mode_entry *e,
-			const struct file_id *id,
-			const char *sharepath,
-			const char *fname,
-			const char *sname,
+static int enum_file_fn(struct file_id id,
+			const struct share_mode_data *d,
+			const struct share_mode_entry *e,
 			void *private_data)
 {
 	struct file_enum_count *fenum =
@@ -124,19 +121,25 @@ static int enum_file_fn(const struct share_mode_entry *e,
 	/* need to count the number of locks on a file */
 
 	ZERO_STRUCT( fsp );
-	fsp.file_id = *id;
+	fsp.file_id = id;
 
 	if ( (brl = brl_get_locks(talloc_tos(), &fsp)) != NULL ) {
 		num_locks = brl_num_locks(brl);
 		TALLOC_FREE(brl);
 	}
 
-	if ( strcmp( fname, "." ) == 0 ) {
-		fullpath = talloc_asprintf(fenum->ctx, "C:%s", sharepath );
+	if ( strcmp(d->base_name, "." ) == 0 ) {
+		fullpath = talloc_asprintf(
+			fenum->ctx,
+			"C:%s",
+			d->servicepath);
 	} else {
-		fullpath = talloc_asprintf(fenum->ctx, "C:%s/%s%s",
-					   sharepath, fname,
-					   sname ? sname : "");
+		fullpath = talloc_asprintf(
+			fenum->ctx,
+			"C:%s/%s%s",
+			d->servicepath,
+			d->base_name,
+			(d->stream_name != NULL) ? d->stream_name : "");
 	}
 	if (!fullpath) {
 		return 0;
@@ -222,7 +225,7 @@ static void init_srv_share_info_1(struct pipes_struct *p,
 	char *remark = lp_comment(p->mem_ctx, snum);
 
 	if (remark) {
-		remark = talloc_sub_advanced(
+		remark = talloc_sub_full(
 			p->mem_ctx, lp_servicename(talloc_tos(), snum),
 			get_current_username(), lp_path(talloc_tos(), snum),
 			p->session_info->unix_token->uid, get_current_username(),
@@ -250,7 +253,7 @@ static void init_srv_share_info_2(struct pipes_struct *p,
 
 	remark = lp_comment(p->mem_ctx, snum);
 	if (remark) {
-		remark = talloc_sub_advanced(
+		remark = talloc_sub_full(
 			p->mem_ctx, lp_servicename(talloc_tos(), snum),
 			get_current_username(), lp_path(talloc_tos(), snum),
 			p->session_info->unix_token->uid, get_current_username(),
@@ -315,7 +318,7 @@ static void init_srv_share_info_501(struct pipes_struct *p,
 	char *remark = lp_comment(p->mem_ctx, snum);
 
 	if (remark) {
-		remark = talloc_sub_advanced(
+		remark = talloc_sub_full(
 			p->mem_ctx, lp_servicename(talloc_tos(), snum),
 			get_current_username(), lp_path(talloc_tos(), snum),
 			p->session_info->unix_token->uid, get_current_username(),
@@ -349,7 +352,7 @@ static void init_srv_share_info_502(struct pipes_struct *p,
 	char *remark = lp_comment(ctx, snum);
 
 	if (remark) {
-		remark = talloc_sub_advanced(
+		remark = talloc_sub_full(
 			p->mem_ctx, lp_servicename(talloc_tos(), snum),
 			get_current_username(), lp_path(talloc_tos(), snum),
 			p->session_info->unix_token->uid, get_current_username(),
@@ -390,7 +393,7 @@ static void init_srv_share_info_1004(struct pipes_struct *p,
 	char *remark = lp_comment(p->mem_ctx, snum);
 
 	if (remark) {
-		remark = talloc_sub_advanced(
+		remark = talloc_sub_full(
 			p->mem_ctx, lp_servicename(talloc_tos(), snum),
 			get_current_username(), lp_path(talloc_tos(), snum),
 			p->session_info->unix_token->uid, get_current_username(),
@@ -841,11 +844,9 @@ static WERROR init_srv_sess_info_0(struct pipes_struct *p,
  * find out the session on which this file is open and bump up its count
  **********************************************************************/
 
-static int count_sess_files_fn(const struct share_mode_entry *e,
-			       const struct file_id *id,
-			       const char *sharepath,
-			       const char *fname,
-			       const char *sname,
+static int count_sess_files_fn(struct file_id fid,
+			       const struct share_mode_data *d,
+			       const struct share_mode_entry *e,
 			       void *data)
 {
 	struct sess_file_info *info = data;
@@ -969,18 +970,16 @@ static WERROR init_srv_sess_info_1(struct pipes_struct *p,
  find the share connection on which this open exists.
  ********************************************************************/
 
-static int share_file_fn(const struct share_mode_entry *e,
-			 const struct file_id *id,
-			 const char *sharepath,
-			 const char *fname,
-			 const char *sname,
+static int share_file_fn(struct file_id fid,
+			 const struct share_mode_data *d,
+			 const struct share_mode_entry *e,
 			 void *data)
 {
 	struct share_file_stat *sfs = data;
 	uint32_t i;
 	uint32_t offset = sfs->total_entries - sfs->resp_entries;
 
-	if (strequal(sharepath, sfs->in_sharepath)) {
+	if (strequal(d->servicepath, sfs->in_sharepath)) {
 		for (i=0; i < sfs->resp_entries; i++) {
 			if (serverid_equal(&e->pid, &sfs->svrid_arr[offset + i])) {
 				sfs->netconn_arr[i].num_open ++;
@@ -2345,7 +2344,7 @@ WERROR _srvsvc_NetGetFileSecurity(struct pipes_struct *p,
 		goto error_exit;
 	}
 
-	nt_status = create_conn_struct_tos_cwd(server_messaging_context(),
+	nt_status = create_conn_struct_tos_cwd(global_messaging_context(),
 					       snum,
 					       lp_path(frame, snum),
 					       p->session_info,
@@ -2479,7 +2478,7 @@ WERROR _srvsvc_NetSetFileSecurity(struct pipes_struct *p,
 		goto error_exit;
 	}
 
-	nt_status = create_conn_struct_tos_cwd(server_messaging_context(),
+	nt_status = create_conn_struct_tos_cwd(global_messaging_context(),
 					       snum,
 					       lp_path(frame, snum),
 					       p->session_info,
@@ -2683,11 +2682,9 @@ struct enum_file_close_state {
 	struct messaging_context *msg_ctx;
 };
 
-static int enum_file_close_fn(const struct share_mode_entry *e,
-			      const struct file_id *id,
-			      const char *sharepath,
-			      const char *fname,
-			      const char *sname,
+static int enum_file_close_fn(struct file_id id,
+			      const struct share_mode_data *d,
+			      const struct share_mode_entry *e,
 			      void *private_data)
 {
 	char msg[MSG_SMB_SHARE_MODE_ENTRY_SIZE];
@@ -2704,10 +2701,10 @@ static int enum_file_close_fn(const struct share_mode_entry *e,
 	}
 
 	/* Ok - send the close message. */
-	DBG_DEBUG("request to close file %s, %s\n", sharepath,
-		  share_mode_str(talloc_tos(), 0, id, e));
+	DBG_DEBUG("request to close file %s, %s\n", d->servicepath,
+		  share_mode_str(talloc_tos(), 0, &id, e));
 
-	share_mode_entry_to_message(msg, id, e);
+	share_mode_entry_to_message(msg, &id, e);
 
 	state->r->out.result = ntstatus_to_werror(
 		messaging_send_buf(state->msg_ctx,
