@@ -22,6 +22,7 @@
 
 #include "includes.h"
 #include "system/filesys.h"
+#include "lib/util/server_id.h"
 #include "smbd/smbd.h"
 #include "smbd/globals.h"
 #include "libcli/security/security.h"
@@ -29,6 +30,7 @@
 #include "librpc/gen_ndr/ndr_open_files.h"
 #include "serverid.h"
 #include "fake_file.h"
+#include "locking/leases_db.h"
 
 NTSTATUS vfs_default_durable_cookie(struct files_struct *fsp,
 				    TALLOC_CTX *mem_ctx,
@@ -116,11 +118,10 @@ NTSTATUS vfs_default_durable_cookie(struct files_struct *fsp,
 	cookie.stat_info.st_ex_mtime = fsp->fsp_name->st.st_ex_mtime;
 	cookie.stat_info.st_ex_ctime = fsp->fsp_name->st.st_ex_ctime;
 	cookie.stat_info.st_ex_btime = fsp->fsp_name->st.st_ex_btime;
-	cookie.stat_info.st_ex_calculated_birthtime = fsp->fsp_name->st.st_ex_calculated_birthtime;
+	cookie.stat_info.st_ex_iflags = fsp->fsp_name->st.st_ex_iflags;
 	cookie.stat_info.st_ex_blksize = fsp->fsp_name->st.st_ex_blksize;
 	cookie.stat_info.st_ex_blocks = fsp->fsp_name->st.st_ex_blocks;
 	cookie.stat_info.st_ex_flags = fsp->fsp_name->st.st_ex_flags;
-	cookie.stat_info.st_ex_mask = fsp->fsp_name->st.st_ex_mask;
 
 	ndr_err = ndr_push_struct_blob(cookie_blob, mem_ctx, &cookie,
 			(ndr_push_flags_fn_t)ndr_push_vfs_default_durable_cookie);
@@ -193,10 +194,7 @@ NTSTATUS vfs_default_durable_disconnect(struct files_struct *fsp,
 
 	/* Ensure any pending write time updates are done. */
 	if (fsp->update_write_time_event) {
-		update_write_time_handler(fsp->conn->sconn->ev_ctx,
-					fsp->update_write_time_event,
-					timeval_current(),
-					(void *)fsp);
+		fsp_flush_write_time_update(fsp);
 	}
 
 	/*
@@ -269,11 +267,10 @@ NTSTATUS vfs_default_durable_disconnect(struct files_struct *fsp,
 	cookie.stat_info.st_ex_mtime = fsp->fsp_name->st.st_ex_mtime;
 	cookie.stat_info.st_ex_ctime = fsp->fsp_name->st.st_ex_ctime;
 	cookie.stat_info.st_ex_btime = fsp->fsp_name->st.st_ex_btime;
-	cookie.stat_info.st_ex_calculated_birthtime = fsp->fsp_name->st.st_ex_calculated_birthtime;
+	cookie.stat_info.st_ex_iflags = fsp->fsp_name->st.st_ex_iflags;
 	cookie.stat_info.st_ex_blksize = fsp->fsp_name->st.st_ex_blksize;
 	cookie.stat_info.st_ex_blocks = fsp->fsp_name->st.st_ex_blocks;
 	cookie.stat_info.st_ex_flags = fsp->fsp_name->st.st_ex_flags;
-	cookie.stat_info.st_ex_mask = fsp->fsp_name->st.st_ex_mask;
 
 	ndr_err = ndr_push_struct_blob(&new_cookie_blob, mem_ctx, &cookie,
 			(ndr_push_flags_fn_t)ndr_push_vfs_default_durable_cookie);
@@ -304,30 +301,6 @@ static bool vfs_default_durable_reconnect_check_stat(
 {
 	int ret;
 
-	if (cookie_st->st_ex_dev != fsp_st->st_ex_dev) {
-		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
-			  "stat_ex.%s differs: "
-			  "cookie:%llu != stat:%llu, "
-			  "denying durable reconnect\n",
-			  name,
-			  "st_ex_dev",
-			  (unsigned long long)cookie_st->st_ex_dev,
-			  (unsigned long long)fsp_st->st_ex_dev));
-		return false;
-	}
-
-	if (cookie_st->st_ex_ino != fsp_st->st_ex_ino) {
-		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
-			  "stat_ex.%s differs: "
-			  "cookie:%llu != stat:%llu, "
-			  "denying durable reconnect\n",
-			  name,
-			  "st_ex_ino",
-			  (unsigned long long)cookie_st->st_ex_ino,
-			  (unsigned long long)fsp_st->st_ex_ino));
-		return false;
-	}
-
 	if (cookie_st->st_ex_mode != fsp_st->st_ex_mode) {
 		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
 			  "stat_ex.%s differs: "
@@ -472,17 +445,15 @@ static bool vfs_default_durable_reconnect_check_stat(
 		return false;
 	}
 
-	if (cookie_st->st_ex_calculated_birthtime !=
-	    fsp_st->st_ex_calculated_birthtime)
-	{
+	if (cookie_st->st_ex_iflags != fsp_st->st_ex_iflags) {
 		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
 			  "stat_ex.%s differs: "
 			  "cookie:%llu != stat:%llu, "
 			  "denying durable reconnect\n",
 			  name,
 			  "st_ex_calculated_birthtime",
-			  (unsigned long long)cookie_st->st_ex_calculated_birthtime,
-			  (unsigned long long)fsp_st->st_ex_calculated_birthtime));
+			  (unsigned long long)cookie_st->st_ex_iflags,
+			  (unsigned long long)fsp_st->st_ex_iflags));
 		return false;
 	}
 
@@ -522,18 +493,6 @@ static bool vfs_default_durable_reconnect_check_stat(
 		return false;
 	}
 
-	if (cookie_st->st_ex_mask != fsp_st->st_ex_mask) {
-		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
-			  "stat_ex.%s differs: "
-			  "cookie:%llu != stat:%llu, "
-			  "denying durable reconnect\n",
-			  name,
-			  "st_ex_mask",
-			  (unsigned long long)cookie_st->st_ex_mask,
-			  (unsigned long long)fsp_st->st_ex_mask));
-		return false;
-	}
-
 	return true;
 }
 
@@ -572,10 +531,11 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	 * call below.
 	 */
 
-	ZERO_STRUCT(cookie);
-
-	ndr_err = ndr_pull_struct_blob(&old_cookie, talloc_tos(), &cookie,
-			(ndr_pull_flags_fn_t)ndr_pull_vfs_default_durable_cookie);
+	ndr_err = ndr_pull_struct_blob_all(
+		&old_cookie,
+		talloc_tos(),
+		&cookie,
+		(ndr_pull_flags_fn_t)ndr_pull_vfs_default_durable_cookie);
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
 		status = ndr_map_error2ntstatus(ndr_err);
 		return status;
@@ -598,8 +558,11 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	}
 
 	/* Create an smb_filename with stream_name == NULL. */
-	smb_fname = synthetic_smb_fname(talloc_tos(), cookie.base_name,
-					NULL, NULL);
+	smb_fname = synthetic_smb_fname(talloc_tos(),
+					cookie.base_name,
+					NULL,
+					NULL,
+					0);
 	if (smb_fname == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -725,29 +688,47 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	fsp->oplock_type = e->op_type;
 
 	if (fsp->oplock_type == LEASE_OPLOCK) {
-		struct share_mode_lease *l = &lck->data->leases[e->lease_idx];
-		struct smb2_lease_key key;
+		uint32_t current_state;
+		uint16_t lease_version, epoch;
 
-		key.data[0] = l->lease_key.data[0];
-		key.data[1] = l->lease_key.data[1];
+		/*
+		 * Ensure the existing client guid matches the
+		 * stored one in the share_mode_entry.
+		 */
+		if (!GUID_equal(fsp_client_guid(fsp),
+				&e->client_guid)) {
+			TALLOC_FREE(lck);
+			fsp_free(fsp);
+			return NT_STATUS_OBJECT_NAME_NOT_FOUND;
+		}
 
-		fsp->lease = find_fsp_lease(fsp, &key, l);
+		status = leases_db_get(
+			&e->client_guid,
+			&e->lease_key,
+			&file_id,
+			&current_state, /* current_state */
+			NULL, /* breaking */
+			NULL, /* breaking_to_requested */
+			NULL, /* breaking_to_required */
+			&lease_version, /* lease_version */
+			&epoch); /* epoch */
+		if (!NT_STATUS_IS_OK(status)) {
+			TALLOC_FREE(lck);
+			fsp_free(fsp);
+			return status;
+		}
+
+		fsp->lease = find_fsp_lease(
+			fsp,
+			&e->lease_key,
+			current_state,
+			lease_version,
+			epoch);
 		if (fsp->lease == NULL) {
 			TALLOC_FREE(lck);
 			fsp_free(fsp);
 			return NT_STATUS_NO_MEMORY;
 		}
-
-		/*
-		 * Ensure the existing client guid matches the
-		 * stored one in the share_mode_lease.
-		 */
-		if (!GUID_equal(fsp_client_guid(fsp),
-				&l->client_guid)) {
-			TALLOC_FREE(lck);
-			fsp_free(fsp);
-			return NT_STATUS_OBJECT_NAME_NOT_FOUND;
-		}
 	}
 
 	fsp->initial_allocation_size = cookie.initial_allocation_size;
@@ -861,6 +842,8 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 	}
 
+	(void)dos_mode(fsp->conn, fsp->fsp_name);
+
 	ok = vfs_default_durable_reconnect_check_stat(&cookie.stat_info,
 						      &fsp->fsp_name->st,
 						      fsp_str_dbg(fsp));
