@@ -463,22 +463,6 @@ bool file_find_subpath(files_struct *dir_fsp)
 	return false;
 }
 
-/****************************************************************************
- Sync open files on a connection.
-****************************************************************************/
-
-void file_sync_all(connection_struct *conn)
-{
-	files_struct *fsp, *next;
-
-	for (fsp=conn->sconn->files; fsp; fsp=next) {
-		next=fsp->next;
-		if ((conn == fsp->conn) && (fsp->fh->fd != -1)) {
-			sync_file(conn, fsp, True /* write through */);
-		}
-	}
-}
-
 /****************************************************************************
  Free up a fsp.
 ****************************************************************************/
@@ -794,7 +778,18 @@ const struct GUID *fsp_client_guid(const files_struct *fsp)
 
 size_t fsp_fullbasepath(struct files_struct *fsp, char *buf, size_t buflen)
 {
-	int len;
+	int len = 0;
+	char tmp_buf[1] = {'\0'};
+
+	/*
+	 * Don't pass NULL buffer to snprintf (to satisfy static checker)
+	 * Some callers will call this function with NULL for buf and
+	 * 0 for buflen in order to get length of fullbasepatch (without
+	 * needing to allocate or write to buf)
+	 */
+	if (buf == NULL) {
+		buf = tmp_buf;
+	}
 
 	len = snprintf(buf, buflen, "%s/%s", fsp->conn->connectpath,
 		       fsp->fsp_name->base_name);
