@@ -28,6 +28,7 @@
 #include "smbd/globals.h"
 #include "msdfs.h"
 #include "auth.h"
+#include "../auth/auth_util.h"
 #include "lib/param/loadparm.h"
 #include "libcli/security/security.h"
 #include "librpc/gen_ndr/ndr_dfsblobs.h"
@@ -240,7 +241,6 @@ static NTSTATUS parse_dfs_path(connection_struct *conn,
 *********************************************************/
 
 static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
-			    struct tevent_context *ev,
 			    struct messaging_context *msg,
 			    connection_struct **pconn,
 			    int snum,
@@ -258,7 +258,12 @@ static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	sconn->ev_ctx = ev;
+	sconn->ev_ctx = samba_tevent_context_init(sconn);
+	if (sconn->ev_ctx == NULL) {
+		TALLOC_FREE(sconn);
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	sconn->msg_ctx = msg;
 
 	conn = conn_new(sconn);
@@ -301,7 +306,12 @@ static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
 			TALLOC_FREE(conn);
 			return NT_STATUS_NO_MEMORY;
 		}
-		vfs_user = conn->session_info->unix_info->unix_name;
+		/* unix_info could be NULL in session_info */
+		if (conn->session_info->unix_info != NULL) {
+			vfs_user = conn->session_info->unix_info->unix_name;
+		} else {
+			vfs_user = get_current_username();
+		}
 	} else {
 		/* use current authenticated user in absence of session_info */
 		vfs_user = get_current_username();
@@ -353,62 +363,102 @@ static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
+	talloc_free(conn->origpath);
+	conn->origpath = talloc_strdup(conn, conn->connectpath);
+	if (conn->origpath == NULL) {
+		conn_free(conn);
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	conn->fs_capabilities = SMB_VFS_FS_CAPABILITIES(conn, &conn->ts_res);
-	*pconn = conn;
+	conn->tcon_done = true;
+	*pconn = talloc_move(ctx, &conn);
 
 	return NT_STATUS_OK;
 }
 
+static int conn_struct_tos_destructor(struct conn_struct_tos *c)
+{
+	if (c->oldcwd_fname != NULL) {
+		vfs_ChDir(c->conn, c->oldcwd_fname);
+		TALLOC_FREE(c->oldcwd_fname);
+	}
+	SMB_VFS_DISCONNECT(c->conn);
+	conn_free(c->conn);
+	return 0;
+}
+
 /********************************************************
  Fake up a connection struct for the VFS layer, for use in
  applications (such as the python bindings), that do not want the
  global working directory changed under them.
 
  SMB_VFS_CONNECT requires root privileges.
-*********************************************************/
+ This temporary uses become_root() and unbecome_root().
 
-NTSTATUS create_conn_struct(TALLOC_CTX *ctx,
-			    struct tevent_context *ev,
-			    struct messaging_context *msg,
-			    connection_struct **pconn,
-			    int snum,
-			    const char *path,
-			    const struct auth_session_info *session_info)
+ But further impersonation has to be cone by the caller.
+*********************************************************/
+NTSTATUS create_conn_struct_tos(struct messaging_context *msg,
+				int snum,
+				const char *path,
+				const struct auth_session_info *session_info,
+				struct conn_struct_tos **_c)
 {
+	struct conn_struct_tos *c = NULL;
 	NTSTATUS status;
+
+	*_c = NULL;
+
+	c = talloc_zero(talloc_tos(), struct conn_struct_tos);
+	if (c == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	become_root();
-	status = create_conn_struct_as_root(ctx, ev,
-					    msg, pconn,
-					    snum, path,
+	status = create_conn_struct_as_root(c,
+					    msg,
+					    &c->conn,
+					    snum,
+					    path,
 					    session_info);
 	unbecome_root();
+	if (!NT_STATUS_IS_OK(status)) {
+		TALLOC_FREE(c);
+		return status;
+	}
 
-	return status;
+	talloc_set_destructor(c, conn_struct_tos_destructor);
+
+	*_c = c;
+	return NT_STATUS_OK;
 }
 
 /********************************************************
  Fake up a connection struct for the VFS layer.
  Note: this performs a vfs connect and CHANGES CWD !!!! JRA.
 
- The old working directory is returned on *poldcwd, allocated on ctx.
+ See also the comment for create_conn_struct_tos() above!
+
+ The CWD change is reverted by the destructor of
+ conn_struct_tos when the current talloc_tos() is destroyed.
 *********************************************************/
-
-NTSTATUS create_conn_struct_cwd(TALLOC_CTX *ctx,
-				struct tevent_context *ev,
-				struct messaging_context *msg,
-				connection_struct **pconn,
-				int snum,
-				const char *path,
-				const struct auth_session_info *session_info,
-				char **poldcwd)
+NTSTATUS create_conn_struct_tos_cwd(struct messaging_context *msg,
+				    int snum,
+				    const char *path,
+				    const struct auth_session_info *session_info,
+				    struct conn_struct_tos **_c)
 {
-	connection_struct *conn;
-	char *oldcwd;
+	struct conn_struct_tos *c = NULL;
+	struct smb_filename smb_fname_connectpath = {0};
+	NTSTATUS status;
 
-	NTSTATUS status = create_conn_struct(ctx, ev,
-					     msg, &conn,
-					     snum, path,
-					     session_info);
+	*_c = NULL;
+
+	status = create_conn_struct_tos(msg,
+					snum,
+					path,
+					session_info,
+					&c);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -419,26 +469,29 @@ NTSTATUS create_conn_struct_cwd(TALLOC_CTX *ctx,
 	 * user we will fail.... WTF ? JRA.
 	 */
 
-	oldcwd = vfs_GetWd(ctx, conn);
-	if (oldcwd == NULL) {
+	c->oldcwd_fname = vfs_GetWd(c, c->conn);
+	if (c->oldcwd_fname == NULL) {
 		status = map_nt_error_from_unix(errno);
 		DEBUG(3, ("vfs_GetWd failed: %s\n", strerror(errno)));
-		conn_free(conn);
+		TALLOC_FREE(c);
 		return status;
 	}
 
-	if (vfs_ChDir(conn,conn->connectpath) != 0) {
+	smb_fname_connectpath = (struct smb_filename) {
+		.base_name = c->conn->connectpath
+	};
+
+	if (vfs_ChDir(c->conn, &smb_fname_connectpath) != 0) {
 		status = map_nt_error_from_unix(errno);
-		DEBUG(3,("create_conn_struct: Can't ChDir to new conn path %s. "
-			"Error was %s\n",
-			conn->connectpath, strerror(errno) ));
-		conn_free(conn);
+		DBG_NOTICE("Can't ChDir to new conn path %s. "
+			   "Error was %s\n",
+			   c->conn->connectpath, strerror(errno));
+		TALLOC_FREE(c->oldcwd_fname);
+		TALLOC_FREE(c);
 		return status;
 	}
 
-	*pconn = conn;
-	*poldcwd = oldcwd;
-
+	*_c = c;
 	return NT_STATUS_OK;
 }
 
@@ -567,9 +620,8 @@ static bool parse_msdfs_symlink(TALLOC_CTX *ctx,
 
 static bool is_msdfs_link_internal(TALLOC_CTX *ctx,
 			connection_struct *conn,
-			const char *path,
-			char **pp_link_target,
-			SMB_STRUCT_STAT *sbufp)
+			struct smb_filename *smb_fname,
+			char **pp_link_target)
 {
 	int referral_len = 0;
 #if defined(HAVE_BROKEN_READLINK)
@@ -579,7 +631,6 @@ static bool is_msdfs_link_internal(TALLOC_CTX *ctx,
 #endif
 	size_t bufsize = 0;
 	char *link_target = NULL;
-	struct smb_filename smb_fname;
 
 	if (pp_link_target) {
 		bufsize = 1024;
@@ -593,33 +644,28 @@ static bool is_msdfs_link_internal(TALLOC_CTX *ctx,
 		link_target = link_target_buf;
 	}
 
-	ZERO_STRUCT(smb_fname);
-	smb_fname.base_name = discard_const_p(char, path);
-
-	if (SMB_VFS_LSTAT(conn, &smb_fname) != 0) {
+	if (SMB_VFS_LSTAT(conn, smb_fname) != 0) {
 		DEBUG(5,("is_msdfs_link_read_target: %s does not exist.\n",
-			path));
+			smb_fname->base_name));
 		goto err;
 	}
-	if (!S_ISLNK(smb_fname.st.st_ex_mode)) {
+	if (!S_ISLNK(smb_fname->st.st_ex_mode)) {
 		DEBUG(5,("is_msdfs_link_read_target: %s is not a link.\n",
-					path));
+			smb_fname->base_name));
 		goto err;
 	}
-	if (sbufp != NULL) {
-		*sbufp = smb_fname.st;
-	}
 
-	referral_len = SMB_VFS_READLINK(conn, path, link_target, bufsize - 1);
+	referral_len = SMB_VFS_READLINK(conn, smb_fname,
+				link_target, bufsize - 1);
 	if (referral_len == -1) {
 		DEBUG(0,("is_msdfs_link_read_target: Error reading "
 			"msdfs link %s: %s\n",
-			path, strerror(errno)));
+			smb_fname->base_name, strerror(errno)));
 		goto err;
 	}
 	link_target[referral_len] = '\0';
 
-	DEBUG(5,("is_msdfs_link_internal: %s -> %s\n",path,
+	DEBUG(5,("is_msdfs_link_internal: %s -> %s\n", smb_fname->base_name,
 				link_target));
 
 	if (!strnequal(link_target, "msdfs:", 6)) {
@@ -640,14 +686,12 @@ static bool is_msdfs_link_internal(TALLOC_CTX *ctx,
 **********************************************************************/
 
 bool is_msdfs_link(connection_struct *conn,
-		const char *path,
-		SMB_STRUCT_STAT *sbufp)
+		struct smb_filename *smb_fname)
 {
 	return is_msdfs_link_internal(talloc_tos(),
 					conn,
-					path,
-					NULL,
-					sbufp);
+					smb_fname,
+					NULL);
 }
 
 /*****************************************************************
@@ -706,8 +750,7 @@ static NTSTATUS dfs_path_lookup(TALLOC_CTX *ctx,
 
 	/* Optimization - check if we can redirect the whole path. */
 
-	if (is_msdfs_link_internal(ctx, conn, smb_fname->base_name,
-				   pp_targetpath, NULL)) {
+	if (is_msdfs_link_internal(ctx, conn, smb_fname, pp_targetpath)) {
 		/* XX_ALLOW_WCARD_XXX is called from search functions. */
 		if (ucf_flags &
 				(UCF_COND_ALLOW_WCARD_LCOMP|
@@ -770,8 +813,7 @@ static NTSTATUS dfs_path_lookup(TALLOC_CTX *ctx,
 		}
 
 		if (is_msdfs_link_internal(ctx, conn,
-					   smb_fname->base_name, pp_targetpath,
-					   NULL)) {
+					   smb_fname, pp_targetpath)) {
 			DEBUG(4, ("dfs_path_lookup: Redirecting %s because "
 				  "parent %s is dfs link\n", dfspath,
 				  smb_fname_str_dbg(smb_fname)));
@@ -962,15 +1004,17 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 			   int *consumedcntp,
 			   bool *self_referralp)
 {
-	struct connection_struct *conn;
+	TALLOC_CTX *frame = talloc_stackframe();
+	struct conn_struct_tos *c = NULL;
+	struct connection_struct *conn = NULL;
 	char *targetpath = NULL;
 	int snum;
 	NTSTATUS status = NT_STATUS_NOT_FOUND;
 	bool dummy;
-	struct dfs_path *pdp = talloc(ctx, struct dfs_path);
-	char *oldpath;
+	struct dfs_path *pdp = talloc_zero(frame, struct dfs_path);
 
 	if (!pdp) {
+		TALLOC_FREE(frame);
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -979,13 +1023,14 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 	status = parse_dfs_path(NULL, dfs_path, False, allow_broken_path,
 				pdp, &dummy);
 	if (!NT_STATUS_IS_OK(status)) {
+		TALLOC_FREE(frame);
 		return status;
 	}
 
 	jucn->service_name = talloc_strdup(ctx, pdp->servicename);
 	jucn->volume_name = talloc_strdup(ctx, pdp->reqpath);
 	if (!jucn->service_name || !jucn->volume_name) {
-		TALLOC_FREE(pdp);
+		TALLOC_FREE(frame);
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -994,15 +1039,17 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 	if(snum < 0) {
 		char *service_name = NULL;
 		if ((snum = find_service(ctx, jucn->service_name, &service_name)) < 0) {
+			TALLOC_FREE(frame);
 			return NT_STATUS_NOT_FOUND;
 		}
 		if (!service_name) {
+			TALLOC_FREE(frame);
 			return NT_STATUS_NO_MEMORY;
 		}
 		TALLOC_FREE(jucn->service_name);
 		jucn->service_name = talloc_strdup(ctx, service_name);
 		if (!jucn->service_name) {
-			TALLOC_FREE(pdp);
+			TALLOC_FREE(frame);
 			return NT_STATUS_NO_MEMORY;
 		}
 	}
@@ -1011,7 +1058,7 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 		DEBUG(3,("get_referred_path: |%s| in dfs path %s is not "
 			"a dfs root.\n",
 			pdp->servicename, dfs_path));
-		TALLOC_FREE(pdp);
+		TALLOC_FREE(frame);
 		return NT_STATUS_NOT_FOUND;
 	}
 
@@ -1029,7 +1076,7 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 		int refcount;
 
 		if (*lp_msdfs_proxy(talloc_tos(), snum) == '\0') {
-			TALLOC_FREE(pdp);
+			TALLOC_FREE(frame);
 			return self_ref(ctx,
 					dfs_path,
 					jucn,
@@ -1042,35 +1089,34 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
  		 * the configured target share.
  		 */
 
-		tmp = talloc_asprintf(talloc_tos(), "msdfs:%s",
-				      lp_msdfs_proxy(talloc_tos(), snum));
+		tmp = talloc_asprintf(frame, "msdfs:%s",
+				      lp_msdfs_proxy(frame, snum));
 		if (tmp == NULL) {
-			TALLOC_FREE(pdp);
+			TALLOC_FREE(frame);
 			return NT_STATUS_NO_MEMORY;
 		}
 
 		if (!parse_msdfs_symlink(ctx, snum, tmp, &ref, &refcount)) {
-			TALLOC_FREE(tmp);
-			TALLOC_FREE(pdp);
+			TALLOC_FREE(frame);
 			return NT_STATUS_INVALID_PARAMETER;
 		}
-		TALLOC_FREE(tmp);
 		jucn->referral_count = refcount;
 		jucn->referral_list = ref;
 		*consumedcntp = strlen(dfs_path);
-		TALLOC_FREE(pdp);
+		TALLOC_FREE(frame);
 		return NT_STATUS_OK;
 	}
 
-	status = create_conn_struct_cwd(ctx,
-					server_event_context(),
-					server_messaging_context(),
-					&conn, snum,
-					lp_path(talloc_tos(), snum), NULL, &oldpath);
+	status = create_conn_struct_tos_cwd(global_messaging_context(),
+					    snum,
+					    lp_path(frame, snum),
+					    NULL,
+					    &c);
 	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(pdp);
+		TALLOC_FREE(frame);
 		return status;
 	}
+	conn = c->conn;
 
 	/*
 	 * TODO
@@ -1082,7 +1128,7 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 		conn->sconn->remote_address =
 			tsocket_address_copy(remote_address, conn->sconn);
 		if (conn->sconn->remote_address == NULL) {
-			TALLOC_FREE(pdp);
+			TALLOC_FREE(frame);
 			return NT_STATUS_NO_MEMORY;
 		}
 	}
@@ -1090,7 +1136,7 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 		conn->sconn->local_address =
 			tsocket_address_copy(local_address, conn->sconn);
 		if (conn->sconn->local_address == NULL) {
-			TALLOC_FREE(pdp);
+			TALLOC_FREE(frame);
 			return NT_STATUS_NO_MEMORY;
 		}
 	}
@@ -1132,10 +1178,7 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 
 	status = NT_STATUS_OK;
  err_exit:
-	vfs_ChDir(conn, oldpath);
-	SMB_VFS_DISCONNECT(conn);
-	conn_free(conn);
-	TALLOC_FREE(pdp);
+	TALLOC_FREE(frame);
 	return status;
 }
 
@@ -1265,51 +1308,56 @@ bool create_junction(TALLOC_CTX *ctx,
  Forms a valid Unix pathname from the junction
  **********************************************************************/
 
-static bool junction_to_local_path(const struct junction_map *jucn,
-				   char **pp_path_out,
-				   connection_struct **conn_out,
-				   char **oldpath)
+static bool junction_to_local_path_tos(const struct junction_map *jucn,
+				       char **pp_path_out,
+				       connection_struct **conn_out)
 {
+	struct conn_struct_tos *c = NULL;
 	int snum;
+	char *path_out = NULL;
 	NTSTATUS status;
 
 	snum = lp_servicenumber(jucn->service_name);
 	if(snum < 0) {
 		return False;
 	}
-	status = create_conn_struct_cwd(talloc_tos(),
-					server_event_context(),
-					server_messaging_context(),
-					conn_out,
-					snum, lp_path(talloc_tos(), snum), NULL, oldpath);
+	status = create_conn_struct_tos_cwd(global_messaging_context(),
+					    snum,
+					    lp_path(talloc_tos(), snum),
+					    NULL,
+					    &c);
 	if (!NT_STATUS_IS_OK(status)) {
 		return False;
 	}
 
-	*pp_path_out = talloc_asprintf(*conn_out,
+	path_out = talloc_asprintf(c,
 			"%s/%s",
 			lp_path(talloc_tos(), snum),
 			jucn->volume_name);
-	if (!*pp_path_out) {
-		vfs_ChDir(*conn_out, *oldpath);
-		SMB_VFS_DISCONNECT(*conn_out);
-		conn_free(*conn_out);
+	if (path_out == NULL) {
+		TALLOC_FREE(c);
 		return False;
 	}
+	*pp_path_out = path_out;
+	*conn_out = c->conn;
 	return True;
 }
 
 bool create_msdfs_link(const struct junction_map *jucn)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	char *path = NULL;
-	char *cwd;
 	char *msdfs_link = NULL;
 	connection_struct *conn;
 	int i=0;
 	bool insert_comma = False;
 	bool ret = False;
+	struct smb_filename *smb_fname = NULL;
+	bool ok;
 
-	if(!junction_to_local_path(jucn, &path, &conn, &cwd)) {
+	ok = junction_to_local_path_tos(jucn, &path, &conn);
+	if (!ok) {
+		TALLOC_FREE(frame);
 		return False;
 	}
 
@@ -1350,27 +1398,24 @@ bool create_msdfs_link(const struct junction_map *jucn)
 	DEBUG(5,("create_msdfs_link: Creating new msdfs link: %s -> %s\n",
 		path, msdfs_link));
 
-	if(SMB_VFS_SYMLINK(conn, msdfs_link, path) < 0) {
+	smb_fname = synthetic_smb_fname(frame,
+				path,
+				NULL,
+				NULL,
+				0);
+	if (smb_fname == NULL) {
+		errno = ENOMEM;
+		goto out;
+	}
+
+	if(SMB_VFS_SYMLINK(conn, msdfs_link, smb_fname) < 0) {
 		if (errno == EEXIST) {
-			struct smb_filename *smb_fname;
-
-			smb_fname = synthetic_smb_fname(talloc_tos(),
-						path,
-						NULL,
-						NULL,
-						0);
-			if (smb_fname == NULL) {
-				errno = ENOMEM;
-				goto out;
-			}
-
 			if(SMB_VFS_UNLINK(conn, smb_fname)!=0) {
 				TALLOC_FREE(smb_fname);
 				goto out;
 			}
-			TALLOC_FREE(smb_fname);
 		}
-		if (SMB_VFS_SYMLINK(conn, msdfs_link, path) < 0) {
+		if (SMB_VFS_SYMLINK(conn, msdfs_link, smb_fname) < 0) {
 			DEBUG(1,("create_msdfs_link: symlink failed "
 				 "%s -> %s\nError: %s\n",
 				 path, msdfs_link, strerror(errno)));
@@ -1381,30 +1426,32 @@ bool create_msdfs_link(const struct junction_map *jucn)
 	ret = True;
 
 out:
-	vfs_ChDir(conn, cwd);
-	SMB_VFS_DISCONNECT(conn);
-	conn_free(conn);
+	TALLOC_FREE(frame);
 	return ret;
 }
 
 bool remove_msdfs_link(const struct junction_map *jucn)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	char *path = NULL;
-	char *cwd;
 	connection_struct *conn;
 	bool ret = False;
 	struct smb_filename *smb_fname;
+	bool ok;
 
-	if (!junction_to_local_path(jucn, &path, &conn, &cwd)) {
+	ok = junction_to_local_path_tos(jucn, &path, &conn);
+	if (!ok) {
+		TALLOC_FREE(frame);
 		return false;
 	}
 
-	smb_fname = synthetic_smb_fname(talloc_tos(),
+	smb_fname = synthetic_smb_fname(frame,
 					path,
 					NULL,
 					NULL,
 					0);
 	if (smb_fname == NULL) {
+		TALLOC_FREE(frame);
 		errno = ENOMEM;
 		return false;
 	}
@@ -1413,10 +1460,7 @@ bool remove_msdfs_link(const struct junction_map *jucn)
 		ret = True;
 	}
 
-	TALLOC_FREE(smb_fname);
-	vfs_ChDir(conn, cwd);
-	SMB_VFS_DISCONNECT(conn);
-	conn_free(conn);
+	TALLOC_FREE(frame);
 	return ret;
 }
 
@@ -1426,18 +1470,20 @@ bool remove_msdfs_link(const struct junction_map *jucn)
 
 static int count_dfs_links(TALLOC_CTX *ctx, int snum)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	size_t cnt = 0;
 	DIR *dirp = NULL;
 	const char *dname = NULL;
 	char *talloced = NULL;
-	const char *connect_path = lp_path(talloc_tos(), snum);
-	const char *msdfs_proxy = lp_msdfs_proxy(talloc_tos(), snum);
-	connection_struct *conn;
+	const char *connect_path = lp_path(frame, snum);
+	const char *msdfs_proxy = lp_msdfs_proxy(frame, snum);
+	struct conn_struct_tos *c = NULL;
+	connection_struct *conn = NULL;
 	NTSTATUS status;
-	char *cwd;
 	struct smb_filename *smb_fname = NULL;
 
 	if(*connect_path == '\0') {
+		TALLOC_FREE(frame);
 		return 0;
 	}
 
@@ -1445,16 +1491,18 @@ static int count_dfs_links(TALLOC_CTX *ctx, int snum)
 	 * Fake up a connection struct for the VFS layer.
 	 */
 
-	status = create_conn_struct_cwd(talloc_tos(),
-					server_event_context(),
-					server_messaging_context(),
-					&conn,
-					snum, connect_path, NULL, &cwd);
+	status = create_conn_struct_tos_cwd(global_messaging_context(),
+					    snum,
+					    connect_path,
+					    NULL,
+					    &c);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(3, ("create_conn_struct failed: %s\n",
 			  nt_errstr(status)));
+		TALLOC_FREE(frame);
 		return 0;
 	}
+	conn = c->conn;
 
 	/* Count a link for the msdfs root - convention */
 	cnt = 1;
@@ -1464,7 +1512,7 @@ static int count_dfs_links(TALLOC_CTX *ctx, int snum)
 		goto out;
 	}
 
-	smb_fname = synthetic_smb_fname(talloc_tos(),
+	smb_fname = synthetic_smb_fname(frame,
 					".",
 					NULL,
 					NULL,
@@ -1481,21 +1529,26 @@ static int count_dfs_links(TALLOC_CTX *ctx, int snum)
 
 	while ((dname = vfs_readdirname(conn, dirp, NULL, &talloced))
 	       != NULL) {
-		if (is_msdfs_link(conn,
-				dname,
-				NULL)) {
+		struct smb_filename *smb_dname =
+			synthetic_smb_fname(frame,
+					dname,
+					NULL,
+					NULL,
+					0);
+		if (smb_dname == NULL) {
+			goto out;
+		}
+		if (is_msdfs_link(conn, smb_dname)) {
 			cnt++;
 		}
 		TALLOC_FREE(talloced);
+		TALLOC_FREE(smb_dname);
 	}
 
 	SMB_VFS_CLOSEDIR(conn,dirp);
 
 out:
-	TALLOC_FREE(smb_fname);
-	vfs_ChDir(conn, cwd);
-	SMB_VFS_DISCONNECT(conn);
-	conn_free(conn);
+	TALLOC_FREE(frame);
 	return cnt;
 }
 
@@ -1507,24 +1560,27 @@ static int form_junctions(TALLOC_CTX *ctx,
 				struct junction_map *jucn,
 				size_t jn_remain)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	size_t cnt = 0;
 	DIR *dirp = NULL;
 	const char *dname = NULL;
 	char *talloced = NULL;
-	const char *connect_path = lp_path(talloc_tos(), snum);
-	char *service_name = lp_servicename(talloc_tos(), snum);
-	const char *msdfs_proxy = lp_msdfs_proxy(talloc_tos(), snum);
-	connection_struct *conn;
+	const char *connect_path = lp_path(frame, snum);
+	char *service_name = lp_servicename(frame, snum);
+	const char *msdfs_proxy = lp_msdfs_proxy(frame, snum);
+	struct conn_struct_tos *c = NULL;
+	connection_struct *conn = NULL;
 	struct referral *ref = NULL;
-	char *cwd;
 	struct smb_filename *smb_fname = NULL;
 	NTSTATUS status;
 
 	if (jn_remain == 0) {
+		TALLOC_FREE(frame);
 		return 0;
 	}
 
 	if(*connect_path == '\0') {
+		TALLOC_FREE(frame);
 		return 0;
 	}
 
@@ -1532,16 +1588,18 @@ static int form_junctions(TALLOC_CTX *ctx,
 	 * Fake up a connection struct for the VFS layer.
 	 */
 
-	status = create_conn_struct_cwd(ctx,
-					server_event_context(),
-					server_messaging_context(),
-					&conn, snum, connect_path, NULL,
-					&cwd);
+	status = create_conn_struct_tos_cwd(global_messaging_context(),
+					    snum,
+					    connect_path,
+					    NULL,
+					    &c);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(3, ("create_conn_struct failed: %s\n",
 			  nt_errstr(status)));
+		TALLOC_FREE(frame);
 		return 0;
 	}
+	conn = c->conn;
 
 	/* form a junction for the msdfs root - convention
 	   DO NOT REMOVE THIS: NT clients will not work with us
@@ -1582,7 +1640,7 @@ static int form_junctions(TALLOC_CTX *ctx,
 		goto out;
 	}
 
-	smb_fname = synthetic_smb_fname(talloc_tos(),
+	smb_fname = synthetic_smb_fname(frame,
 					".",
 					NULL,
 					NULL,
@@ -1600,16 +1658,26 @@ static int form_junctions(TALLOC_CTX *ctx,
 	while ((dname = vfs_readdirname(conn, dirp, NULL, &talloced))
 	       != NULL) {
 		char *link_target = NULL;
+		struct smb_filename *smb_dname = NULL;
+
 		if (cnt >= jn_remain) {
 			DEBUG(2, ("form_junctions: ran out of MSDFS "
 				"junction slots"));
 			TALLOC_FREE(talloced);
 			goto out;
 		}
+		smb_dname = synthetic_smb_fname(talloc_tos(),
+				dname,
+				NULL,
+				NULL,
+				0);
+		if (smb_dname == NULL) {
+			TALLOC_FREE(talloced);
+			goto out;
+		}
 		if (is_msdfs_link_internal(ctx,
 					conn,
-					dname, &link_target,
-					NULL)) {
+					smb_dname, &link_target)) {
 			if (parse_msdfs_symlink(ctx, snum,
 					link_target,
 					&jucn[cnt].referral_list,
@@ -1630,6 +1698,7 @@ static int form_junctions(TALLOC_CTX *ctx,
 			TALLOC_FREE(link_target);
 		}
 		TALLOC_FREE(talloced);
+		TALLOC_FREE(smb_dname);
 	}
 
 out:
@@ -1638,9 +1707,7 @@ out:
 		SMB_VFS_CLOSEDIR(conn,dirp);
 	}
 
-	TALLOC_FREE(smb_fname);
-	vfs_ChDir(conn, cwd);
-	conn_free(conn);
+	TALLOC_FREE(frame);
 	return cnt;
 }
 
@@ -1695,35 +1762,27 @@ struct junction_map *enum_msdfs_links(TALLOC_CTX *ctx, size_t *p_num_jn)
 
 NTSTATUS resolve_dfspath_wcard(TALLOC_CTX *ctx,
 				connection_struct *conn,
-				bool dfs_pathnames,
 				const char *name_in,
 				uint32_t ucf_flags,
 				bool allow_broken_path,
 				char **pp_name_out,
 				bool *ppath_contains_wcard)
 {
-	bool path_contains_wcard;
+	bool path_contains_wcard = false;
 	NTSTATUS status = NT_STATUS_OK;
 
-	if (dfs_pathnames) {
-		status = dfs_redirect(ctx,
-					conn,
-					name_in,
-					ucf_flags,
-					allow_broken_path,
-					pp_name_out,
-					&path_contains_wcard);
+	status = dfs_redirect(ctx,
+				conn,
+				name_in,
+				ucf_flags,
+				allow_broken_path,
+				pp_name_out,
+				&path_contains_wcard);
 
-		if (NT_STATUS_IS_OK(status) && ppath_contains_wcard != NULL) {
-			*ppath_contains_wcard = path_contains_wcard;
-		}
-	} else {
-		/*
-		 * Cheat and just return a copy of the in ptr.
-		 * Once srvstr_get_path() uses talloc it'll
-		 * be a talloced ptr anyway.
-		 */
-		*pp_name_out = discard_const_p(char, name_in);
+	if (NT_STATUS_IS_OK(status) &&
+				ppath_contains_wcard != NULL &&
+				path_contains_wcard) {
+		*ppath_contains_wcard = path_contains_wcard;
 	}
 	return status;
 }
