@@ -23,6 +23,7 @@
 #include "system/passwd.h"
 #include "passdb/lookup_sid.h"
 #include "libsmb/libsmb.h"
+#include "libcli/security/dom_sid.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_QUOTA
@@ -56,24 +57,14 @@ static uint64_t limit_unix2nt(uint64_t in, uint64_t bsize)
 	return ret;
 }
 
-static uint64_t limit_blk2inodes(uint64_t in)
-{
-	uint64_t ret = (uint64_t)0;
-	
-	ret = (uint64_t)(in/2);
-	
-	if (ret == 0 && in != 0)
-		ret = (uint64_t)1;
-
-	return ret;	
-}
-
 NTSTATUS vfs_get_ntquota(files_struct *fsp, enum SMB_QUOTA_TYPE qtype,
 			 struct dom_sid *psid, SMB_NTQUOTA_STRUCT *qt)
 {
 	int ret;
 	SMB_DISK_QUOTA D;
 	unid_t id;
+	struct smb_filename *smb_fname_cwd = NULL;
+	int saved_errno = 0;
 
 	ZERO_STRUCT(D);
 
@@ -86,12 +77,29 @@ NTSTATUS vfs_get_ntquota(files_struct *fsp, enum SMB_QUOTA_TYPE qtype,
 	id.uid = -1;
 
 	if (psid && !sid_to_uid(psid, &id.uid)) {
+		struct dom_sid_buf buf;
 		DEBUG(0,("sid_to_uid: failed, SID[%s]\n",
-			 sid_string_dbg(psid)));
+			 dom_sid_str_buf(psid, &buf)));
 		return NT_STATUS_NO_SUCH_USER;
 	}
 
-	ret = SMB_VFS_GET_QUOTA(fsp->conn, ".", qtype, id, &D);
+	smb_fname_cwd = synthetic_smb_fname(talloc_tos(),
+				".",
+				NULL,
+				NULL,
+				0);
+	if (smb_fname_cwd == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	ret = SMB_VFS_GET_QUOTA(fsp->conn, smb_fname_cwd, qtype, id, &D);
+	if (ret == -1) {
+		saved_errno = errno;
+	}
+	TALLOC_FREE(smb_fname_cwd);
+	if (saved_errno != 0) {
+		errno = saved_errno;
+	}
 
 	if (psid)
 		qt->sid    = *psid;
@@ -126,12 +134,10 @@ int vfs_set_ntquota(files_struct *fsp, enum SMB_QUOTA_TYPE qtype, struct dom_sid
 	D.hardlimit = limit_nt2unix(qt->hardlim,D.bsize);
 	D.qflags     = qt->qflags;
 
-	D.isoftlimit = limit_blk2inodes(D.softlimit);
-	D.ihardlimit = limit_blk2inodes(D.hardlimit);
-
 	if (psid && !sid_to_uid(psid, &id.uid)) {
+		struct dom_sid_buf buf;
 		DEBUG(0,("sid_to_uid: failed, SID[%s]\n",
-			 sid_string_dbg(psid)));
+			 dom_sid_str_buf(psid, &buf)));
 	}
 
 	ret = SMB_VFS_SET_QUOTA(fsp->conn, qtype, id, &D);
@@ -139,7 +145,7 @@ int vfs_set_ntquota(files_struct *fsp, enum SMB_QUOTA_TYPE qtype, struct dom_sid
 	return ret;
 }
 
-static bool allready_in_quota_list(SMB_NTQUOTA_LIST *qt_list, uid_t uid)
+static bool already_in_quota_list(SMB_NTQUOTA_LIST *qt_list, uid_t uid)
 {
 	SMB_NTQUOTA_LIST *tmp_list = NULL;
 	
@@ -175,12 +181,13 @@ int vfs_get_user_ntquota_list(files_struct *fsp, SMB_NTQUOTA_LIST **qt_list)
 		SMB_NTQUOTA_STRUCT tmp_qt;
 		SMB_NTQUOTA_LIST *tmp_list_ent;
 		struct dom_sid	sid;
+		struct dom_sid_buf buf;
 		NTSTATUS status;
 
 		ZERO_STRUCT(tmp_qt);
 
-		if (allready_in_quota_list((*qt_list),usr->pw_uid)) {
-			DEBUG(5,("record for uid[%ld] allready in the list\n",(long)usr->pw_uid));
+		if (already_in_quota_list((*qt_list),usr->pw_uid)) {
+			DEBUG(5,("record for uid[%ld] already in the list\n",(long)usr->pw_uid));
 			continue;
 		}
 
@@ -195,13 +202,14 @@ int vfs_get_user_ntquota_list(files_struct *fsp, SMB_NTQUOTA_LIST **qt_list)
 		}
 		if (tmp_qt.softlim == 0 && tmp_qt.hardlim == 0) {
 			DEBUG(5,("no quota entry for sid[%s] path[%s]\n",
-				 sid_string_dbg(&sid),
+				 dom_sid_str_buf(&sid, &buf),
 				 fsp->conn->connectpath));
 			continue;
 		}
 
 		DEBUG(15,("quota entry for id[%s] path[%s]\n",
-			  sid_string_dbg(&sid), fsp->conn->connectpath));
+			  dom_sid_str_buf(&sid, &buf),
+			  fsp->conn->connectpath));
 
 		if ((tmp_list_ent=talloc_zero(mem_ctx,SMB_NTQUOTA_LIST))==NULL) {
 			DEBUG(0,("TALLOC_ZERO() failed\n"));
