@@ -1395,6 +1395,7 @@ void reply_ntcancel(struct smb_request *req)
 {
 	struct smbXsrv_connection *xconn = req->xconn;
 	struct smbd_server_connection *sconn = req->sconn;
+	bool found;
 
 	/*
 	 * Go through and cancel any pending change notifies.
@@ -1402,8 +1403,10 @@ void reply_ntcancel(struct smb_request *req)
 
 	START_PROFILE(SMBntcancel);
 	srv_cancel_sign_response(xconn);
-	remove_pending_change_notify_requests_by_mid(sconn, req->mid);
-	remove_pending_lock_requests_by_mid_smb1(sconn, req->mid);
+	found = remove_pending_change_notify_requests_by_mid(sconn, req->mid);
+	if (!found) {
+		smbd_smb1_brl_finish_by_mid(sconn, req->mid);
+	}
 
 	DEBUG(3,("reply_ntcancel: cancel called on mid = %llu.\n",
 		(unsigned long long)req->mid));
@@ -1806,8 +1809,10 @@ static void call_nt_transact_notify_change(connection_struct *conn,
 
 	if (fsp->notify == NULL) {
 
-		status = change_notify_create(fsp, filter, recursive);
-
+		status = change_notify_create(fsp,
+					      max_param_count,
+					      filter,
+					      recursive);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(10, ("change_notify_create returned %s\n",
 				   nt_errstr(status)));
@@ -2340,9 +2345,9 @@ static enum ndr_err_code fill_qtlist_from_sids(TALLOC_CTX *mem_ctx,
 
 		ok = sid_to_uid(&sids[i], &list_item->uid);
 		if (!ok) {
-			char buf[DOM_SID_STR_BUFLEN];
-			dom_sid_string_buf(&sids[i], buf, sizeof(buf));
-			DBG_WARNING("Could not convert SID %s to uid\n", buf);
+			struct dom_sid_buf buf;
+			DBG_WARNING("Could not convert SID %s to uid\n",
+				    dom_sid_str_buf(&sids[i], &buf));
 			/* No idea what to return here... */
 			return NDR_ERR_INVALID_POINTER;
 		}
@@ -2472,10 +2477,11 @@ static enum ndr_err_code extract_sids_from_buf(TALLOC_CTX *mem_ctx,
 		*num = i;
 
 		for (iter = sid_list, i = 0; iter; iter = iter->next, i++) {
+			struct dom_sid_buf buf;
 			(*sids)[i] = iter->sid;
 			DBG_DEBUG("quota SID[%u] %s\n",
 				(unsigned int)i,
-				sid_string_dbg(&iter->sid));
+				dom_sid_str_buf(&iter->sid, &buf));
 		}
 	}
 	err = NDR_ERR_SUCCESS;
