@@ -26,6 +26,7 @@
 #include <Python.h>
 #include "includes.h"
 #include "python/py3compat.h"
+#include "python/modules.h"
 #include "smbd/smbd.h"
 #include "libcli/util/pyerrors.h"
 #include "librpc/rpc/pyrpc_util.h"
@@ -88,27 +89,19 @@ static int set_sys_acl_conn(const char *fname,
 {
 	int ret;
 	struct smb_filename *smb_fname = NULL;
-	mode_t saved_umask;
 
 	TALLOC_CTX *frame = talloc_stackframe();
 
-	/* we want total control over the permissions on created files,
-	   so set our umask to 0 */
-	saved_umask = umask(0);
-
 	smb_fname = synthetic_smb_fname_split(frame,
 					fname,
 					lp_posix_pathnames());
 	if (smb_fname == NULL) {
 		TALLOC_FREE(frame);
-		umask(saved_umask);
 		return -1;
 	}
 
 	ret = SMB_VFS_SYS_ACL_SET_FILE( conn, smb_fname, acltype, theacl);
 
-	umask(saved_umask);
-
 	TALLOC_FREE(frame);
 	return ret;
 }
@@ -135,23 +128,27 @@ static NTSTATUS init_files_struct(TALLOC_CTX *mem_ctx,
 	}
 	fsp->conn = conn;
 
-	/* we want total control over the permissions on created files,
-	   so set our umask to 0 */
-	saved_umask = umask(0);
-
 	smb_fname = synthetic_smb_fname_split(fsp,
 					      fname,
 					      lp_posix_pathnames());
 	if (smb_fname == NULL) {
-		umask(saved_umask);
 		return NT_STATUS_NO_MEMORY;
 	}
 
 	fsp->fsp_name = smb_fname;
+
+	/*
+	 * we want total control over the permissions on created files,
+	 * so set our umask to 0 (this matters if flags contains O_CREAT)
+	 */
+	saved_umask = umask(0);
+
 	fsp->fh->fd = SMB_VFS_OPEN(conn, smb_fname, fsp, flags, 00644);
+
+	umask(saved_umask);
+
 	if (fsp->fh->fd == -1) {
 		int err = errno;
-		umask(saved_umask);
 		if (err == ENOENT) {
 			return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 		}
@@ -164,7 +161,6 @@ static NTSTATUS init_files_struct(TALLOC_CTX *mem_ctx,
 		DEBUG(0,("Error doing fstat on open file %s (%s)\n",
 			 smb_fname_str_dbg(smb_fname),
 			 strerror(errno) ));
-		umask(saved_umask);
 		return map_nt_error_from_unix(errno);
 	}
 
@@ -454,7 +450,6 @@ static PyObject *py_smbd_chown(PyObject *self, PyObject *args, PyObject *kwargs)
 	char *fname, *service = NULL;
 	int uid, gid;
 	TALLOC_CTX *frame;
-	mode_t saved_umask;
 	struct smb_filename *smb_fname = NULL;
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "sii|z",
@@ -470,10 +465,6 @@ static PyObject *py_smbd_chown(PyObject *self, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 
-	/* we want total control over the permissions on created files,
-	   so set our umask to 0 */
-	saved_umask = umask(0);
-
 	smb_fname = synthetic_smb_fname(talloc_tos(),
 					fname,
 					NULL,
@@ -481,7 +472,6 @@ static PyObject *py_smbd_chown(PyObject *self, PyObject *args, PyObject *kwargs)
 					lp_posix_pathnames() ?
 						SMB_FILENAME_POSIX_PATH : 0);
 	if (smb_fname == NULL) {
-		umask(saved_umask);
 		TALLOC_FREE(frame);
 		errno = ENOMEM;
 		return PyErr_SetFromErrno(PyExc_OSError);
@@ -489,14 +479,11 @@ static PyObject *py_smbd_chown(PyObject *self, PyObject *args, PyObject *kwargs)
 
 	ret = SMB_VFS_CHOWN(conn, smb_fname, uid, gid);
 	if (ret != 0) {
-		umask(saved_umask);
 		TALLOC_FREE(frame);
 		errno = ret;
 		return PyErr_SetFromErrno(PyExc_OSError);
 	}
 
-	umask(saved_umask);
-
 	TALLOC_FREE(frame);
 
 	Py_RETURN_NONE;
@@ -552,7 +539,8 @@ static PyObject *py_smbd_unlink(PyObject *self, PyObject *args, PyObject *kwargs
 /*
   check if we have ACL support
  */
-static PyObject *py_smbd_have_posix_acls(PyObject *self)
+static PyObject *py_smbd_have_posix_acls(PyObject *self,
+		PyObject *Py_UNUSED(ignored))
 {
 #ifdef HAVE_POSIX_ACLS
 	return PyBool_FromLong(true);
@@ -796,6 +784,8 @@ static PyObject *py_smbd_mkdir(PyObject *self, PyObject *args, PyObject *kwargs)
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct connection_struct *conn = NULL;
 	struct smb_filename *smb_fname = NULL;
+	int ret;
+	mode_t saved_umask;
 
 	if (!PyArg_ParseTupleAndKeywords(args,
 					 kwargs,
@@ -826,8 +816,15 @@ static PyObject *py_smbd_mkdir(PyObject *self, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 
+	/* we want total control over the permissions on created files,
+	   so set our umask to 0 */
+	saved_umask = umask(0);
 
-	if (SMB_VFS_MKDIR(conn, smb_fname, 00755) == -1) {
+	ret = SMB_VFS_MKDIR(conn, smb_fname, 00755);
+
+	umask(saved_umask);
+
+	if (ret == -1) {
 		DBG_ERR("mkdir error=%d (%s)\n", errno, strerror(errno));
 		TALLOC_FREE(frame);
 		return NULL;
@@ -887,31 +884,40 @@ static PyMethodDef py_smbd_methods[] = {
 		(PyCFunction)py_smbd_have_posix_acls, METH_NOARGS,
 		NULL },
 	{ "set_simple_acl",
-		(PyCFunction)py_smbd_set_simple_acl, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_set_simple_acl),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "set_nt_acl",
-		(PyCFunction)py_smbd_set_nt_acl, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_set_nt_acl),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "get_nt_acl",
-		(PyCFunction)py_smbd_get_nt_acl, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_get_nt_acl),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "get_sys_acl",
-		(PyCFunction)py_smbd_get_sys_acl, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_get_sys_acl),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "set_sys_acl",
-		(PyCFunction)py_smbd_set_sys_acl, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_set_sys_acl),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "chown",
-		(PyCFunction)py_smbd_chown, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_chown),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "unlink",
-		(PyCFunction)py_smbd_unlink, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_unlink),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "mkdir",
-		(PyCFunction)py_smbd_mkdir, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_mkdir),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ "create_file",
-		(PyCFunction)py_smbd_create_file, METH_VARARGS|METH_KEYWORDS,
+		PY_DISCARD_FUNC_SIG(PyCFunction, py_smbd_create_file),
+		METH_VARARGS|METH_KEYWORDS,
 		NULL },
 	{ NULL }
 };
