@@ -70,7 +70,7 @@ NTSTATUS smbd_smb2_request_process_close(struct smbd_smb2_request *req)
 		return smbd_smb2_request_error(req, NT_STATUS_FILE_CLOSED);
 	}
 
-	subreq = smbd_smb2_close_send(req, req->ev_ctx,
+	subreq = smbd_smb2_close_send(req, req->sconn->ev_ctx,
 				      req, in_fsp, in_flags);
 	if (subreq == NULL) {
 		return smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
@@ -313,6 +313,7 @@ static struct tevent_req *smbd_smb2_close_send(TALLOC_CTX *mem_ctx,
 {
 	struct tevent_req *req;
 	struct smbd_smb2_close_state *state;
+	unsigned i;
 	NTSTATUS status;
 
 	req = tevent_req_create(mem_ctx, &state,
@@ -324,6 +325,17 @@ static struct tevent_req *smbd_smb2_close_send(TALLOC_CTX *mem_ctx,
 	state->in_fsp = in_fsp;
 	state->in_flags = in_flags;
 
+	in_fsp->closing = true;
+
+	i = 0;
+	while (i < in_fsp->num_aio_requests) {
+		bool ok = tevent_req_cancel(in_fsp->aio_requests[i]);
+		if (ok) {
+			continue;
+		}
+		i += 1;
+	}
+
 	if (in_fsp->num_aio_requests != 0) {
 		in_fsp->deferred_close = tevent_wait_send(in_fsp, ev);
 		if (tevent_req_nomem(in_fsp->deferred_close, req)) {
