@@ -228,7 +228,7 @@ NTSTATUS smbd_smb2_request_process_create(struct smbd_smb2_request *smb2req)
 	}
 
 	tsubreq = smbd_smb2_create_send(smb2req,
-				       smb2req->ev_ctx,
+				       smb2req->sconn->ev_ctx,
 				       smb2req,
 				       in_oplock_level,
 				       in_impersonation_level,
@@ -1258,8 +1258,6 @@ static void smbd_smb2_create_before_exec(struct tevent_req *req)
 			}
 		}
 	}
-
-	return;
 }
 
 static void smbd_smb2_create_after_exec(struct tevent_req *req)
@@ -1276,6 +1274,9 @@ static void smbd_smb2_create_after_exec(struct tevent_req *req)
 	DEBUG(10, ("smbd_smb2_create_send: "
 		   "response construction phase\n"));
 
+	state->out_file_attributes = dos_mode(state->result->conn,
+					      state->result->fsp_name);
+
 	if (state->mxac != NULL) {
 		NTTIME last_write_time;
 
@@ -1391,8 +1392,9 @@ static void smbd_smb2_create_after_exec(struct tevent_req *req)
 
 	if (state->qfid != NULL) {
 		uint8_t p[32];
-		uint64_t file_index = get_FileIndex(state->result->conn,
-						    &state->result->fsp_name->st);
+		uint64_t file_id = SMB_VFS_FS_FILE_ID(
+			state->result->conn,
+			&state->result->fsp_name->st);
 		DATA_BLOB blob = data_blob_const(p, sizeof(p));
 
 		ZERO_STRUCT(p);
@@ -1401,7 +1403,7 @@ static void smbd_smb2_create_after_exec(struct tevent_req *req)
 		   the MS plugfest. The first 8 bytes are the "volume index"
 		   == inode, the second 8 bytes are the "volume id",
 		   == dev. This will be updated in the SMB2 doc. */
-		SBVAL(p, 0, file_index);
+		SBVAL(p, 0, file_id);
 		SIVAL(p, 8, state->result->fsp_name->st.st_ex_dev);/* FileIndexHigh */
 
 		status = smb2_create_blob_add(state->out_context_blobs,
@@ -1473,8 +1475,6 @@ static void smbd_smb2_create_finish(struct tevent_req *req)
 		state->out_create_action = state->info;
 	}
 	result->op->create_action = state->out_create_action;
-	state->out_file_attributes = dos_mode(result->conn,
-					   result->fsp_name);
 
 	state->out_creation_ts = get_create_timespec(smb1req->conn,
 					result, result->fsp_name);
@@ -1504,7 +1504,6 @@ static void smbd_smb2_create_finish(struct tevent_req *req)
 
 	tevent_req_done(req);
 	tevent_req_post(req, state->ev);
-	return;
 }
 
 static NTSTATUS smbd_smb2_create_recv(struct tevent_req *req,
@@ -1759,7 +1758,7 @@ bool schedule_deferred_open_message_smb2(
 		(unsigned long long)mid ));
 
 	tevent_schedule_immediate(state->im,
-			smb2req->ev_ctx,
+			smb2req->sconn->ev_ctx,
 			smbd_smb2_create_request_dispatch_immediate,
 			smb2req);
 
@@ -1791,7 +1790,7 @@ static bool smbd_smb2_create_cancel(struct tevent_req *req)
 
 	remove_deferred_open_message_smb2_internal(smb2req, mid);
 
-	tevent_req_defer_callback(req, smb2req->ev_ctx);
+	tevent_req_defer_callback(req, smb2req->sconn->ev_ctx);
 	tevent_req_nterror(req, NT_STATUS_CANCELLED);
 	return true;
 }
