@@ -611,7 +611,7 @@ static void do_per_share_checks(int s)
 			"excludes octal 010 (S_IXGRP).\n\n",
 			lp_servicename(talloc_tos(), s));
 	}
-	if (lp_printing(s) == PRINT_CUPS && *(lp_print_command(talloc_tos(), s)) != '\0') {
+	if (lp_printing(s) == PRINT_CUPS && *(lp_print_command(s)) != '\0') {
 		fprintf(stderr,
 			"Warning: Service %s defines a print command, but "
 			"parameter is ignored when using CUPS libraries.\n\n",
@@ -650,12 +650,55 @@ static void do_per_share_checks(int s)
 
 	struct poptOption long_options[] = {
 		POPT_AUTOHELP
-		{"suppress-prompt", 's', POPT_ARG_VAL, &silent_mode, 1, "Suppress prompt for enter"},
-		{"verbose", 'v', POPT_ARG_NONE, &show_defaults, 1, "Show default options too"},
-		{"skip-logic-checks", 'l', POPT_ARG_NONE, &skip_logic_checks, 1, "Skip the global checks"},
-		{"show-all-parameters", '\0', POPT_ARG_VAL, &show_all_parameters, True, "Show the parameters, type, possible values" },
-		{"parameter-name", '\0', POPT_ARG_STRING, &parameter_name, 0, "Limit testparm to a named parameter" },
-		{"section-name", '\0', POPT_ARG_STRING, &section_name, 0, "Limit testparm to a named section" },
+		{
+			.longName   = "suppress-prompt",
+			.shortName  = 's',
+			.argInfo    = POPT_ARG_VAL,
+			.arg        = &silent_mode,
+			.val        = 1,
+			.descrip    = "Suppress prompt for enter",
+		},
+		{
+			.longName   = "verbose",
+			.shortName  = 'v',
+			.argInfo    = POPT_ARG_NONE,
+			.arg        = &show_defaults,
+			.val        = 1,
+			.descrip    = "Show default options too",
+		},
+		{
+			.longName   = "skip-logic-checks",
+			.shortName  = 'l',
+			.argInfo    = POPT_ARG_NONE,
+			.arg        = &skip_logic_checks,
+			.val        = 1,
+			.descrip    = "Skip the global checks",
+		},
+		{
+			.longName   = "show-all-parameters",
+			.shortName  = '\0',
+			.argInfo    = POPT_ARG_VAL,
+			.arg        = &show_all_parameters,
+			.val        = True,
+			.descrip    = "Show the parameters, type, possible "
+				      "values",
+		},
+		{
+			.longName   = "parameter-name",
+			.shortName  = '\0',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = &parameter_name,
+			.val        = 0,
+			.descrip    = "Limit testparm to a named parameter",
+		},
+		{
+			.longName   = "section-name",
+			.shortName  = '\0',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = &section_name,
+			.val        = 0,
+			.descrip    = "Limit testparm to a named section",
+		},
 		POPT_COMMON_VERSION
 		POPT_COMMON_DEBUGLEVEL
 		POPT_COMMON_OPTION
@@ -666,11 +709,11 @@ static void do_per_share_checks(int s)
 
 	smb_init_locale();
 	/*
-	 * Set the default debug level to 2.
+	 * Set the default debug level to 1.
 	 * Allow it to be overridden by the command line,
 	 * not by smb.conf.
 	 */
-	lp_set_cmdline("log level", "2");
+	lp_set_cmdline("log level", "1");
 
 	pc = poptGetContext(NULL, argc, argv, long_options,
 			    POPT_CONTEXT_KEEP_FIRST);
@@ -699,8 +742,6 @@ static void do_per_share_checks(int s)
 		goto done;
 	}
 
-	cmdline_messaging_context(config_file);
-
 	fprintf(stderr,"Load smb config files from %s\n",config_file);
 
 	if (!lp_load_with_registry_shares(config_file)) {
