@@ -167,8 +167,10 @@ NTSTATUS _wbint_Sids2UnixIDs(struct pipes_struct *p,
 
 	dom = idmap_find_domain_with_sid(d->name.string, d->sid);
 	if (dom == NULL) {
+		struct dom_sid_buf buf;
 		DEBUG(10, ("idmap domain %s:%s not found\n",
-			   d->name.string, sid_string_dbg(d->sid)));
+			   d->name.string,
+			   dom_sid_str_buf(d->sid, &buf)));
 
 		for (i=0; i<num_ids; i++) {
 
@@ -202,6 +204,15 @@ NTSTATUS _wbint_Sids2UnixIDs(struct pipes_struct *p,
 
 	status = dom->methods->sids_to_unixids(dom, id_map_ptrs);
 
+	if (NT_STATUS_EQUAL(status, STATUS_SOME_UNMAPPED)) {
+		/*
+		 * This is okay. We need to transfer the mapped ones
+		 * up to our caller. The individual mappings carry the
+		 * information whether they are mapped or not.
+		 */
+		status = NT_STATUS_OK;
+	}
+
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(10, ("sids_to_unixids returned %s\n",
 			   nt_errstr(status)));
@@ -552,7 +563,7 @@ NTSTATUS _wbint_DsGetDcName(struct pipes_struct *p, struct wbint_DsGetDcName *r)
 	bool try_dsrgetdcname = false;
 
 	if (domain == NULL) {
-		return dsgetdcname(p->mem_ctx, server_messaging_context(),
+		return dsgetdcname(p->mem_ctx, global_messaging_context(),
 				   r->in.domain_name, r->in.domain_guid,
 				   r->in.site_name ? r->in.site_name : "",
 				   r->in.flags,
@@ -746,7 +757,7 @@ again:
 NTSTATUS _wbint_ChangeMachineAccount(struct pipes_struct *p,
 				     struct wbint_ChangeMachineAccount *r)
 {
-	struct messaging_context *msg_ctx = server_messaging_context();
+	struct messaging_context *msg_ctx = global_messaging_context();
 	struct winbindd_domain *domain;
 	NTSTATUS status;
 	struct rpc_pipe_client *netlogon_pipe = NULL;
@@ -990,8 +1001,13 @@ NTSTATUS _winbind_SamLogon(struct pipes_struct *p,
 				       identity_info->account_name.string,
 				       identity_info->domain_name.string,
 				       identity_info->workstation.string,
+				       identity_info->logon_id,
+				       "SamLogon",
+				       0,
 				       challenge,
 				       lm_response, nt_response,
+				       p->remote_address,
+				       p->local_address,
 				       &r->out.authoritative,
 				       true, /* skip_sam */
 				       &flags,
@@ -1501,7 +1517,7 @@ static WERROR _winbind_LogonControl_CHANGE_PASSWORD(struct pipes_struct *p,
 			     struct winbindd_domain *domain,
 			     struct winbind_LogonControl *r)
 {
-	struct messaging_context *msg_ctx = server_messaging_context();
+	struct messaging_context *msg_ctx = global_messaging_context();
 	NTSTATUS status;
 	struct rpc_pipe_client *netlogon_pipe = NULL;
 	struct netlogon_creds_cli_context *netlogon_creds_ctx = NULL;
