@@ -126,6 +126,7 @@ static NTSTATUS add_trusted_domain(const char *domain_name,
 	const char **ignored_domains = NULL;
 	const char **dom = NULL;
 	int role = lp_server_role();
+	struct dom_sid_buf buf;
 
 	if (is_null_sid(sid)) {
 		DBG_ERR("Got null SID for domain [%s]\n", domain_name);
@@ -169,7 +170,8 @@ static NTSTATUS add_trusted_domain(const char *domain_name,
 		if (check_domain != NULL) {
 			DBG_ERR("SID [%s] already used by domain [%s], "
 				"expected [%s]\n",
-				sid_string_dbg(sid), check_domain->name,
+				dom_sid_str_buf(sid, &buf),
+				check_domain->name,
 				domain->name);
 			return NT_STATUS_INVALID_PARAMETER;
 		}
@@ -292,7 +294,7 @@ static NTSTATUS add_trusted_domain(const char *domain_name,
 
 	DBG_NOTICE("Added domain [%s] [%s] [%s]\n",
 		   domain->name, domain->alt_name,
-		   sid_string_dbg(&domain->sid));
+		   dom_sid_str_buf(&domain->sid, &buf));
 
 	*_d = domain;
 	return NT_STATUS_OK;
@@ -400,7 +402,7 @@ static void add_trusted_domains( struct winbindd_domain *domain )
 	state->request.length = sizeof(state->request);
 	state->request.cmd = WINBINDD_LIST_TRUSTDOM;
 
-	req = wb_domain_request_send(state, server_event_context(),
+	req = wb_domain_request_send(state, global_event_context(),
 				     domain, &state->request);
 	if (req == NULL) {
 		DEBUG(1, ("wb_domain_request_send failed\n"));
@@ -459,6 +461,7 @@ static void trustdom_list_done(struct tevent_req *req)
 		uint32_t trust_type;
 		uint32_t trust_attribs;
 		uint32_t trust_flags;
+		int error = 0;
 
 		DBG_DEBUG("parsing response line '%s'\n", p);
 
@@ -504,7 +507,15 @@ static void trustdom_list_done(struct tevent_req *req)
 			break;
 		}
 
-		trust_flags = (uint32_t)strtoul(q, NULL, 10);
+		trust_flags = (uint32_t)smb_strtoul(q,
+						    NULL,
+						    10,
+						    &error,
+						    SMB_STR_STANDARD);
+		if (error != 0) {
+			DBG_ERR("Failed to convert trust_flags\n");
+			break;
+		}
 
 		q = strtok(NULL, "\\");
 		if (q == NULL) {
@@ -512,7 +523,15 @@ static void trustdom_list_done(struct tevent_req *req)
 			break;
 		}
 
-		trust_type = (uint32_t)strtoul(q, NULL, 10);
+		trust_type = (uint32_t)smb_strtoul(q,
+						   NULL,
+						   10,
+						   &error,
+						   SMB_STR_STANDARD);
+		if (error != 0) {
+			DBG_ERR("Failed to convert trust_type\n");
+			break;
+		}
 
 		q = strtok(NULL, "\n");
 		if (q == NULL) {
@@ -520,7 +539,15 @@ static void trustdom_list_done(struct tevent_req *req)
 			break;
 		}
 
-		trust_attribs = (uint32_t)strtoul(q, NULL, 10);
+		trust_attribs = (uint32_t)smb_strtoul(q,
+						      NULL,
+						      10,
+						      &error,
+						      SMB_STR_STANDARD);
+		if (error != 0) {
+			DBG_ERR("Failed to convert trust_attribs\n");
+			break;
+		}
 
 		if (!within_forest) {
 			trust_flags &= ~NETR_TRUST_FLAG_IN_FOREST;
@@ -696,7 +723,7 @@ static void rescan_forest_trusts( void )
 
 		if ( (flags & NETR_TRUST_FLAG_INBOUND) &&
 		     (type == LSA_TRUST_TYPE_UPLEVEL) &&
-		     (attribs == LSA_TRUST_ATTRIBUTE_FOREST_TRANSITIVE) )
+		     (attribs & LSA_TRUST_ATTRIBUTE_FOREST_TRANSITIVE) )
 		{
 			/* add the trusted domain if we don't know
 			   about it */
@@ -1455,7 +1482,9 @@ struct winbindd_domain *find_default_route_domain(void)
 
 struct winbindd_domain *find_lookup_domain_from_sid(const struct dom_sid *sid)
 {
-	DBG_DEBUG("SID [%s]\n", sid_string_dbg(sid));
+	struct dom_sid_buf buf;
+
+	DBG_DEBUG("SID [%s]\n", dom_sid_str_buf(sid, &buf));
 
 	/*
 	 * SIDs in the S-1-22-{1,2} domain and well-known SIDs should be handled
@@ -1664,6 +1693,10 @@ char *fill_domain_username_talloc(TALLOC_CTX *mem_ctx,
 		can_assume = false;
 	}
 
+	if (user == NULL) {
+		return NULL;
+	}
+
 	tmp_user = talloc_strdup(mem_ctx, user);
 	if (!strlower_m(tmp_user)) {
 		TALLOC_FREE(tmp_user);
@@ -2138,6 +2171,7 @@ bool parse_xidlist(TALLOC_CTX *mem_ctx, const char *xidstr,
 		struct unixid xid;
 		unsigned long long id;
 		char *endp;
+		int error = 0;
 
 		switch (p[0]) {
 		case 'U':
@@ -2152,8 +2186,8 @@ bool parse_xidlist(TALLOC_CTX *mem_ctx, const char *xidstr,
 
 		p += 1;
 
-		id = strtoull(p, &endp, 10);
-		if ((id == ULLONG_MAX) && (errno == ERANGE)) {
+		id = smb_strtoull(p, &endp, 10, &error, SMB_STR_STANDARD);
+		if (error != 0) {
 			goto fail;
 		}
 		if (*endp != '\n') {
