@@ -41,11 +41,25 @@
 #endif
 
 /*Workaround for heimdal define vs samba define*/
-#ifdef HAVE_LIBINTL_H
+#if defined(HAVE_LIBINTL_H) && defined(HAVE_BINDTEXTDOMAIN) &&\
+    defined(HAVE_TEXTDOMAIN)
 #define LIBINTL
 #endif
 
 /* heimdal now wants some atomic ops - ask for the non-atomic ones for Samba */
 #define HEIM_BASE_NON_ATOMIC 1
 
+/* lib/replace provides an XSI Compatable strerror_r so use that */
+#define STRERROR_R_PROTO_COMPATIBLE
+
+/* lib/replace provides a closefrom */
+#ifndef HAVE_CLOSEFROM
+#define HAVE_CLOSEFROM 1
+#endif
+
+/* lib/replace provides a getprogname */
+#ifndef HAVE_GETPROGNAME
+#define HAVE_GETPROGNAME 1
+#endif
+
 #endif
