@@ -461,6 +461,7 @@ enum smb_fileinfo_level {
 		     RAW_FILEINFO_COMPRESSION_INFORMATION    = SMB_QFILEINFO_COMPRESSION_INFORMATION,
 		     RAW_FILEINFO_NETWORK_OPEN_INFORMATION   = SMB_QFILEINFO_NETWORK_OPEN_INFORMATION,
 		     RAW_FILEINFO_ATTRIBUTE_TAG_INFORMATION  = SMB_QFILEINFO_ATTRIBUTE_TAG_INFORMATION,
+		     RAW_FILEINFO_NORMALIZED_NAME_INFORMATION= SMB_QFILEINFO_NORMALIZED_NAME_INFORMATION,
 		     /* SMB2 specific levels */
 		     RAW_FILEINFO_SMB2_ALL_EAS               = 0x0f01,
 		     RAW_FILEINFO_SMB2_ALL_INFORMATION       = 0x1201
@@ -643,6 +644,8 @@ union smb_fileinfo {
 	} ea_info;
 
 	/* RAW_FILEINFO_NAME_INFO and RAW_FILEINFO_NAME_INFORMATION interfaces */
+	/* RAW_FILEINFO_ALT_NAME_INFO and RAW_FILEINFO_ALT_NAME_INFORMATION interfaces */
+	/* RAW_FILEINFO_NORMALIZED_NAME_INFORMATION interface */
 	struct {
 		enum smb_fileinfo_level level;
 		struct {
@@ -651,7 +654,7 @@ union smb_fileinfo {
 		struct {
 			struct smb_wire_string fname;
 		} out;
-	} name_info;
+	} name_info, alt_name_info, normalized_name_info;
 
 	/* RAW_FILEINFO_ALL_INFO and RAW_FILEINFO_ALL_INFORMATION interfaces */
 	struct {
@@ -704,17 +707,6 @@ union smb_fileinfo {
 		} out;
 	} all_info2;
 
-	/* RAW_FILEINFO_ALT_NAME_INFO and RAW_FILEINFO_ALT_NAME_INFORMATION interfaces */
-	struct {
-		enum smb_fileinfo_level level;
-		struct {
-			union smb_handle_or_path file;
-		} in;
-		struct {
-			struct smb_wire_string fname;
-		} out;
-	} alt_name_info;
-
 	/* RAW_FILEINFO_STREAM_INFO and RAW_FILEINFO_STREAM_INFORMATION interfaces */
 	struct {
 		enum smb_fileinfo_level level;
@@ -2086,12 +2078,7 @@ union smb_lock {
 			uint16_t lock_count;
 			uint32_t lock_sequence;
 			/* struct smb2_handle handle; */
-			struct smb2_lock_element {
-				uint64_t offset;
-				uint64_t length;
-				uint32_t flags;
-				uint32_t reserved;
-			} *locks;
+			struct smb2_lock_element *locks;
 		} in;
 		struct {
 			/* static body buffer 4 (0x04) bytes */
@@ -2291,16 +2278,17 @@ union smb_ioctl {
 
 			/* static body buffer 56 (0x38) bytes */
 			/* uint16_t buffer_code;  0x39 = 0x38 + 1 */
-			uint16_t _pad;
+			uint16_t reserved;
 			uint32_t function;
 			/*struct smb2_handle handle;*/
 			/* uint32_t out_ofs; */
 			/* uint32_t out_size; */
-			uint32_t unknown2;
+			uint32_t max_input_response;
 			/* uint32_t in_ofs; */
 			/* uint32_t in_size; */
-			uint32_t max_response_size;
-			uint64_t flags;
+			uint32_t max_output_response;
+			uint32_t flags;
+			uint32_t reserved2;
 
 			/* dynamic body */
 			DATA_BLOB out;
@@ -2311,15 +2299,15 @@ union smb_ioctl {
 
 			/* static body buffer 48 (0x30) bytes */
 			/* uint16_t buffer_code;  0x31 = 0x30 + 1 */
-			uint16_t _pad;
+			uint16_t reserved;
 			uint32_t function;
 			/* struct smb2_handle handle; */
 			/* uint32_t in_ofs; */
 			/* uint32_t in_size; */
 			/* uint32_t out_ofs; */
 			/* uint32_t out_size; */
-			uint32_t unknown2;
-			uint32_t unknown3;
+			uint32_t flags;
+			uint32_t reserved2;
 
 			/* dynamic body */
 			DATA_BLOB in;
