@@ -45,16 +45,13 @@
 #include "libds/common/roles.h"
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
-#if defined(HAVE_GCRYPT_H) && !defined(HAVE_GNUTLS3)
-#include <gcrypt.h>
-#endif
 
-#define DCESRV_INTERFACE_BACKUPKEY_BIND(call, iface) \
-	dcesrv_interface_backupkey_bind(call, iface)
-static NTSTATUS dcesrv_interface_backupkey_bind(struct dcesrv_call_state *dce_call,
+#define DCESRV_INTERFACE_BACKUPKEY_BIND(context, iface) \
+	dcesrv_interface_backupkey_bind(context, iface)
+static NTSTATUS dcesrv_interface_backupkey_bind(struct dcesrv_connection_context *context,
 						const struct dcesrv_interface *iface)
 {
-	return dcesrv_interface_bind_require_privacy(dce_call, iface);
+	return dcesrv_interface_bind_require_privacy(context, iface);
 }
 
 static const unsigned rsa_with_var_num[] = { 1, 2, 840, 113549, 1, 1, 1 };
@@ -588,6 +585,8 @@ static WERROR bkrp_client_wrap_decrypt_data(struct dcesrv_call_state *dce_call,
 					    struct bkrp_BackupKey *r,
 					    struct ldb_context *ldb_ctx)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct bkrp_client_side_wrapped uncrypt_request;
 	DATA_BLOB blob;
 	enum ndr_err_code ndr_err;
@@ -723,7 +722,7 @@ static WERROR bkrp_client_wrap_decrypt_data(struct dcesrv_call_state *dce_call,
 							   uncrypted_secretv2.payload_key,
 							   uncrypt_request.access_check,
 							   uncrypt_request.access_check_len,
-							   dce_call->conn->auth_state.session_info);
+							   session_info);
 			if (!W_ERROR_IS_OK(werr)) {
 				return werr;
 			}
@@ -767,7 +766,7 @@ static WERROR bkrp_client_wrap_decrypt_data(struct dcesrv_call_state *dce_call,
 							   uncrypted_secretv3.payload_key,
 							   uncrypt_request.access_check,
 							   uncrypt_request.access_check_len,
-							   dce_call->conn->auth_state.session_info);
+							   session_info);
 			if (!W_ERROR_IS_OK(werr)) {
 				return werr;
 			}
@@ -835,10 +834,7 @@ static WERROR create_heimdal_rsa_key(TALLOC_CTX *ctx, hx509_context *hctx,
 		DBG_ERR("TLS error: %s\n", gnutls_strerror(ret));
 		return WERR_INTERNAL_ERROR;
 	}
-#if defined(HAVE_GCRYPT_H) && !defined(HAVE_GNUTLS3)
-	DEBUG(3,("Enabling QUICK mode in gcrypt\n"));
-	gcry_control(GCRYCTL_ENABLE_QUICK_RANDOM, 0);
-#endif
+
 	ret = gnutls_x509_privkey_init(&gtls_key);
 	if (ret != 0) {
 		gnutls_global_deinit();
@@ -1497,6 +1493,8 @@ static WERROR bkrp_do_retrieve_default_server_wrap_key(TALLOC_CTX *mem_ctx,
 static WERROR bkrp_server_wrap_decrypt_data(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx,
 					    struct bkrp_BackupKey *r ,struct ldb_context *ldb_ctx)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	WERROR werr;
 	struct bkrp_server_side_wrapped decrypt_request;
 	DATA_BLOB sid_blob, encrypted_blob, symkey_blob;
@@ -1600,7 +1598,7 @@ static WERROR bkrp_server_wrap_decrypt_data(struct dcesrv_call_state *dce_call,
 		return WERR_INVALID_ACCESS;
 	}
 
-	caller_sid = &dce_call->conn->auth_state.session_info->security_token->sids[PRIMARY_USER_SID_INDEX];
+	caller_sid = &session_info->security_token->sids[PRIMARY_USER_SID_INDEX];
 
 	if (!dom_sid_equal(&rc4payload.sid, caller_sid)) {
 		return WERR_INVALID_ACCESS;
@@ -1653,6 +1651,8 @@ static WERROR bkrp_generic_decrypt_data(struct dcesrv_call_state *dce_call, TALL
 static WERROR bkrp_server_wrap_encrypt_data(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx,
 					    struct bkrp_BackupKey *r ,struct ldb_context *ldb_ctx)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	DATA_BLOB sid_blob, encrypted_blob, symkey_blob, server_wrapped_blob;
 	WERROR werr;
 	struct dom_sid *caller_sid;
@@ -1699,7 +1699,7 @@ static WERROR bkrp_server_wrap_encrypt_data(struct dcesrv_call_state *dce_call,
 		}
 	}
 
-	caller_sid = &dce_call->conn->auth_state.session_info->security_token->sids[PRIMARY_USER_SID_INDEX];
+	caller_sid = &session_info->security_token->sids[PRIMARY_USER_SID_INDEX];
 
 	dump_data_pw("server_key: \n", server_key.key, sizeof(server_key.key));
 
