@@ -116,9 +116,10 @@ struct dcesrv_handle *dcesrv_handle_lookup(struct dcesrv_call_state *call,
 				return NULL;
 			}
 			if (!dom_sid_equal(h->sid, sid)) {
-				DEBUG(0,(__location__ ": Attempt to use invalid sid %s - %s\n",
-					 dom_sid_string(context, h->sid),
-					 dom_sid_string(context, sid)));
+				struct dom_sid_buf buf1, buf2;
+				DBG_ERR("Attempt to use invalid sid %s - %s\n",
+					dom_sid_str_buf(h->sid, &buf1),
+					dom_sid_str_buf(sid, &buf2));
 				return NULL;
 			}
 			if (call->auth_state->auth_level < h->min_auth_level) {
