@@ -92,7 +92,7 @@ static void single_accept_connection(struct tevent_context *ev,
 static void single_new_task(struct tevent_context *ev,
 			    struct loadparm_context *lp_ctx,
 			    const char *service_name,
-			    void (*new_task)(struct tevent_context *,
+			    struct task_server *(*new_task)(struct tevent_context *,
 				             struct loadparm_context *,
 					     struct server_id, void *, void *),
 			    void *private_data,
@@ -102,7 +102,7 @@ static void single_new_task(struct tevent_context *ev,
 	pid_t pid = getpid();
 	/* start our taskids at MAX_INT32, the first 2^31 tasks are is reserved for fd numbers */
 	static uint32_t taskid = INT32_MAX;
-       
+	struct task_server *task = NULL;
 	/*
 	 * We use the PID so we cannot collide in with cluster ids
 	 * generated in other single mode tasks, and, and won't
@@ -112,19 +112,35 @@ static void single_new_task(struct tevent_context *ev,
 	 * Using the pid unaltered makes debugging of which process
 	 * owns the messaging socket easier.
 	 */
-	new_task(ev, lp_ctx, cluster_id(pid, taskid++), private_data, NULL);
+	task = new_task(ev, lp_ctx, cluster_id(pid, taskid++), private_data, NULL);
+	if (task != NULL && service_details->post_fork != NULL) {
+		struct process_details pd = initial_process_details;
+		service_details->post_fork(task, &pd);
+	}
 }
 
-
-/* called when a task goes down */
-static void single_terminate(struct tevent_context *ev,
-			     struct loadparm_context *lp_ctx,
-			     const char *reason,
-			     void *process_context)
+/*
+ * Called when a task goes down
+ */
+static void single_terminate_task(struct tevent_context *ev,
+				  struct loadparm_context *lp_ctx,
+				  const char *reason,
+				  bool fatal,
+				  void *process_context)
 {
 	DBG_NOTICE("single_terminate: reason[%s]\n",reason);
 }
 
+/*
+ * Called when a connection has ended
+ */
+static void single_terminate_connection(struct tevent_context *ev,
+					struct loadparm_context *lp_ctx,
+					const char *reason,
+					void *process_context)
+{
+}
+
 /* called to set a title of a task or connection */
 static void single_set_title(struct tevent_context *ev, const char *title) 
 {
@@ -133,9 +149,10 @@ static void single_set_title(struct tevent_context *ev, const char *title)
 const struct model_ops single_ops = {
 	.name			= "single",
 	.model_init		= single_model_init,
-	.new_task               = single_new_task,
+	.new_task		= single_new_task,
 	.accept_connection	= single_accept_connection,
-	.terminate              = single_terminate,
+	.terminate_task		= single_terminate_task,
+	.terminate_connection	= single_terminate_connection,
 	.set_title		= single_set_title,
 };
 
