@@ -91,10 +91,6 @@ NTSTATUS torture_local_init(TALLOC_CTX *ctx)
 	
 	torture_suite_add_simple_test(suite, 
 				      "crypto.md4", torture_local_crypto_md4);
-	torture_suite_add_simple_test(suite, "crypto.md5", 
-				      torture_local_crypto_md5);
-	torture_suite_add_simple_test(suite, "crypto.hmacmd5", 
-				      torture_local_crypto_hmacmd5);
 	torture_suite_add_simple_test(suite, "crypto.aes_cmac_128",
 				      torture_local_crypto_aes_cmac_128);
 	torture_suite_add_simple_test(suite, "crypto.aes_ccm_128",
