@@ -54,7 +54,7 @@ static void winbindd_done(struct tevent_req *subreq)
 /*
   startup a copy of winbindd as a child daemon
 */
-static void winbindd_task_init(struct task_server *task)
+static NTSTATUS winbindd_task_init(struct task_server *task)
 {
 	struct tevent_req *subreq;
 	const char *winbindd_path;
@@ -76,12 +76,13 @@ static void winbindd_task_init(struct task_server *task)
 	if (subreq == NULL) {
 		DEBUG(0, ("Failed to start winbindd as child daemon\n"));
 		task_server_terminate(task, "Failed to startup winbindd task", true);
-		return;
+		return NT_STATUS_UNSUCCESSFUL;
 	}
 
 	tevent_req_set_callback(subreq, winbindd_done, task);
 
 	DEBUG(5,("Started winbindd as a child daemon\n"));
+	return NT_STATUS_OK;
 }
 
 /* called at winbindd startup - register ourselves as a server service */
@@ -89,16 +90,16 @@ NTSTATUS server_service_winbindd_init(TALLOC_CTX *);
 
 NTSTATUS server_service_winbindd_init(TALLOC_CTX *ctx)
 {
-	struct service_details details = {
+	static const struct service_details details = {
 		.inhibit_fork_on_accept = true,
 		.inhibit_pre_fork = true,
+		.task_init = winbindd_task_init,
+		.post_fork = NULL
 	};
 
-	NTSTATUS status = register_server_service(ctx, "winbindd",
-						  winbindd_task_init, &details);
+	NTSTATUS status = register_server_service(ctx, "winbindd", &details);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
-	return register_server_service(ctx, "winbind", winbindd_task_init,
-				       &details);
+	return register_server_service(ctx, "winbind", &details);
 }
